%{
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "parser.tab.h"

/*#define DEBUG*/
%}
D	[0-9]
E	[EeDd][+-]?{D}+
SIGN    ([+-]?)


%%



^\#[^\n]*\n                       {     
/* comment line */
#ifdef DEBUG 
	printf("COMMENT LINE:");ECHO;
#endif    
	return COMMENT;
}

\#[^\n]*                       {     

#ifdef DEBUG 
	printf("COMMENT:");ECHO;
#endif    
}
                               
^[ \t\r]*\n                      {
/* blank line */
#ifdef DEBUG 
	ECHO;
#endif    
	return BLANK;
}                     

[ \t\r\n]                      {
/* use to drop blankspace/tab/cr */
#ifdef DEBUG 
	ECHO;
#endif    
}                                  


{SIGN}{D}+ |
{SIGN}{D}+"."{D}*({E})?	|
{SIGN}{D}*"."{D}+({E})?	|
{SIGN}{D}+({E})	               {
sscanf(yytext, "%lf", &yylval.dval);
#ifdef DEBUG 
	printf("double_value: %lf",yylval.dval);
#endif            
        return REAL;
}


\=                              {        
#ifdef DEBUG 
	ECHO;
#endif        
        return '=';
}  

     
                                                                
                                
set           |
vsource       |
isource       |
boundary      |
METHOD        | 
SOLVE         |
EXTRACT       |
IMPORT        |
ATTACH        |
REFINE        |
PLOT                            { 
	      
#ifdef DEBUG 
	printf("KEYWORD:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return KEYWORD;
}  


     

MeshFile     |
ModelFile    |
Carrier      |
DeviceDepth  |
LatticeTemp  |
MeshScale			{ 
	      
#ifdef DEBUG 
	printf("PARAMETER:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return PARAMETER;
}       



Type        |
ID          			{ 
	      
#ifdef DEBUG 
	printf("PARAMETER:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return PARAMETER;
}       



Tdelay      |
Vconst      |
Vamp        |
Iconst      |
Iamp        |
Freq        |
TRC	    |
TFD         |
TFC         |
Vlo         |
Vhi         |
Ilo         |
Ihi         |	
Tr          |
Tf          |
Pw          |
Pr	     			{ 
	      
#ifdef DEBUG 
	printf("PARAMETER:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return PARAMETER;
}       



Res            |
Cap            |
Ind            |
Thick          |
QF             |
Kapa           |
WorkFunction   |
Vbarrier       			{ 
	      
#ifdef DEBUG 
	printf("PARAMETER:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return PARAMETER;
}        

Variable       |
Measure        |
Dispersion                      { 
	      
#ifdef DEBUG 
	printf("PARAMETER:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return PARAMETER;
}        

Scheme         |
SNES           |
FluxFunc       |
Reconstruct    |
IVRecord       |
IVFile         | 
CoreFile       |
AscFile        |
Electrode      |
VScan          |
VStart         |
VStep          |
VStop          |
VApp           |
TStart         |
TStep          |
CFL            |
TStop          |
AUTOSAVE	                 { 
	      
#ifdef DEBUG 
	printf("PARAMETER:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return PARAMETER;
}        
   
Resolution     |
PSFile         |
Style          |
AzAngle        |
ElAngle				{ 
	      
#ifdef DEBUG 
	printf("PARAMETER:");ECHO;
#endif            
        strcpy(yylval.sval, yytext);
        return PARAMETER;
}        
   


[a-zA-Z0-9\_][a-zA-Z0-9\.\_]*      {        
#ifdef DEBUG 
	printf("STRING:%s",yytext);
#endif            
        strcpy(yylval.sval,yytext); 
        return STRING;
}


\"[a-zA-Z0-9\.\_\ \-]*\"           {        
yytext[yyleng-1]='\0';
#ifdef DEBUG 
	printf("STRING:%s",&yytext[1]);
#endif            
        strcpy(yylval.sval, &yytext[1]); 
        return STRING;
}



%%
int yywrap()
{
#ifdef DEBUG 
	printf("\nline %d\n",yylineno);
#endif            
        return 1;
  
}   


