//i think each class should have max independent
//DABC keeps independent --- it only need the name of cgns file 
//and CmdParseBuf list for initial.  

#ifndef _bc_h_
#define _bc_h_
#include "mesh.h"
#include "vsource.h"
#include "cmdbuf.h"

//boundary type
const int NeumannBoundary     = 1001;
const int OhmicContract       = 1002;
const int SchottkyContract    = 1003;
const int InsulatorInterface  = 1004;
const int InsulatorContract   = 1005;
const int GateContract        = 1006;


class BDC
{
  public:
	int         BCType;                  //support 5 types of Contract and insulator interface now
	Segment*    psegment;                //the pointer to the correlative segment
	double      R,C,L;                   //extern electrical parameters for electrode
	double      WorkFunction;            //for Gate contract	
	double      Vbarrier;		     //for Schottky contract
	double      Thick;                   //for simple SiSiO2 interface  
	double      Kapa;		     //heat transfer
	double      QF;                      //InsulatorInterface fixed charge
	double      V;                       //the application voltage 
	double	    P,I;		     //the potential and current of this electrode
	double      Ic;	 	   	     //the current of electrode capacitance 	
	vector<VSource*> vsrc;		     //vsource point array belong to this bc
};


class DABC
{
  protected:
    	
    	vector<BDC> bc_array;                   // dynamic array for bc
        double mesh_scale;
  protected:
  	//these functions parse CmdParseBuf 
 	int SetBCNeumannBoundary(list<CmdParseBuf>::iterator);  
	int SetBCOhmicContract(list<CmdParseBuf>::iterator);
	int SetBCInsulatorInterface(list<CmdParseBuf>::iterator);  
	int SetBCInsulatorContract(list<CmdParseBuf>::iterator);  
	int SetBCSchottkyContract(list<CmdParseBuf>::iterator);
	int SetBCGateContract(list<CmdParseBuf>::iterator);
    
  public:
    	int  size()	                    { return bc_array.size(); }
    	void clear()	                    { bc_array.clear(); }  	
    	BDC & operator[]  (int bc_index)    { return bc_array[bc_index];  }  
    	
    	int Get_bc_index  (char *bc_name)
    	{
		for(int i=0;i<bc_array.size();i++)
	  	if(!strcmp(bc_array[i].psegment->label,bc_name))
			return i;
		return -1;	
	}    
	
    	void Set_Vapp(double t)
    	{
    		for(int i=0;i<bc_array.size();i++)
    		{
    			bc_array[i].V = 0;
 	        	for(int j=0;j<bc_array[i].vsrc.size();j++)
 	        		bc_array[i].V+=bc_array[i].vsrc[j]->vapp(t);
 	        }		
 	}   
 	void Clear_Vapp()
    	{
    	        for(int i=0;i<bc_array.size();i++)
 	              	bc_array[i].V = 0;
  	}        	 	     	 	
  public:
   	DABC() {}
   	int InitBC(int,Segment**,list<CmdParseBuf>&,double);	                                                     
    	~DABC ()         {  }
};

#endif
