// the main mesh structure are defined here
// 2005.03.16 by gdiso

#ifndef _mesh_h_
#define _mesh_h_

#include <iostream>
#include <string>
#include <vector>
#include <list>
#include <stack>
#include <queue>
#include <algorithm>

using namespace std;



class  	Node				// node
{
  public:
 	 double      	  x;			// x coordinate
  	 double      	  y;			// y coordinate
  	 int              index;                // the index in the zone
  	 int         	  bc_index;		// the index to boundary of the node
  	 int              zone_index;           // the zone index;
};

class Segment 				// boundary segment
{
  public:
	int       bc_index;		// boundary label
	int       zone_index;           // this segment belongs to which zone
	int       interface;		// -1 means a boundary segment,>=0 is the interface index
	char      label[32];
	int       edge_num;
	vector<int>  edge_array;	// segment edges,each edge have 2 nodes
	vector<int>  node_array;        // segment nodes.
	void      set_node_array()      //convert edge_array to node_array here
	{
	  node_array.clear();
	  for(int i=0; i<edge_array.size();i++)
	    if(!count(node_array.begin(),node_array.end(),edge_array[i]))
			node_array.push_back(edge_array[i]);
	}

};

class Tri 			        // triangle or rectangle
{
  public:
	int 	zone_index;             // zone index
	int 	A,B,C,D;		// three or four nodes
	double  area;
};

class VEdge 	                	//Voronoi edge
{
  public:
  	int 	   cell1;		//2 cell
 	int 	   cell2;
 	int        c1_nb;		//cell2 index in cell 1's nb_array
 	int        c2_nb;		//cell1 index in cell 2's nb_array
 	double     x1,y1;
 	double     x2,y2;
 	double     length;
};

class  MirrorNode
{
  public:
	double   x,y;
	int      index;                  //the global index num of mirror node
	int      nbindex;                //the index num of mirror node in cell's neighbor
	double   elen;
	double   ilen;
 	double   angle;                  //vector angle to control volume face.
 	double   nx,ny;                  //vector norm to boundary
};

class VoronoiCell                        //VoronoiCell
{
  public:
 	double      x,y;                   //cell centre
 	int         nb_num;
 	int         *nb_array;             //it's neighbor nodes
 	int         *inb_array;            //inverse index of neighbor nodes
 	double      *elen;
 	double      *ilen;
 	double      *angle;
 	double      area;
 	int         *celledge;
 	int         bc_index;		   //the index to boundary of the node
 	int	    mirror_num;            //for build ghost cell
  	MirrorNode  *mirror;
  	double      garea;                 //the area of ghost cell
  	VoronoiCell();
  	VoronoiCell(const VoronoiCell&);
  	~VoronoiCell();

};

class Interface
{
  public:
	char conn_name[32];
	char bc_name[32];
	char zone1_name[32];
	char zone2_name[32];
	int zone1;
	int zone2;
	int flag;
	int node_num;
	vector<int> index_array1;
	vector<int> index_array2;
	vector<int> gindex_array;
	void  clear()
	{ node_num = 0;
	  index_array1.clear();
	  index_array2.clear();
	  gindex_array.clear();
	}
};


class DAVCELL
{ private:
    	int cell_num;	      // total number of elements
    	VoronoiCell *cell_array;  // dynamic array
  public:
    	int size()	            		{ return cell_num;    }
    	VoronoiCell *GetPointer (int i)  	{ return cell_array + i; }
    	const VoronoiCell & operator[]  (int i) { return cell_array[i];  }
  public:
     	void Init(int c)                	{ cell_num = c;  cell_array = new VoronoiCell[c]; }
        void clear()				{ delete [] cell_array; cell_num = 0; cell_array = 0;}
     	DAVCELL ()		             	{ cell_num = 0; cell_array = 0; }
    	~DAVCELL ()		             	{ delete [] cell_array; }
};

class ZONE
{
  public:
	int         zone_index;                 //zone array index
	char        zonename[32];               //cgns zone name
	char        zonelabel[32];              //optional description
	int         offset;                     //the offset of the node index in global index
	int         zone_conn_num;              //the zone connect number
	vector<string>   donor_zone_name;	//the array of dornor zone name
	vector<int>      donor_zone_array;       //the array of dornor zone index

	double      field_area_vcell;           //field area summed  by vcell !may have problem
	double      field_area_tcell;           //field area summed  by tri cell
	vector<Node>     danode;
	vector<Tri>      datri;
	vector<Segment>  dasegment;		//used for mesh refine
	vector<VEdge>    davedge;
	DAVCELL     	 davcell;

  public:
	int 	    setup_voronoicell();
	int 	    build_voronoicell_data();
	int         build_ghost_cell();
	void        ghost_cell_debug();
};


class ZoneInterface
{
  public:
  	vector<Interface> interface;

  public:
  	int  Find(int z1,int z2)
  	{ if(interface.size()==0) return -1;
  	  for(int i=0;i<interface.size();i++)
  	  {
  	  	if(interface[i].zone1==z1&&interface[i].zone2==z2)
  		    return i;
  		else if(interface[i].zone1==z2&&interface[i].zone2==z1)
  		    return i;
  	  }
  	  return -1;
  	}
  	int  Find(char* z1,char* z2)
  	{
  	  if(interface.size()==0) return -1;
  	  for(int i=0;i<interface.size();i++)
  	  {
  	    if(!strcmp(interface[i].zone1_name,z1)&&!strcmp(interface[i].zone2_name,z2))
  		 return i;
  	    else if(!strcmp(interface[i].zone1_name,z2)&&!strcmp(interface[i].zone2_name,z1))
  	  	 return i;
  	  }
  	  return -1;
  	}
  	int Find_donor(int z1,int n,int &z2,int &dn)
  	{
  	  for(int i=0;i<interface.size();i++)
  	  {
  	  	if(interface[i].zone1==z1)
  	  	{
  		    z2 =  interface[i].zone2;
  		    for(int j=0;j<interface[i].index_array1.size();j++)
  		       if(interface[i].index_array1[j]==n)
  		       {
  		       	 dn = interface[i].index_array2[j];
  		         return 0;
  		       }
  		}
  		if(interface[i].zone2==z1)
  		{
  		    z2 =  interface[i].zone1;
  		    for(int j=0;j<interface[i].index_array2.size();j++)
  		       if(interface[i].index_array2[j]==n)
  		       {
  		       	 dn = interface[i].index_array1[j];
  		         return 0;
  		       }
  		 }
  	  }
  	  return -1;
  	}
  public:
   	Interface & operator[](int i)	    { return interface[i];}
   	int   size()			    { return interface.size(); }
   	void  Clear_flag_all()              { for(int i=0;i<interface.size();i++) interface[i].flag = 0;}
  	void  Set_flag_all()                { for(int i=0;i<interface.size();i++) interface[i].flag = 1;}
  	void  Clear_flag(int z1,int z2)	    { interface[Find(z1,z2)].flag = 0;}
  	void  Set_flag(int z1,int z2)       { interface[Find(z1,z2)].flag = 1;}
  	int   Flag(int z1,int z2)    	    { return interface[Find(z1,z2)].flag; }

  	int   IsBelong(int inf,int z)
  	{
  	   if(interface[inf].zone1 == z) return 1;
  	   else if(interface[inf].zone2 == z) return 1;
  	   return 0;
    	}

    	vector<int> * Get_node_array(int inf,int z)
  	{
  	   if(interface[inf].zone1 == z) return &interface[inf].index_array1;
  	   else if(interface[inf].zone2 == z) return &interface[inf].index_array2;
	   return (vector<int> *)0;
    	}

};

class MESH
{
  protected:

	char        basename[32];               //cgns base name
	double      mesh_scale;                 //this value will be mul to coordinate
	int         bc_num;                     //total bc num  summed by each zone num
	vector<Node>    gnode;			//these are global mesh structure
	vector<Tri>     gtri;
	vector<Segment> gsegment;		//used for mesh refine

	vector<Node>    gnode_old;		//refined mesh
	vector<Tri>     gtri_old;
	vector<Segment> gsegment_old;

	int         zone_num;
	ZONE        *zone;			//for compute
	ZoneInterface zinterface;

  private:
        //variables and functions for drawing mesh
	double 	    xMin;			// minimum value of x coordinate
	double      xMax;		        // maximum value of x coordinate
	double      yMin;			// minimum value of y coordinate
	double      yMax;			// maximum value of y coordinate
	int	    cx;				// number of x screen coordinates
	int	    cy;				// number of y screen coordinates
	int	    cxBorder;			// border in x direction
	int	    cyBorder;			// border in y direction
	double      xDelta;			// delta x spacing
	double      yDelta;			// delta y spacing

	inline int  xMAP (double x) {return (int) (cx*(    (x-xMin)/xDelta))+cxBorder;}
	inline int  yMAP (double y) {return (int) (cy*(1.0-(y-yMin)/yDelta))+cyBorder;}
	void        FillPoly (int,double*,int,int);
	void        RedrawMesh(void *);

  protected:
  	ZONE*       Get_zone(const char *);
	ZONE*       Get_zone(const int);
	Segment*    Get_segment(const int);
	int*        reorder_zone(int);              //needed only by solver.
  public:
	int	    import_cgns(const char *cgnsfile);
	void 	    export_mesh(const char *cgnsfile);
	void 	    draw_mesh_screen(void *);
	void        draw_mesh_ps(char *psfile);
	void  	    build_zone();
	int         zone_to_field();
	int         field_to_zone();
	void        reorder_mesh();
	void	    clear_mesh();
  public:

	MESH        ():mesh_scale(1e6),bc_num(0),zone_num(0),zone(0)
	 {xMin = +1.0e100;   yMin = +1.0e100;   xMax = -1.0e100;   yMax = -1.0e100;}
 	~MESH       () {delete [] zone;}
	void        set_mesh_scale(double scale) {mesh_scale = scale;}

};

#endif
