//sevel types of vsource are defined here
//reference: spice vsource model
#ifndef _vsource_h_
#define _vsource_h_
#include <math.h>

//virtual base class
class VSource
{
	public:
		char label[32];
		virtual double vapp(double t)=0;
};
		

class VDC: public VSource 
{
	private:
		double td;
		double Vdc;
        public:
        	VDC(double t1,double v1): td(t1),Vdc(v1){};
        	double vapp(double t) 
         	{ return t>=td? Vdc:0;};
};



class VSIN: public VSource 
{
	private:
		double td;
		double Vamp;
		double fre;
	public:  
		VSIN(double t1,double v1,double f1): td(t1),Vamp(v1),fre(f1){};
        	double vapp(double t) 
         	{ return t>=td? Vamp*sin(2*3.14159265359*fre*(t-td)):0;};
};


class VPULSE: public VSource 
{	
	private:
		double td;
		double tr;
		double tf;
		double pw;
		double pr;
		double Vlo,Vhi;
	public:  
		VPULSE(double t1,double v1,double v2,double t2,double t3,double t4, double t5):
		 td(t1),tr(t2),tf(t3),pw(t4),pr(t5),Vlo(v1),Vhi(v2){};
        	double vapp(double t) 
         	{ 
         		if(t<td)
         			return Vlo;
         		else { 
         			t-=td;
         			while(t>pr) t-=pr;
         			if(t<tr)
         				return Vlo+t*(Vhi-Vlo)/tr;
         			else if(t<tr+pw)
         				return Vhi;
         			else if(t<tr+pw+tf)
         				return Vhi-(t-tr-pw)*(Vhi-Vlo)/tf;
         			else    return Vlo;
         		      }			
         				
         	}	
};


class VEXP: public VSource 
{	
	private:
		double td;
		double trc;
		double tfd;
		double tfc;
		double Vlo,Vhi;
	public:  
		VEXP(double t1,double v1,double v2,double t2,double t3,double t4):
		 td(t1),trc(t2),tfd(t3),tfc(t4),Vlo(v1),Vhi(v2){};
        	double vapp(double t) 
         	{ 
         		if(t<=td)
         			return Vlo;
         		else if(t<=tfd)
         			return Vlo+(Vhi-Vlo)*(1-exp(-(t-td)/trc));
         		else
         			return Vlo+(Vhi-Vlo)*(1-exp(-(t-td)/trc))+(Vlo-Vhi)*(1-exp(-(t-tfd)/tfc));		
         	        
         	};	
};

#endif
