#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include <cgnslib.h>

#include "xgraph.h"
#include "mesh.h"
#include "geom.h"

VoronoiCell::VoronoiCell():  mirror_num(0),nb_num(0) //constructor
{ nb_array  = (int*) NULL;
  inb_array = (int*) NULL;
  elen	    = (double*) NULL;
  ilen	    = (double*) NULL;
  angle     = (double*) NULL;
  celledge  = (int*) NULL;
  mirror    = (MirrorNode*) NULL;
}

VoronoiCell::VoronoiCell(const VoronoiCell& org) //copy constructor
{
  x =  org.x;  	  y =  org.y;
  nb_num =  org.nb_num;
  area   =  org.area;
  bc_index = org.bc_index;
  mirror_num =  org.mirror_num;
  garea      =  org.garea;
  nb_array  = (int*) NULL;
  inb_array = (int*) NULL;
  elen	    = (double*) NULL;
  ilen	    = (double*) NULL;
  angle     = (double*) NULL;
  celledge  = (int*) NULL;
  mirror    = (MirrorNode*) NULL;
  if(nb_num)
  {
   	nb_array = new int[nb_num];
   	inb_array= new int[nb_num];
  	elen = new double[nb_num];
   	ilen = new double[nb_num];
   	angle= new double[nb_num];
   	celledge = new int[nb_num];
   	for(int i=0;i<nb_num; i++)
  	{
   		nb_array[i] = org.nb_array[i];
   		inb_array[i] = org.inb_array[i];
   		elen[i] = org.elen[i];
   		ilen[i] = org.ilen[i];
   		angle[i] = org.angle[i];
   		celledge[i] = org.celledge[i];
   	}
  }
  if(mirror_num)
  {
   	mirror   = new MirrorNode[mirror_num];
   	for(int i=0;i<mirror_num; i++)
  		mirror[i] = org.mirror[i];

  }
}

VoronoiCell::~VoronoiCell()
{
  delete [] nb_array  ;
  delete [] inb_array ;
  delete [] elen      ;
  delete [] ilen      ;
  delete [] angle     ;
  delete [] celledge  ;
  delete [] mirror    ;
}

/* ----------------------------------------------------------------------------
 * ZONE::setup_voronoicell:  This function setup voronoicell data structure
 * the adjacency infomation will be build here.
 * if the Triangle is count clock wise ordered, this function works well
 */

typedef struct
{
	int node1,node2;
	int flag1,flag2;
} ATABLE;

int ZONE::setup_voronoicell()
{
	int adjtabsize = 0;

	vector<ATABLE> **zone_atable;
	zone_atable = new vector<ATABLE>*[danode.size()];
	for(int i=0;i<danode.size();i++)
		zone_atable[i] = new vector<ATABLE>;

	ATABLE tmp_atable;
	tmp_atable.flag1=0; tmp_atable.flag2=0;

	davcell.clear();
	davcell.Init(danode.size());   //init davcell, cell num = node num

	//insert triangle into each node's adjacent table
	for(int i=0;i<datri.size();i++)
	{
		if(datri[i].D != -1)
		{	//rectangle insert adjacent nodes only!
			tmp_atable.node1 =  datri[i].B;
			tmp_atable.node2 =  datri[i].D;
			zone_atable[datri[i].A]->push_back(tmp_atable);

			tmp_atable.node1 =  datri[i].C;
			tmp_atable.node2 =  datri[i].A;
			zone_atable[datri[i].B]->push_back(tmp_atable);

			tmp_atable.node1 =  datri[i].D;
			tmp_atable.node2 =  datri[i].B;
			zone_atable[datri[i].C]->push_back(tmp_atable);

			tmp_atable.node1 =  datri[i].A;
			tmp_atable.node2 =  datri[i].C;
			zone_atable[datri[i].D]->push_back(tmp_atable);

		}
		else
		{	//triangle
			tmp_atable.node1 =  datri[i].B;
			tmp_atable.node2 =  datri[i].C;
			zone_atable[datri[i].A]->push_back(tmp_atable);

			tmp_atable.node1 =  datri[i].C;
			tmp_atable.node2 =  datri[i].A;
			zone_atable[datri[i].B]->push_back(tmp_atable);

			tmp_atable.node1 =  datri[i].A;
			tmp_atable.node2 =  datri[i].B;
			zone_atable[datri[i].C]->push_back(tmp_atable);
		}
	}


	stack<int> clockw;
	queue<int> cclockw;
	for(int i=0;i<davcell.size();i++)
	{
		vector<ATABLE>::iterator pt =  zone_atable[i]->begin();
		int v1,v2;

		v1 = pt->node1;
		v2 = pt->node2;
		//search it's neighbor triangles. begin from zone_atable[0]
		//clock wise search.push the nodes into a stack
		clockw.push(v1); pt->flag1=1;
		for(int num=0;num<zone_atable[i]->size();num++)
		  for(pt=zone_atable[i]->begin();pt!=zone_atable[i]->end();pt++)
		    if(pt->node2==v1&&pt->flag1==0&&pt->flag2==0)
		  {

		  	v1=pt->node1; 	pt->flag1 =1; 	pt->flag2 =1;
		    	if(v1!=v2) clockw.push(v1);

		  }
		//count clock wise search, push into a queue
		pt =  zone_atable[i]->begin();
		pt->flag2 = 1;
		cclockw.push(v2);

		for(int num=0;num<zone_atable[i]->size();num++)
		   for(pt=zone_atable[i]->begin();pt!=zone_atable[i]->end();pt++)
		    if(pt->node1==v2&&pt->flag1==0&&pt->flag2==0)
		  {
		  	v2=pt->node2;  	pt->flag1 =1;  	pt->flag2 =1;
		  	cclockw.push(v2);
		  }
		//alloc memory for each vcell, set pointers,cp bc_index form danode
		VoronoiCell *pCell = davcell.GetPointer(i);
		int clockwsize = clockw.size();
		int cclockwsize = cclockw.size();
		int neighbor = clockw.size() + cclockw.size();

		pCell->nb_num = neighbor;
		pCell->bc_index = danode[i].bc_index;
		pCell->nb_array = new int[neighbor];
 		pCell->elen  = new double[neighbor];
 		pCell->ilen  = new double[neighbor];
 		pCell->angle = new double[neighbor];
 		pCell->celledge = new int[neighbor];
 		pCell->inb_array = new int[neighbor];

                for(int j=0;j<pCell->nb_num;j++)  pCell->celledge[j] = -1;
 		//now read the neighbors of this node in count clock order
 		for(int j=0;j<clockwsize;j++)
 		{
 		  pCell->nb_array[j] = clockw.top();
 		  clockw.pop();
 		}
 		for(int j=clockwsize;j<cclockwsize+clockwsize;j++)
 		{
 		  pCell->nb_array[j] = cclockw.front();
 		  cclockw.pop();
 		}

 		adjtabsize += neighbor;

	}

	//adjtabsize equal to vedge*2
	davedge.clear();
	davedge.resize(adjtabsize/2);


	//compute invert index of neighbour nodes
	//i == davcell[j].nb_array[davcell[i].inb_array[j]]
	for(int i=0;i<davcell.size();i++)
        {
        	VoronoiCell *pCell = davcell.GetPointer(i);
           	for(int j=0;j<davcell[i].nb_num;j++)
        	  for(int k=0;k<davcell[davcell[i].nb_array[j]].nb_num;k++)
        	  {
        	  	if(i==davcell[davcell[i].nb_array[j]].nb_array[k])
        			pCell->inb_array[j]=k;
     		  }
	}

	//not needed any more, free them.
	for(int i=0;i<davcell.size();i++)
	 	delete zone_atable[i];
	delete [] zone_atable ;
	return 0;

}

/* ----------------------------------------------------------------------------
 * ZONE::setup_voronoicell:  This function setup voronoicell data structure
 * the adjacency infomation will be build here.
 * if the Triangle is not count clock wise ordered, but the zone is rectangular.
 * please use this  function.
 */
/*
int ZONE::setup_voronoicell()
{
	int adjtabsize=0;
	list<int>::iterator pt;
	list<int>**pneighborlist;
	davcell.clear();
	davcell.Init(danode.size());   //init davcell, cell num = node num

	//compute max triangle, it may be useful
	double max_angle = 0;
	for(int i=0;i<datri.size();i++)
	{
		 double angle= MaxTriAngle(danode[datri[i].A].x,danode[datri[i].A].y,
		 	      	       	   danode[datri[i].B].x,danode[datri[i].B].y,
		                	   danode[datri[i].C].x,danode[datri[i].C].y);
		 if(max_angle<angle) max_angle=angle;
		 if(datri[i].D!=-1)
		        angle= MaxTriAngle(danode[datri[i].A].x,danode[datri[i].A].y,
		 	      	       	   danode[datri[i].C].x,danode[datri[i].C].y,
		                	   danode[datri[i].D].x,danode[datri[i].D].y);
		 if(max_angle<angle) max_angle=angle;
	}
	//alloc list. each voronoi cell will have it's own neighborlist.
	//my god ,tens of thounds of lists i should alloc.
	//stupid code

	pneighborlist=new (list<int>*) [davcell.size()];
	for(int i=0;i<davcell.size();i++)	pneighborlist[i] = new list<int>;
	//first for each cell, insert all neighbor nodes into the each node's neighborlist.
	for(int i=0;i<datri.size();i++)
	{
		if(datri[i].D != -1)
		{	//rectangle insert adjacent nodes only!
			pneighborlist[datri[i].A]->push_front(datri[i].D);
			pneighborlist[datri[i].A]->push_front(datri[i].B);
			pneighborlist[datri[i].B]->push_front(datri[i].A);
			pneighborlist[datri[i].B]->push_front(datri[i].C);
			pneighborlist[datri[i].C]->push_front(datri[i].B);
			pneighborlist[datri[i].C]->push_front(datri[i].D);
			pneighborlist[datri[i].D]->push_front(datri[i].C);
			pneighborlist[datri[i].D]->push_front(datri[i].A);
		}
		else
		{	//triangle
			pneighborlist[datri[i].A]->push_front(datri[i].C);
			pneighborlist[datri[i].A]->push_front(datri[i].B);
			pneighborlist[datri[i].B]->push_front(datri[i].A);
			pneighborlist[datri[i].B]->push_front(datri[i].C);
			pneighborlist[datri[i].C]->push_front(datri[i].B);
			pneighborlist[datri[i].C]->push_front(datri[i].A);
		}
	}

	//then remove the overlap nodes in pneighborlist
	for(int i=0;i<davcell.size();i++)
	  for(pt=pneighborlist[i]->begin();pt!=pneighborlist[i]->end();)
	    if(*pt==i||count(pneighborlist[i]->begin(),pneighborlist[i]->end(),*pt)>1 )
	    	pt = pneighborlist[i]->erase(pt);
	    else
		pt++;

	//alloc memory for each vcell, set pointers,cp bc_index form danode
	VoronoiCell *pCell = davcell.GetPointer(0);
	for(int i=0;i<davcell.size();i++,pCell++)
	{
		pCell->nb_num = pneighborlist[i]->size();
		pCell->nb_array = new int[pneighborlist[i]->size()];
 		pCell->elen  = new double[pneighborlist[i]->size()];
 		pCell->ilen  = new double[pneighborlist[i]->size()];
 		pCell->angle = new double[pneighborlist[i]->size()];
 		pCell->celledge = new int[pneighborlist[i]->size()];
 		pCell->inb_array = new int[pneighborlist[i]->size()];

		adjtabsize += pneighborlist[i]->size();
	}
	//adjtabsize equal to vedge*2
	davedge.clear();
	davedge.resize(adjtabsize/2);

	// use angle to horizontal line to reorder the neighbour nodes as count clocked
	double x1,y1,x2,y2,x3,y3,r,angle;
	map<double,int>  adjmap;
	map<double,int> ::iterator padjmap;
	list<int> NODElist;
	list<int>::iterator pNODElist, pstart;

	for(int i=0;i<davcell.size();i++)
	{       //compute each angle to horizontal line
		//use container map to order them
		for(pt=pneighborlist[i]->begin();pt!=pneighborlist[i]->end();pt++)
	        {
	    		x2 = danode[i].x;	  	y2 = danode[i].y;
	        	x1 = danode[*pt].x;             y1 = danode[*pt].y;
	        	r  = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
	        	x3 = danode[i].x+r;	        y3 = danode[i].y;
	        	angle = CompAngle(x1,y1,x2,y2,x3,y3);
	  		adjmap.insert(map<double,int>::value_type(angle,*pt));
	  	}
	    	int    *pnb_index = davcell.GetPointer(i)->nb_array;
	    	double *pangle = davcell.GetPointer(i)->angle;

	  	for(padjmap = adjmap.begin();padjmap!=adjmap.end();padjmap++)
	  	{
	     		*pnb_index++ = padjmap->second;
	      		*pangle++ = padjmap->first;
	  	}
	  	adjmap.clear();

	  	//boundary cell may have problem. the nb_array may be dissevered by bd nodes
	  	//i have to reorder it to make sure that bd nodes are located at first and last
	  	//positions of nb_array array
	  	int do_reorder = 0;
	  	if( davcell[i].bc_index != 0) //it is a bd cell
	  	{	for(int j=0;j<davcell[i].nb_num-1;j++)
	  	 	   if(fabs(davcell[i].angle[j]-davcell[i].angle[j+1])>max_angle)
	  	           	 {do_reorder=j+1;break;}

	  	        // the angle between line i,j and line i,j+1 are larger than max_angle
	  	        // denote that j and j+1 are bd nodes.
	  	        // j+1 will be set at the beginning of nb_array ,j set as last value
	  	}
	  	if(do_reorder)
		{

			//push all the neighbors into a list
			for(int j=0;j<davcell[i].nb_num;j++)
	  			NODElist.push_back(davcell[i].nb_array[j]);
	     	   	pNODElist=NODElist.begin();
	     	   	while((*pNODElist)!=davcell[i].nb_array[do_reorder]) pNODElist++;
	     	   	//find do_reorder point
	     	   	pstart = pNODElist;
	     	   	pnb_index = davcell.GetPointer(i)->nb_array;
	     	   	//change the order
	     	        for(;pNODElist!=NODElist.end();pNODElist++)
	     			*pnb_index++ = *pNODElist;
	     	   	for(pNODElist=NODElist.begin();pNODElist!=pstart;pNODElist++)
	     			*pnb_index++ = *pNODElist;
	     	  	//recoder finished
	     	  	NODElist.clear();
	     	  	do_reorder=0;
	     	  	//recompute angle
	     	  	pangle = davcell.GetPointer(i)->angle;
	     	   	for(int j=0;j<davcell[i].nb_num;j++)
	     	  	{
	     			x2 = danode[i].x;	  	y2 = danode[i].y;
	     		   	Node nb_node = danode[davcell[i].nb_array[j]];
	     	     	   	x1 = nb_node.x;
	      	     	   	y1 = nb_node.y;
	      	     	   	r  = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
	     	     	   	x3 = x2+r;
	     	     	   	y3 = y2;
	     	      	   	*pangle++ = CompAngle(x1,y1,x2,y2,x3,y3);
	    	    	}
	  	 }


	}
	//compute invert index of neighbour nodes
	//davcell[i] == davcell[davcell[j].nb_array[davcell[i].inb_array[j]]]
	for(int i=0;i<davcell.size();i++)
        {
        	VoronoiCell *pCell = davcell.GetPointer(i);
           	for(int j=0;j<davcell[i].nb_num;j++)
        	  for(int k=0;k<davcell[davcell[i].nb_array[j]].nb_num;k++)
        	  {
        	  	if(i==davcell[davcell[i].nb_array[j]].nb_array[k])
        			pCell->inb_array[j]=k;
     		  }
	}

	//not needed any more, free them.
	for(int i=0;i<davcell.size();i++)
	 	delete pneighborlist[i];
	delete [] pneighborlist ;
	return 0;

}
*/
/* ----------------------------------------------------------------------------
 * ZONE::setup_voronoicell: the remain work is eary to do. ilen,elen and area
 * will be setted here.
 */
int ZONE::build_voronoicell_data()
{
	//copy some data from danode
	for(int i=0;i<davcell.size();i++)
	{
		VoronoiCell *pCell = davcell.GetPointer(i);
		pCell->x = danode[i].x;
		pCell->y = danode[i].y;
	  	pCell->bc_index = danode[i].bc_index;
	}

	//comput angle: the angle from horizontal line to it's neighbor node
	double x1,y1,x2,y2,x3,y3,r,angle;
	for(int i=0;i<davcell.size();i++)
	{
		double *pangle = davcell.GetPointer(i)->angle;
	     	for(int j=0;j<davcell[i].nb_num;j++)
	     	{
	     		x2 = danode[i].x;	  	y2 = danode[i].y;
	     	   	Node nb_node = danode[davcell[i].nb_array[j]];
	     	   	x1 = nb_node.x;
	      	   	y1 = nb_node.y;
	      	   	r  = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
	     	   	x3 = x2+r;
	     	   	y3 = y2;
	     	   	*pangle++ = CompAngle(x1,y1,x2,y2,x3,y3);
	    	}
	}

	//comput ilen: length between node i and it's neighbor
	for(int i=0;i<davcell.size();i++)
	{
		VoronoiCell *pCell = davcell.GetPointer(i);
	  	for(int j=0;j<davcell[i].nb_num;j++)
	  	{
	  		Node n = danode[davcell[i].nb_array[j]];
	 		pCell->ilen[j]=Distance(danode[i].x,danode[i].y,n.x,n.y);
	 	}
	}

        //next to comput elen , length of Voronoi Cell's Edge
        //at the same time ,davedge will be set
	list<double> listxc,listyc; //two list to store location of CircleCenter
	list<double>::iterator pxc,pyc;
	double xc,yc,xc_next,yc_next;
	vector<VEdge>:: iterator pEdge = davedge.begin();
        int nedge = 0;
	for(int i=0;i<davcell.size();i++)
        {
        	//fill the list with CircleCenter
        	for(int j=0;j<davcell[i].nb_num-1;j++)
        	{
        	  CircleCenter(danode[i].x,danode[i].y,
        	               danode[davcell[i].nb_array[j]].x,
        	               danode[davcell[i].nb_array[j]].y,
	  	 	       danode[davcell[i].nb_array[j+1]].x,
        	               danode[davcell[i].nb_array[j+1]].y,
	  	 	       xc,yc);
	  	  listxc.push_back(xc);
	  	  listyc.push_back(yc);
	  	}
	  	int nb_first= davcell[i].nb_array[0];
	  	int nb_last = davcell[i].nb_array[davcell[i].nb_num-1];
	  	//if all the 3 points are bd ndoes
   		if (davcell[i].bc_index&&davcell[nb_first].bc_index&&
   		          davcell[nb_last].bc_index)
	  	  {
	  	    	xc = (danode[i].x+danode[nb_first].x)/2.0;
	  	 	yc = (danode[i].y+danode[nb_first].y)/2.0;
	  	 	listxc.push_front(xc);
	  	 	listyc.push_front(yc);
	  	 	xc = (danode[i].x+danode[nb_last].x)/2.0;
	  	 	yc = (danode[i].y+danode[nb_last].y)/2.0;
	  	 	listxc.push_back(xc);
	  	 	listyc.push_back(yc);
          	   }
	  	else //compute CircleCenter of the last triangle
	  	   {
	  	   	CircleCenter(danode[i].x,danode[i].y,
	  	   	       danode[nb_first].x,
	  	   	       danode[nb_first].y,
	  	   	       danode[nb_last].x,
	  	   	       danode[nb_last].y,
	  	 	       xc,yc);
	  	 	listxc.push_front(xc);
	  	 	listyc.push_front(yc);
	  	        listxc.push_back(xc);
	  	        listyc.push_back(yc);
	       	    }

               	pxc=listxc.begin();
               	pyc=listyc.begin();
               	VoronoiCell *pCell = davcell.GetPointer(i);


               	for(int j=0;j<davcell[i].nb_num;j++)
               	{
                        xc = *pxc++;       yc = *pyc++;
              		xc_next = *pxc;    yc_next = *pyc;
               		pCell->elen[j] = Distance(xc,yc,xc_next,yc_next);

               		// each vedge belongs to 2 vcells

               		int i_index = davcell[i].inb_array[j];
               		if(davcell[i].celledge[j]==-1 &&
               		  davcell[davcell[i].nb_array[j]].celledge[i_index]==-1)
               		{       //this edge not registered. set data to it
               			pEdge->cell1 = i;
               			pEdge->cell2 = davcell[i].nb_array[j];
               			pEdge->c1_nb = j;
               			pEdge->c2_nb = davcell[i].inb_array[j];

               			pEdge->length = pCell->elen[j];
               			pEdge->x1 = xc;       pEdge->y1 = yc;
               			pEdge->x2 = xc_next;  pEdge->y2 = yc_next;
               			davcell.GetPointer(i)->celledge[j] = nedge;
               			davcell.GetPointer(davcell[i].nb_array[j])->celledge[i_index] = nedge;
               			pEdge++; nedge++;
               		}

               	}

               	listxc.clear();
               	listyc.clear();
	}

	// compute each cell's volume
	for(int i=0;i<davcell.size();i++)
	{
	    VoronoiCell *pCell = davcell.GetPointer(i);
	    pCell->garea = 0;
	    pCell->area = 0;
	    for(int j=0;j<davcell[i].nb_num;j++)
		pCell->area += davcell[i].elen[j]*davcell[i].ilen[j]/4.0;
	}

	// compute total volume
	field_area_vcell =0;
	for(int i=0;i<davcell.size();i++)
	      	field_area_vcell += davcell[i].area;

        //printf("field area    : %e \n",field_area_vcell);
	return 0;

}


/* ----------------------------------------------------------------------------
 * ZONE::build_ghost_cell:  This function build ghost cell
 * for all the boundary nodes
 */
int ZONE::build_ghost_cell()
{
	for(int i=0;i<danode.size();i++)
	  if(danode[i].bc_index)    //it is a bd node
	  {
	     	VoronoiCell *pCell = davcell.GetPointer(i);
	     	//determine this bd node is on edge or in corner
	       	double x =  danode[i].x;
	     	double y =  danode[i].y;
	     	double x1 = danode[davcell[i].nb_array[0]].x;
	     	double y1 = danode[davcell[i].nb_array[0]].y;
	     	double x2 = danode[davcell[i].nb_array[davcell[i].nb_num-1]].x;
	     	double y2 = danode[davcell[i].nb_array[davcell[i].nb_num-1]].y;
	     	double  r1= Distance(x,y,x1,y1);
	     	double  r2= Distance(x,y,x2,y2);
	     	double  r=max(r1,r2);
	     	if(TriArea(x,y,x1,y1,x2,y2)<0.5*r1*r2*sin(3.14*1.0/180.0))
	     	{
	     	   // the triangle area is nearly zero. a edge node
	     	   // alloc memory
	     	   pCell->mirror_num = davcell[i].nb_num;
     		   pCell->mirror = new MirrorNode[davcell[i].nb_num];
	     	   //compute GhostPoint location and angle,elen,ilen
	     	   for(int k=0; k<davcell[i].nb_num; k++)
	     	   {
	     	   	pCell->mirror[k].index = davcell[i].nb_array[k];
	     	   	pCell->mirror[k].nbindex = k;
	     	   	MirrorPoint(danode[davcell[i].nb_array[k]].x,danode[davcell[i].nb_array[k]].y,x,y,x1,y1,
	     	   	     pCell->mirror[k].x,pCell->mirror[k].y);

			pCell->mirror[k].angle = CompAngle(pCell->mirror[k].x,pCell->mirror[k].y,x,y,x+r,y);
			pCell->mirror[k].elen = davcell[i].elen[k];
			pCell->mirror[k].ilen = davcell[i].ilen[k];
		    }
		    //compute ghost cell area and norm vector to bd
		    for(int k=0; k<davcell[i].nb_num; k++)
		    {
		    	double xc = danode[davcell[i].mirror[k].index].x;
		    	double yc = danode[davcell[i].mirror[k].index].y;
		    	double xcg = davcell[i].mirror[k].x;
		    	double ycg = davcell[i].mirror[k].y;
		    	if(Distance(xc,yc,xcg,ycg)<1e-12)
		    	{
		    		pCell->mirror[k].nx = 0;
		    	        pCell->mirror[k].ny = 0;
		    	}
		    	else
		    	{      pCell->mirror[k].nx = (xcg-xc)/Distance(xc,yc,xcg,ycg);
		    	       pCell->mirror[k].ny = (ycg-yc)/Distance(xc,yc,xcg,ycg);
		    	}
		    }
		    pCell->garea = davcell[i].area;
 		}
		else
	     	{ //the node is  on corner
	     	  // alloc memory. each node have 3 mirror node
	     	   pCell->mirror_num = 3*davcell[i].nb_num;
     		   pCell->mirror = new MirrorNode[3*davcell[i].nb_num];
	     	   int num = davcell[i].nb_num;
	     	   pCell->garea = 3*davcell[i].area;

	     	  //compute MirrorPoint,determined by line (x,y),(x1,y1)
	     	   for(int k=0;k<num;k++)
	     	   {
	     	   	pCell->mirror[k].index = davcell[i].nb_array[k];
	     	   	pCell->mirror[k].nbindex = k;
	     	   	MirrorPoint(danode[davcell[i].nb_array[k]].x,danode[davcell[i].nb_array[k]].y,x,y,x1,y1,
	     	   	     pCell->mirror[k].x,pCell->mirror[k].y);
	     	 	pCell->mirror[k].angle =CompAngle(pCell->mirror[k].x,pCell->mirror[k].y,x,y,x+r,y);
			pCell->mirror[k].elen = davcell[i].elen[k];
			pCell->mirror[k].ilen = davcell[i].ilen[k];
		   }

		  //compute MirrorPoint,determined by line (x,y),(x1,y1)
		    for(int k=0;k<num;k++)
	     	   {
	     	   	pCell->mirror[k+num].index = davcell[i].nb_array[k];
	     	   	pCell->mirror[k+num].nbindex = k;
	     	   	MirrorPoint(danode[davcell[i].nb_array[k]].x,danode[davcell[i].nb_array[k]].y,x,y,x2,y2,
	     	   	     pCell->mirror[k+num].x,pCell->mirror[k+num].y);
			pCell->mirror[k+num].angle = CompAngle(pCell->mirror[k+num].x,pCell->mirror[k+num].y,x,y,x+r,y);
			pCell->mirror[k+num].elen = davcell[i].elen[k];
			pCell->mirror[k+num].ilen = davcell[i].ilen[k];
		   }

		  //compute MirrorPoint,determined by point(x,y)
		    for(int k=0;k<num;k++)
	     	   {
	     	   	pCell->mirror[k+2*num].index = davcell[i].nb_array[k];
	     	   	pCell->mirror[k+2*num].nbindex = k;
	     	   	pCell->mirror[k+2*num].x = 2*x - danode[davcell[i].nb_array[k]].x;
	     	   	pCell->mirror[k+2*num].y = 2*y - danode[davcell[i].nb_array[k]].y;
			pCell->mirror[k+2*num].angle = CompAngle(pCell->mirror[k+2*num].x,pCell->mirror[k+2*num].y,x,y,x+r,y);
			pCell->mirror[k+2*num].elen = davcell[i].elen[k];
			pCell->mirror[k+2*num].ilen = davcell[i].ilen[k];
		   }
		//compute norm vector to bd

		    for(int k=0; k<3*num; k++)
		    {
		    	double xc = danode[davcell[i].mirror[k].index].x;
		    	double yc = danode[davcell[i].mirror[k].index].y;
		    	double xcg = davcell[i].mirror[k].x;
		    	double ycg = davcell[i].mirror[k].y;
		    	if(Distance(xc,yc,xcg,ycg)<1e-12)
		    	{
		    		pCell->mirror[k].nx = 0;
		    	        pCell->mirror[k].ny = 0;
		    	}
		    	else
		    	{      pCell->mirror[k].nx=(xcg-xc)/Distance(xc,yc,xcg,ycg);
		    	       pCell->mirror[k].ny=(ycg-yc)/Distance(xc,yc,xcg,ycg);
		    	}
		    }

	     	 }
	}
	return 0;

}


void ZONE::ghost_cell_debug()
{
	cout<<"ghost debug..."<<endl;

	for(int i=0;i<danode.size();i++)
          if(davcell[i].bc_index)
          {
            if(davcell[i].mirror_num!=davcell[i].nb_num) cout<<"corner"<<endl;
            for(int j=0;j<davcell[i].nb_num;j++)
               	cout<<"x="<<danode[davcell[i].nb_array[j]].x<<'\t'<<"y="<<danode[davcell[i].nb_array[j]].y<<endl;

            cout<<"ghost point"<<endl;
            for(int j=0;j<davcell[i].mirror_num;j++)
            {
            	cout<<"x="<<davcell[i].mirror[j].x<<'\t'<<"y="<<davcell[i].mirror[j].y<<endl;
            	cout<<davcell[i].mirror[j].nx<<'\t'<<davcell[i].mirror[j].ny<<endl;
            }
            cout<<endl;
          }

}


//simple error hanfle function
inline int cgns_error_handle(char *s,int index_file)
{
	printf("%s\n",s);
	cg_close(index_file);
        return 1;
}

/* ----------------------------------------------------------------------------
 * MESH::import_cgns:  This function read node location, cell node, bounary
 * definition and solution data from cgns file
 */
int MESH::import_cgns(const char *cgnsfile)
{
	int index_file,index_base,index_zone,index_sect,index_I;
	int nbase,ndescriptor,nsection,nbndry,nconns,npnts,ndata_donor,ElementDataSize,BCNum;
	char sectionname[32],descriptorname[32],boconame[32],solname[32],fieldname[32],connectname[32],donorname[32];

	int  cell_dim,physical_dim,isize[3],iparent_flag,iparentdata;
	int  normalindex,normallistflag,ndataset,nsol,normallist;
	double *x,*y;
	int *ielem;
	int  err;
	ZoneType_t     zonetype,donor_zonetype;
	ElementType_t  elemtype;
	BCType_t       bocotype;
	PointSetType_t ptset,donor_ptset_type;
	DataType_t     normaldatatype,donor_datatype;
	GridLocation_t location;
	GridConnectivityType_t connect_type;


	printf("\n--------------------------------------------------\n");
	//--------------------------------------------------------------------------------------
	printf("\nLoading cgns...\n");
	//open the cgns file
	if(cg_open(cgnsfile,MODE_READ,&index_file))
		{printf("error, can't open cgns file\n");return 1;}
	printf("cgns file     : %s \n",cgnsfile);

	//--------------------------------------------------------------------------------------
	//cgns file should have only one base
	assert(!cg_nbases(index_file,&nbase));
	if(nbase!=1)
		return cgns_error_handle("multibases found,not a supported cgns file",index_file);

	//--------------------------------------------------------------------------------------
	//read base information
	index_base=1; //because only have one base
	assert(!cg_base_read(index_file,index_base,basename,&cell_dim,&physical_dim));
	if(physical_dim!=2)
		return cgns_error_handle("wrong physical_dim found,not a supported cgns file",index_file);
	printf("base name     : %s\n",basename);

	//--------------------------------------------------------------------------------------
	//cgns file may have several zones
	assert(!cg_nzones(index_file,1,&zone_num));
	printf("this cgns file have %d zones\n\n",zone_num);
	zone = new ZONE[zone_num];
	int node_offset = 0;
	//read zone information
	for(index_zone=1;index_zone<=zone_num;index_zone++,node_offset+=isize[0])
	{
		ZONE *pzone = &zone[index_zone-1];
		pzone->zone_index = index_zone-1;

		assert(!cg_zone_type(index_file,index_base,index_zone,&zonetype));
		if(zonetype!=Unstructured)
			return cgns_error_handle("Unstructured zone needed,not a supported cgns file",index_file);
		assert(!cg_zone_read(index_file,index_base,index_zone,pzone->zonename,isize));
		printf("read zone %d...\n",index_zone);
		printf("zone name     : %s\n",pzone->zonename);
		printf("points        : %d\n",isize[0]);
		printf("elements      : %d\n",isize[1]);

		char *label;
		cg_goto(index_file,index_base,"Zone_t",index_zone,"end");
		cg_ndescriptors(&ndescriptor);
		if(ndescriptor)
		{
  			cg_descriptor_read(ndescriptor,descriptorname,&label);
  			strcpy(pzone->zonelabel,label);
  		}
		else
			strcpy(pzone->zonelabel,"Unspecified");
		free(label);

		pzone->offset   = node_offset;
		//alloc double array for x and y coordinate
        	x = new double[isize[0]]; assert(x);
        	y = new double[isize[0]]; assert(y);

        	//read the coordinate
        	int index = 1;
		assert(!cg_coord_read(index_file,index_base,index_zone,"CoordinateX",RealDouble,&index,&isize[0],x));
		assert(!cg_coord_read(index_file,index_base,index_zone,"CoordinateY",RealDouble,&index,&isize[0],y));


        	//assign coordinates to danode, clear bc
		Node tmp_node;
		for(int i=0;i<isize[0];i++)
        	{	//scale length to my own standerd--0.1um
        		tmp_node.x = x[i]*mesh_scale;
        		tmp_node.y = y[i]*mesh_scale;
        		tmp_node.bc_index = 0;
        		tmp_node.index = i;
        		tmp_node.zone_index = index_zone - 1;
        		if (tmp_node.x > xMax) xMax = tmp_node.x;
    			if (tmp_node.x < xMin) xMin = tmp_node.x;
    			if (tmp_node.y > yMax) yMax = tmp_node.y;
    			if (tmp_node.y < yMin) yMin = tmp_node.y;
    			pzone->danode.push_back(tmp_node);
         	}
        	delete [] x; delete [] y;

        	//--------------------------------------------------------------------------------------
        	// find out how many sections
        	assert(!cg_nsections(index_file,index_base,index_zone,&nsection));

        	//read section information
        	for(index_sect = 1; index_sect<=nsection; index_sect++)
		{
			assert(!cg_section_read(index_file,index_base,index_zone,index_sect,
	                	sectionname,&elemtype,&index,&isize[1],&nbndry,&iparent_flag));
			if(elemtype==MIXED) break;
		}
		if(index_sect>nsection)
			return cgns_error_handle("error elem type,not a supported cgns file",index_file);
		//read elements
		assert(!cg_ElementDataSize(index_file,index_base,index_zone,index_sect,&ElementDataSize));

		ielem= new int[ElementDataSize]; assert(ielem);
		assert(!cg_elements_read(index_file,index_base,index_zone,index_sect,ielem,&iparentdata));
		//alloc memory for datri

		Tri tmp_Tri;
		int *pelem = ielem;
		for(int i=0;i<=isize[1]-index;i++)
		{
	    		tmp_Tri.zone_index = index_zone-1;
	    		if(*pelem == TRI_3)
			{       pelem++;
				tmp_Tri.A = *(pelem++)-1; //cgns node index begin with 1. sub it
				tmp_Tri.B = *(pelem++)-1;
				tmp_Tri.C = *(pelem++)-1;
				tmp_Tri.D = -1;
				tmp_Tri.area = TriArea(pzone->danode[tmp_Tri.A].x,pzone->danode[tmp_Tri.A].y,
						       pzone->danode[tmp_Tri.B].x,pzone->danode[tmp_Tri.B].y,
						       pzone->danode[tmp_Tri.C].x,pzone->danode[tmp_Tri.C].y);
			}
	   	 	else if(*pelem == QUAD_4)
	       		{       pelem++;
				tmp_Tri.A = *(pelem++)-1;
				tmp_Tri.B = *(pelem++)-1;
				tmp_Tri.C = *(pelem++)-1;
				tmp_Tri.D = *(pelem++)-1;
				tmp_Tri.area = TriArea(pzone->danode[tmp_Tri.A].x,pzone->danode[tmp_Tri.A].y,
						       pzone->danode[tmp_Tri.B].x,pzone->danode[tmp_Tri.B].y,
						       pzone->danode[tmp_Tri.C].x,pzone->danode[tmp_Tri.C].y)+
					       TriArea(pzone->danode[tmp_Tri.D].x,pzone->danode[tmp_Tri.D].y,
						       pzone->danode[tmp_Tri.B].x,pzone->danode[tmp_Tri.B].y,
						       pzone->danode[tmp_Tri.C].x,pzone->danode[tmp_Tri.C].y);
	       		}
	   		else{
	               		delete ielem;
	                	return cgns_error_handle("unsupported mesh strucure",index_file);
	       		}

	       		pzone->datri.push_back(tmp_Tri);
		}

		//read zone connect information

  		cg_nconns(index_file,index_base,index_zone,&nconns);
  		pzone->zone_conn_num = nconns;
  		pzone->donor_zone_array.clear();
  		pzone->donor_zone_name.resize(nconns);

  		if(nconns>0)
  		{
  		  Interface pitf;
  		  for(index_I=1;index_I<=nconns;index_I++)
  		  {
  		  	cg_conn_info(index_file,index_base,index_zone,index_I,connectname ,
				&location ,&connect_type ,
				&ptset , &npnts ,donorname ,
				&donor_zonetype , &donor_ptset_type ,
				&donor_datatype , &ndata_donor );
		  	pzone->donor_zone_name[index_I-1] = donorname;
		  	if(zinterface.Find(donorname,pzone->zonename)==-1) //the interface is not set
		  	{

		  		strcpy(pitf.conn_name,connectname);
		  		strcpy(pitf.zone1_name,pzone->zonename);
		  		strcpy(pitf.zone2_name,donorname);
		  		pitf.flag = 0;
		  		pitf.node_num = npnts;
		  		int *index_array1 = new int[npnts];
		  		int *index_array2 = new int[npnts];

		  		cg_conn_read(index_file,index_base,index_zone,index_I,index_array1,
		  			     donor_datatype,index_array2);
		  		for(int j=0;j<pitf.node_num;j++)
	  			{
	  			  pitf.index_array1.push_back(index_array1[j]-1);
	  			  pitf.index_array2.push_back(index_array2[j]-1);
	  			}
	  			delete  index_array1;
	  			delete  index_array2;
	  			zinterface.interface.push_back(pitf);
	  			pitf.index_array1.clear();
	  			pitf.index_array2.clear();

		  	}

		  }
		  printf("zone Connectivity read %d nodes\n",npnts);
		}

		//---------------------------------------------------------------------------------------
		//now read ZoneBC
		//the boundary marker will be assigned to each boundary node

		//how many BC in the ZoneBC
        	assert(!cg_nbocos(index_file,index_base,index_zone,&BCNum));
       		printf("BC number     : %d\n",BCNum);
        	Segment tmp_segment;
        	for(int BCIndex=1;BCIndex<=BCNum;BCIndex++)
        	{
           		assert(!cg_boco_info(index_file,index_base,index_zone,BCIndex,boconame,
	     		&bocotype,&ptset,&npnts,&normalindex,&normallistflag,&normaldatatype,&ndataset));
	     		int start,end;

           		//read section information about boundary segment
        		for(index_sect = 1; index_sect<=nsection; index_sect++)
			{
				assert(!cg_section_read(index_file,index_base,index_zone,index_sect,
	                		sectionname,&elemtype,&start,&end,&nbndry,&iparent_flag));
				if(!strcmp(boconame,sectionname)&&elemtype==BAR_2) break;
			}
			if(index_sect>nsection)
				return cgns_error_handle("boundary elem error,not a supported cgns file",index_file);
			//read elements
			assert(!cg_ElementDataSize(index_file,index_base,index_zone,index_sect,&ElementDataSize));
			assert(!cg_elements_read(index_file,index_base,index_zone,index_sect,ielem,&iparentdata));


	   		int bc_elem_range[2];
	   		cg_boco_read(index_file,index_base,index_zone,BCIndex,bc_elem_range,&normallist);
	   		if(bc_elem_range[0]!=start||bc_elem_range[1]!=end)
	   			 return cgns_error_handle("boundary elem error,not a supported cgns file",index_file);

	   		tmp_segment.bc_index = BCIndex+bc_num;
	   		tmp_segment.zone_index = index_zone-1;   //changed is it ok?
	   		strcpy(tmp_segment.label,boconame);
	   		tmp_segment.edge_num = end-start+1;
	   		tmp_segment.interface=-1;

	   		for(int i=0;i<end-start+1;i++)
	     		{
	     			tmp_segment.edge_array.push_back(ielem[2*i]-1);
	     			tmp_segment.edge_array.push_back(ielem[2*i+1]-1);
	     			pzone->danode[ielem[2*i]-1].bc_index=BCIndex+bc_num;
	     			pzone->danode[ielem[2*i+1]-1].bc_index=BCIndex+bc_num;
	     		}
	     		pzone->dasegment.push_back(tmp_segment);
	     		tmp_segment.edge_array.clear();
	  		printf("   BC %d %s... %d edges read\n",BCIndex,boconame,end-start+1);

  		}
  		bc_num += BCNum;  //the zone bc_index must be different in global
  		delete [] ielem;

		printf("zone %d read finished\n\n",index_zone);
	}
	//post process
	for(int i=0;i<zinterface.interface.size();i++)
	{
		zinterface[i].zone1 = Get_zone(zinterface[i].zone1_name)->zone_index;
		zinterface[i].zone2 = Get_zone(zinterface[i].zone2_name)->zone_index;
	}
	for(int i=0;i<zone_num;i++)
	  for(int j=0;j<zone[i].zone_conn_num;j++)
	    zone[i].donor_zone_array.push_back(Get_zone(zone[i].donor_zone_name[j].c_str())->zone_index);

	for(int z=0;z<zone_num;z++)
	  for(int j=0;j<zone[z].dasegment.size();j++)
	   for(int k=0;k<zinterface.interface.size();k++)
	    {
		if(zinterface.IsBelong(k,z))
	   	{
	   	  vector<int>*  p=zinterface.Get_node_array(k,z);
	   	  if(zone[z].dasegment[j].edge_num>0)
	   	    if(count(p->begin(),p->end(),zone[z].dasegment[j].edge_array[0])>0&&
	   	       count(p->begin(),p->end(),zone[z].dasegment[j].edge_array[1])>0)
	   		zone[z].dasegment[j].interface=k;
	   	}
	   }

	for(int i=0;i<zone_num;i++)
	  for(int j=0;j<zone[i].dasegment.size();j++)
	     	zone[i].dasegment[j].set_node_array();

       	printf("mesh read finished\n\n");

	cg_close(index_file);
	return 0;

}



/* ----------------------------------------------------------------------------
 * MESH::build_mesh:  an interface function to call setup_voronoicell()
 * build_voronoicell_data() and build_ghost_cell() for each zone
 */
void  MESH::build_zone()
{

	for(int i=0;i<zone_num;i++)
	{
		zone[i].setup_voronoicell();
		zone[i].build_voronoicell_data();
		zone[i].build_ghost_cell();
		//zone[i].ghost_cell_debug();
	}

	/*
	cout<<bc_num<<endl;
	for(int i=1;i<=bc_num;i++)
	{
		Segment* ps = Get_segment(i);
		if(ps)	cout<<ps->label<<endl;
	}
	*/
	/*cout<<"total interface "<<zinterface.interface.size()<<endl;
	for(int i=0;i<zinterface.interface.size();i++)
	{
	  cout<<zinterface[i].conn_name<<endl;
	  cout<<zinterface[i].zone1_name<<"  "<<zinterface.interface[i].zone2_name<<endl;
	  cout<<zinterface[i].zone1<<"  "<<zinterface.interface[i].zone2<<endl;
	  for(int j=0;j<zinterface[i].node_num;j++)
	  	cout<<zinterface[i].index_array1[j]<<"  "<<zinterface[i].index_array2[j]<<endl;
	}*/
}




/* ----------------------------------------------------------------------------
 * MESH::reorder_zone:  This function reorder the nodes in the specified zone .
 * for reduce the bandwidth of matrix.
 * danodes,datri,dasegment and correlative interface may be modified.
 * it calls setup_voronoicell() first to get adjacent information .
 * when it was done, voronoicell should be rebuilt.
 * it returns a pointer to an array which contain convertor index
 * used as new_index = old_to_new[old_index],this array must be deleted by hand
 */
int * MESH::reorder_zone(int zone_index)
{

	ZONE *pzone = &zone[zone_index];
	pzone->setup_voronoicell();
	//find the zone node which has the minimal x and y coordinates.
	int start = 0;
	for(int i=0;i<pzone->danode.size();i++)
	  if( pzone->danode[i].x<=pzone->danode[start].x && pzone->danode[i].y<=pzone->danode[start].y)
		start = i;
	  else if(pzone->danode[i].x<pzone->danode[start].x)
	  	start = i;

	//ok ,begin at this node. do Breadth-First Search
	int *af = new int[pzone->danode.size()];
	int *new_order = new int[pzone->danode.size()];
	int *old_to_new = new int[pzone->danode.size()];
	int index = 0;
	for(int i=0;i<pzone->danode.size();i++) af[i] = 0;
	queue<int> Q;
	af[start] =1; Q.push(start);
	while(!Q.empty())
	{
	   int current = Q.front();   Q.pop();
	   new_order[index] = current;	//record the order
	   old_to_new[current] = index;
	   index++;
	   for(int j=0;j<pzone->davcell[current].nb_num;j++)
	     if(!af[pzone->davcell[current].nb_array[j]])
	     {
	     	Q.push(pzone->davcell[current].nb_array[j]);
	 	af[pzone->davcell[current].nb_array[j]] = 1;
	     }
	 }
	//we have got the new order. do post process
	vector<Node>   tmpdanode = pzone->danode;
	for(int i=0;i<pzone->danode.size();i++)
	{
		pzone->danode[i] = tmpdanode[new_order[i]];
		pzone->danode[i].index = i;
	}

	//the reorder process will not change the clock wise order.
	for(int i=0;i<pzone->datri.size();i++)
	{
	  pzone->datri[i].A = old_to_new[pzone->datri[i].A];
	  pzone->datri[i].B = old_to_new[pzone->datri[i].B];
	  pzone->datri[i].C = old_to_new[pzone->datri[i].C];
	  if(pzone->datri[i].D!=-1)
	   pzone->datri[i].D = old_to_new[pzone->datri[i].D];
	}

	for(int i=0;i<pzone->dasegment.size();i++)
	{
	  for(int j=0;j<pzone->dasegment[i].edge_array.size();j++)
	  	pzone->dasegment[i].edge_array[j] = old_to_new[pzone->dasegment[i].edge_array[j]];

	  pzone->dasegment[i].set_node_array();

	  if(pzone->dasegment[i].interface>=0)
	  {
	  	int inf = pzone->dasegment[i].interface;
	  	vector<int> *pinf_node = zinterface.Get_node_array(inf,zone_index);
	  	vector<int>:: iterator pint = pinf_node->begin();
	  	for(int j=0; j< pinf_node->size(); j++,pint++)
	  	   *pint = old_to_new[*pint];
	  }
	}

	delete [] af;
	delete [] new_order;
	return old_to_new;

}


/* ----------------------------------------------------------------------------
 * MESH::reorder_mesh:  interface function to call reorder_zone
 *
 */
void   MESH::reorder_mesh()
{

	for(int i=0;i<zone_num;i++)
		delete reorder_zone(i);
}



/* ----------------------------------------------------------------------------
 * MESH::Get_zone:  This function get zone pointer by
 * zone name
 */
ZONE*  MESH::Get_zone(const char *name)
{
	for(int i=0;i<zone_num;i++)
		if(!strcmp(zone[i].zonename,name)) return &zone[i];
	return (ZONE*)NULL;
}

/* ----------------------------------------------------------------------------
 * MESH::Get_zone:  This function get zone pointer by
 * zone index
 */
ZONE*  MESH::Get_zone(const int z)
{
	for(int i=0;i<zone_num;i++)
		if(zone[i].zone_index == z) return &zone[i];
	return (ZONE*)NULL;
}

/* ----------------------------------------------------------------------------
 * MESH::Get_segment:  This function get zone segment pointer by
 * global bc_index(from 1 to bc_num)
 */
Segment* MESH::Get_segment(const int bc)
{
	for(int i=0;i<zone_num;i++)
	  for(int j=0;j<zone[i].dasegment.size();j++)
	    if(bc == zone[i].dasegment[j].bc_index)
	      	return &zone[i].dasegment[j];

	return 	(Segment*)NULL;
}
/* ----------------------------------------------------------------------------
 * MESH::zone_to_field:  This function gather local zone mesh into a global
 * mesh structrue. for adaptive mesh refinement
 */
int  MESH::zone_to_field()
{
	gnode.clear();
	gtri.clear();
	gsegment.clear();

	int total_node_num = 0;
	for(int i=0;i<zone_num;i++)
		total_node_num+=zone[i].danode.size();
	int *sign = new int [total_node_num];
	int *org  = new int [total_node_num];
	int *reorder = new int [total_node_num];
	int *p = sign;
	zinterface.Clear_flag_all();

	for(int i=0;i<zone_num;i++)
	{
	   for(int j=0;j<zone[i].danode.size();j++)
	   {
	     sign[zone[i].offset + j] = zone[i].offset + j;
	     org[zone[i].offset + j] = zone[i].offset + j;
	   }
	   for(int j=0;j<zone[i].donor_zone_array.size();j++)
	   {
	    int z2 = zone[i].donor_zone_array[j];

	    if(zinterface.Flag(i,z2)==0)
	      zinterface.Set_flag(i,z2);
	    else
	    {
	      int inf =  zinterface.Find(i,z2);
	      vector<int>* pinf_node1 = zinterface.Get_node_array(inf,i);
	      vector<int>* pinf_node2 = zinterface.Get_node_array(inf,z2);
	      for(int k=0;k<zinterface[inf].node_num;k++)
	      	   sign[zone[i].offset + pinf_node1->at(k)] = sign[zone[z2].offset + pinf_node2->at(k)];
	    }
	   }
	}

	int gnode_num = 0;
	for(int i=0;i<zone_num;i++)
	   for(int j=0;j<zone[i].danode.size();j++)
	     if(org[zone[i].offset + j]==sign[zone[i].offset + j])
	     {
	     	gnode.push_back(zone[i].danode[j]);
	   	reorder[zone[i].offset + j] = gnode_num++;
	     }

	for(int i=0;i<zone_num;i++)
	     for(int j=0;j<zone[i].datri.size();j++)
	     {
	     	Tri tmp_tri = zone[i].datri[j];
	     	tmp_tri.A = reorder[sign[zone[i].offset + tmp_tri.A]];
	     	tmp_tri.B = reorder[sign[zone[i].offset + tmp_tri.B]];
	     	tmp_tri.C = reorder[sign[zone[i].offset + tmp_tri.C]];
	     	if(tmp_tri.D!=-1)
	       	  tmp_tri.D = reorder[sign[zone[i].offset + tmp_tri.D]];
	     	gtri.push_back(tmp_tri);
	     }

	//i droped duplicated zone interface segment. so the bc_index is not continuous.
	//but it dosen't matter for enough information to build a Planar Straight Line Graph (PSLG) .
	Segment tmp_segment;
	for(int i=0;i<zone_num;i++)
	{
	     for(int j=0;j<zone[i].dasegment.size();j++)
	     {
	     	int flag=0;
	     	Segment zonesegment = zone[i].dasegment[j];
	     	for(int k=0;k<gsegment.size();k++)
	   		if(!strcmp(gsegment[k].label,zonesegment.label))  flag=1;
	   	if(flag==1) continue;	//if two zone segmants have same name, skip one

	     	tmp_segment.bc_index = zonesegment.bc_index;
	     	tmp_segment.zone_index = zonesegment.zone_index;
	     	tmp_segment.interface = zonesegment.interface;
	   	strcpy(tmp_segment.label,zonesegment.label);
	   	tmp_segment.edge_num = zonesegment.edge_num;

	   	for(int k=0;k<2*tmp_segment.edge_num;k++)
	   	{
	   		int index = reorder[sign[zone[i].offset+zonesegment.edge_array[k]]];
	         	tmp_segment.edge_array.push_back(index);

	        }
	       	gsegment.push_back(tmp_segment);
	       	tmp_segment.edge_array.clear();
	     }

	}

	for(int i=0;i<gsegment.size();i++)
	     	gsegment[i].set_node_array();


	delete [] sign;
	delete [] org;
	delete [] reorder;
	return 0;
}

/* ----------------------------------------------------------------------------
 * MESH::field_to_zone:  This function build local zone mesh from  global
 * mesh structrue.
 */
int  MESH::field_to_zone()
{

  //the nodes will be classified by its zone index and insert to each zone
  int **local_index = new int*[zone_num];     //zone_index = local_index[reg][global_index]
  int *conn_nodes = new int[zone_num];          //for zone to zone connectivity node num
  int **conn = new int*[zone_num];            //for zone to zone connectivity information
  int *zonenodes = new int[zone_num];

  for(int i=0; i<zone_num; i++)
  {
  	local_index[i] = new int[gnode.size()];
  	conn[i] = new int[gnode.size()];
  }

  int *af = new int[gnode.size()];

  for(int i=0;i<zone_num;i++)
  {
  	for(int j = 0; j < gnode.size(); j++) local_index[i][j] = -1; //-1 means node not in this zone
  	int *pf = af;
        for(int j = 0; j < gnode.size(); j++) *pf++ = 0;        //visited flag array
        zonenodes[i] = 0;
  	conn_nodes[i] = 0;

	for(int j = 0; j < gtri.size(); j++)     //search for all the nodes
	  if (gtri[j].zone_index == i)
	  {

	  	if (!af[gtri[j].A]) { af[gtri[j].A] = 1; local_index[i][gtri[j].A] = zonenodes[i]++; }
	  	if (!af[gtri[j].B]) { af[gtri[j].B] = 1; local_index[i][gtri[j].B] = zonenodes[i]++; }
	  	if (!af[gtri[j].C]) { af[gtri[j].C] = 1; local_index[i][gtri[j].C] = zonenodes[i]++; }
	  	if (gtri[j].D!= -1)
	  		if (!af[gtri[j].D]) { af[gtri[j].D] = 1;local_index[i][gtri[j].D] = zonenodes[i]++; }
	  }
  }
  delete [] af;

  int node_offset = 0;
  for(int i=0; i<zone_num; node_offset+=zonenodes[i],i++)
  {

  	zone[i].danode.resize(zonenodes[i]);
  	for(int j=0;j<gnode.size();j++)
    	  if(local_index[i][j]>-1)  //process node in this zone
  	  {
  	  	gnode[j].index = local_index[i][j];
  	  	gnode[j].zone_index = i;
  	  	zone[i].danode[local_index[i][j]] = gnode[j];
  	  }

	zone[i].offset = node_offset;

	zone[i].datri.clear();
  	for(int j=0;j<gtri.size();j++)
     	 if (gtri[j].zone_index == i)  //is the triangle belongs to this zone?
   	 {
   	 	Tri tmp_tri = gtri[j];

   	 	if(gtri[j].D==-1)
   		  {
   		  	tmp_tri.A = local_index[i][gtri[j].A];
           	  	tmp_tri.B = local_index[i][gtri[j].B];
           	  	tmp_tri.C = local_index[i][gtri[j].C];
           	  }
           	  else
           	  {
           	    	tmp_tri.A = local_index[i][gtri[j].A];
           	  	tmp_tri.B = local_index[i][gtri[j].B];
           	  	tmp_tri.C = local_index[i][gtri[j].C];
           	  	tmp_tri.D = local_index[i][gtri[j].D];
           	  }
   		zone[i].datri.push_back(tmp_tri);
   	}


   	//process boundary segment here
   	for(int j=0;j<zone[i].dasegment.size();j++)
   	{
   		zone[i].dasegment[j].edge_num = 0;
   		zone[i].dasegment[j].edge_array.clear();
   	}

   	for(int j=0;j<zone[i].dasegment.size();j++)
   	  for(int k=0;k<gsegment.size();k++)
   	   if(!strcmp(zone[i].dasegment[j].label,gsegment[k].label))
      	   {
   		zone[i].dasegment[j].edge_num = gsegment[k].edge_num;
   		for(int h=0; h<	gsegment[k].edge_array.size();h++)
   		{
   			int inedx = local_index[i][gsegment[k].edge_array[h]];
   			zone[i].dasegment[j].edge_array.push_back(inedx);
   		}
   	  }

	for(int j=0;j<zone[i].dasegment.size();j++)
	     	zone[i].dasegment[j].set_node_array();

   	//set zone interface
   	if(zone_num>1)
  	{	//build connectivity data here
  		for(int k=0; k<zone_num; k++)
  		  for(int j=0; j<gnode.size(); j++)
  	  	{
  	   		if(k==i) continue;
  	   		if(local_index[k][j]!=-1 &&local_index[i][j]!=-1)
  	   		{
  	   		   conn[k][conn_nodes[k]++]=local_index[k][j];
  	   		   conn[i][conn_nodes[i]++]=local_index[i][j];
  	   		}
  	   	}
  	}

        int  *conn_i=conn[i];
        zinterface.Clear_flag_all();
        for(int j=0; j<zone_num; j++)
          if(j!=i&&conn_nodes[j]>0&&zinterface.Flag(i,j)==0)
     	  {
       	 	int inf = zinterface.Find(i,j);
       	      	zinterface.Set_flag(i,j);
       	 	int  *conn_j=conn[j];
       	 	zinterface[inf].clear();
       	 	zinterface[inf].node_num = conn_nodes[j];
       	 	vector<int> *pinfi =  zinterface.Get_node_array(inf,i);
       	 	vector<int> *pinfj =  zinterface.Get_node_array(inf,j);
       	 	for(int k=0;k<conn_nodes[j];k++)
       	 	{
       	 		pinfi->push_back(*conn_i++);
       	 		pinfj->push_back(*conn_j++);
       	 	}

	  }

  	for(int j=0; j<zone_num; j++)  	conn_nodes[j]=0;


  }

  /* all things are done */
  for(int i =0; i<zone_num; i++)
  {
     	delete [] local_index[i];
  	delete [] conn[i];
  }
  delete  [] local_index;
  delete  [] conn;
  delete  [] conn_nodes;
  delete  [] zonenodes ;
  return 0;

}

/* ----------------------------------------------------------------------------
 * MESH::export_cgns:  This function write mesh to cgns file.
 */
void MESH::export_mesh(const char *cgnsfile)
{
  int fn,B,Z,C,S,BC,I,SOL,F;
  int size[3];
  double *x,*y;
  int *elem;
  char bcname[32];

  /* remove old file if exist*/
  remove(cgnsfile);

  /* open CGNS file for write */
  cg_open(cgnsfile,MODE_WRITE,&fn);

  /* create base (can give any name) */
  cg_base_write(fn,"GSS_CoreFile",2,2,&B); /*two dimension*/

  /* create zone */
  for(int i=0;i<zone_num;i++)
  {

  	size[0] = zone[i].danode.size();
  	size[1] = zone[i].datri.size();
  	size[2] = 0;

  	x = new double[size[0]];
  	y = new double[size[0]];
  	elem= new int[5*size[1]];
  	//set zone
  	cg_zone_write(fn,B,zone[i].zonename,size,Unstructured,&Z);
  	cg_goto(fn,B,"Zone_t",Z,"end");
  	cg_descriptor_write("RegionType",zone[i].zonelabel);

  	// write grid coordinates
       	for(int j=0;j<zone[i].danode.size();j++)
    	{
  		//convert the unit
  		x[j] = zone[i].danode[j].x/mesh_scale;
        	y[j] = zone[i].danode[j].y/mesh_scale;
       	}

  	cg_coord_write(fn,B,Z,RealDouble,"CoordinateX",x,&C);
        cg_coord_write(fn,B,Z,RealDouble,"CoordinateY",y,&C);

        // set element connectivity here
  	int *pelem=elem;


  	for(int j = 0; j < zone[i].datri.size(); j++)
     	{
   		  if(zone[i].datri[j].D==-1)
   		  {
   		  	*pelem++ = TRI_3;
           	  	*pelem++ = zone[i].datri[j].A+1;
           	  	*pelem++ = zone[i].datri[j].B+1;
           	  	*pelem++ = zone[i].datri[j].C+1;
           	  }
           	  else
           	  {
           	  	*pelem++ = QUAD_4;
           	  	*pelem++ = zone[i].datri[j].A+1;
           	  	*pelem++ = zone[i].datri[j].B+1;
           	  	*pelem++ = zone[i].datri[j].C+1;
           	  	*pelem++ = zone[i].datri[j].D+1;
           	  }
        }

  	cg_section_write(fn,B,Z,"GridElements",MIXED,1,size[1],0,elem,&S);

 	delete [] x;
  	delete [] y;
  	delete [] elem;

    	// write boundary segment

   	int *asegment = new int[2*zone[i].danode.size()];
	int  segrange[2];
	segrange[0] = size[1]+1;
	int  csegment;
       	for(int j=0;j<zone[i].dasegment.size();j++)
    	{
       	    csegment = 0;
       	    int *psegment = asegment;

       	    for(int k=0; k<zone[i].dasegment[j].edge_num; k++)
  	    {
  	  	*psegment++ = zone[i].dasegment[j].edge_array[2*k]+1;
	  	*psegment++ = zone[i].dasegment[j].edge_array[2*k+1]+1;
	   	 csegment++;
	    }

	    if(csegment>0)
	    {
		  segrange[1] = segrange[0] + csegment - 1;
		  //cerr<<segrange[0]<<"  "<<segrange[1]<<endl;
		  cg_section_write(fn,B,Z,zone[i].dasegment[j].label,BAR_2,segrange[0],segrange[1],0,asegment,&S);
                  cg_boco_write(fn,B,Z,zone[i].dasegment[j].label,BCTypeUserDefined,ElementRange,2,segrange,&BC);
                  segrange[0] = segrange[1]+1;
	    }

	}
	delete [] asegment;

	//write zone to zone connectivity data if necessary
   	if(zone_num>1)
  	{	//build connectivity data here
  		int inf;
  		char conn_name[32];
  		for(int j=0; j<zone_num; j++)
  		 if( (inf = zinterface.Find(i,j)) > -1)
  		 {
  		  	vector<int> *p1 = zinterface.Get_node_array(inf,i);
  		  	vector<int> *p2 = zinterface.Get_node_array(inf,j);
  		 	int *conn_local = new int[p1->size()];
  		 	int *conn_donor = new int[p2->size()];
  		 	for(int k=0; k<p1->size();k++)
  		 	{
  		 		conn_local[k]=p1->at(k)+1;
  		 		conn_donor[k]=p2->at(k)+1;
  		 	}
  		 	if(strcmp(zone[i].zonename,zone[j].zonename)>0)
     	  	 		sprintf(conn_name,"%s_to_%s",zone[i].zonename,zone[j].zonename);
     	  	 	else
     	  	 		sprintf(conn_name,"%s_to_%s",zone[j].zonename,zone[i].zonename);
     	  	 	cg_conn_write(fn,B,Z,conn_name,Vertex, Abutting1to1,
				PointList, p1->size(), conn_local,zone[j].zonename,Unstructured,
				PointListDonor,Integer,p2->size(),conn_donor ,&I);
			delete 	conn_local;
			delete 	conn_donor;
		}

  	}

  }


  // close CGNS file
  cg_close(fn);
}

/* ----------------------------------------------------------------------------
 * MESH::clear_mesh:  This function delete mesh structure.
 */
void	MESH::clear_mesh()
{
	gnode.clear();
	gtri.clear();
	gsegment.clear();
	gnode_old.clear();
	gtri_old.clear();
	gsegment_old.clear();
	delete [] zone;
	zinterface.interface.clear();
	bc_num = 0;
	zone_num =0;
	zone = 0;
}

