#include <math.h>
#include "material.h"

Material::Material(int material,double mesh_scale)
:k(2.582e-2),e(1.0)
{
	s_length = mesh_scale; 
	s_time   = 1e12;      //s->ps
	s_temp   = 1.0/3e2;  //K->1/300K
	
	me = 5.6834e8/mesh_scale/mesh_scale;
	eps0 = 8.85e-12*0.624e19/1e2/mesh_scale;
	
	if(material==Si)
	{
		Eg300 = 1.08;  EGALPH = 4.73e-4/s_temp;  EGBETA = 636*s_temp;
		
		NC300 = 2.80e19/pow(s_length,3);	NC_F = 1.5;
		NV300 = 1.04e19/pow(s_length,3);	NV_F = 1.5;
		
		TAUN0 = 1e-7*s_time; NSRHN = 5e16/pow(s_length,3); EXN_TAU=0;
		TAUP0 = 1e-7*s_time; NSRHP = 5e16/pow(s_length,3); EXP_TAU=0;  
		
		ETRAP = 0; C_DIRECT = 0;
		AUGN = 2.8e-31*pow(s_length,6)/s_time; AUGP=9.9e-32*pow(s_length,6)/s_time; 
		
		MUN_MIN = 55.24*pow(s_length,2)/s_time; MUN_MAX = 1429.23*pow(s_length,2)/s_time;
		NUN = -2.3; XIN = -3.8; NREFN = 1.072E17/pow(s_length,3); ALPHAN = 0.73;
				
		MUP_MIN = 49.70*pow(s_length,2)/s_time; MUP_MAX = 479.37*pow(s_length,2)/s_time;
		NUP = -2.2; XIP = -3.7; NREFP = 1.606E17/pow(s_length,3); ALPHAP= 0.70;
		
		EleMessConstant = 0.26*me; HoleMessConstant = 0.81*me;
		
		ELE_TAUW = 2e-13*s_time; HOL_TAUW = ELE_TAUW;
		
		eps = eps0*11.9;
	}
	
	if(material==GaAs)
	{
		Eg300 = 1.424; EGALPH = 5.405e-4/s_temp; EGBETA = 204*s_temp;
		
		NC300 = 4.7e17/pow(s_length,3);	NC_F = 1.5;
		NV300 = 7.0e18/pow(s_length,3);	NV_F = 1.5;
		
		TAUN0 = 1e-9*s_time; NSRHN = 5e22/pow(s_length,3); EXN_TAU=0;
		TAUP0 = 1e-9*s_time; NSRHP = 5e22/pow(s_length,3); EXP_TAU=0;
		
		ETRAP = 0; AUGN = 0; AUGP=0; C_DIRECT = 0; 
		
		MUN_MIN = 0; MUN_MAX = 8500*pow(s_length,2)/s_time; NUN = -1.0; 
		XIN = 0; NREFN = 1.69E17/pow(s_length,3); ALPHAN = 0.436;
				
		MUP_MIN = 0; MUP_MAX = 400*pow(s_length,2)/s_time; NUP = -2.1;
		XIP = 0; NREFP = 2.75E17/pow(s_length,3); ALPHAP= 0.395;
		
		EleMessConstant = 0.063*me; HoleMessConstant = 0.53*me;
		
		ELE_TAUW = 1e-12*s_time; HOL_TAUW = ELE_TAUW;
		
		eps=eps0*13.2;
	}

	if(material==Ge)
	{
		Eg300 = 0.66;	EGALPH = 4.77e-4/s_temp;  EGBETA = 235*s_temp;
		
		NC300 = 1.04e19/pow(s_length,3);	NC_F = 1.5;
		NV300 = 6.0e18/pow(s_length,3);		NV_F = 1.5;
		
		TAUN0 = 1e-7*s_time; NSRHN = 5e16/pow(s_length,3); EXN_TAU=0;
		TAUP0 = 1e-7*s_time; NSRHP = 5e16/pow(s_length,3); EXP_TAU=0;  
		
		ETRAP = 0; C_DIRECT = 0;
		AUGN = 2.8e-31*pow(s_length,6)/s_time; AUGP=9.9e-32*pow(s_length,6)/s_time; 
		
		MUN_MIN = 55.24*pow(s_length,2)/s_time; MUN_MAX = 1429.23*pow(s_length,2)/s_time; NUN = -2.3; 
		XIN = -3.8; NREFN = 1.072E17/pow(s_length,3); ALPHAN = 0.73;
				
		MUP_MIN = 49.70*pow(s_length,2)/s_time; MUP_MAX = 479.37*pow(s_length,2)/s_time; NUP = -2.2;
		XIP = -3.7; NREFP = 1.606E17/pow(s_length,3); ALPHAP= 0.70;
		
		EleMessConstant = 0.12*me; HoleMessConstant = 0.34*me;
		
		ELE_TAUW = 2e-13*s_time; HOL_TAUW = ELE_TAUW;
		
		eps=eps0*16.0;
	}
	
	if(material==SiO2) eps = eps0*3.9;
	
	eps_SiO2 = eps0*3.9;
	
}	

