#include <stdio.h>
#include <string.h>
#include <algorithm>
#include "bc.h"
#include "parser.tab.h"


int DABC::InitBC(int bc_num,Segment** psegment_array,list<CmdParseBuf> &cmdlist,double scale)
{
	list<CmdParseBuf>::iterator  pcmd;
 	list<ArgStruct>::iterator    parg;
	bc_array.resize(bc_num);
	mesh_scale = scale;
	//do some init here
	for(int i=0;i<bc_num;i++)
	{
		//set psegment pointer 
		bc_array[i].psegment = psegment_array[i];
		//set default BC type -- NeumannBoundary
		bc_array[i].BCType = NeumannBoundary;
		bc_array[i].Kapa =0;	
		bc_array[i].V =0;
		bc_array[i].I = 0;
		bc_array[i].Ic = 0; 
		bc_array[i].R = 0;
		bc_array[i].L = 0;
		bc_array[i].C = 0;         	
	}	
	  	
	
	printf("\n--------------------------------------------------\n");
   	printf("\ninit each boundary condition...\n"); 
  	
 	//set bc parameters, attach vsource to bc    	
 	for(pcmd=cmdlist.begin();pcmd!=cmdlist.end();pcmd++)   	 								
 	{
 		if(!strcmp(pcmd->KeyWord,"boundary"))
 		  for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	  	   if(!strcmp(parg->label,"Type"))
 		   {
 				
 			if(!strcmp(parg->argvalue.sval,"NeumannBoundary"))
 				if(SetBCNeumannBoundary(pcmd)) return 1;
 			if(!strcmp(parg->argvalue.sval,"OhmicContract"))
 				if(SetBCOhmicContract(pcmd)) return 1;
 			if(!strcmp(parg->argvalue.sval,"InsulatorInterface"))
 				if(SetBCInsulatorInterface(pcmd)) return 1;
 			if(!strcmp(parg->argvalue.sval,"SchottkyContract"))	   
 				if(SetBCSchottkyContract(pcmd)) return 1;
 			if(!strcmp(parg->argvalue.sval,"GateContract"))	   
 				if(SetBCGateContract(pcmd)) return 1;
 			if(!strcmp(parg->argvalue.sval,"InsulatorContract"))	   
 				if(SetBCInsulatorContract(pcmd)) return 1;			
 		  }
	}   
	printf("\n");
		
	return 0; 

}


int DABC::SetBCNeumannBoundary(list<CmdParseBuf>::iterator pcmd)
{
	int flag = 0;
	list<ArgStruct>::iterator  parg;
	
	char *Identifier;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	  if(!strcmp(parg->label,"ID"))
	        Identifier=parg->argvalue.sval;
	
	for(int i=0;i<bc_array.size();i++)
	{	
		if(!strcmp(bc_array[i].psegment->label,Identifier))	
		{
			flag = 1;
			printf("\nBC[%d] label \"%s\" init as NeumannBoundary...done\n",i+1,Identifier);
			bc_array[i].BCType = NeumannBoundary;
			for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	        		if(!strcmp(parg->label,"Kapa"))
				        bc_array[i].Kapa=parg->argvalue.dval;
		}
	}
	
	if(!flag)
	{
		printf("error: BC label \"%s\" not matched\n",Identifier);
		return 1;
	}
	return 0;
}				
	
int DABC::SetBCOhmicContract(list<CmdParseBuf>::iterator pcmd)
{
	int flag = 0;
	list<ArgStruct>::iterator  parg;
	
	char *Identifier;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	  if(!strcmp(parg->label,"ID"))
	        Identifier=parg->argvalue.sval;
	        
	for(int i=0;i<bc_array.size();i++)
	{	
		if(!strcmp(bc_array[i].psegment->label,Identifier))	
		{
			flag = 1;
			printf("\nBC[%d] label \"%s\" init as OhmicContract...done\n",i+1,Identifier);
			bc_array[i].BCType = OhmicContract;
			for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
			{
	        		if(!strcmp(parg->label,"Res"))
				        bc_array[i].R=parg->argvalue.dval*1.6e-7;
				if(!strcmp(parg->label,"Cap"))
				        bc_array[i].C=parg->argvalue.dval*0.624e19;
				if(!strcmp(parg->label,"Ind"))
				        bc_array[i].L=parg->argvalue.dval*1.6e5;
			}
		}
		
		
	}
	
	if(!flag){
		printf("error: BC label \"%s\" not matched\n",Identifier);
		return 1;
	}
		
	return 0;
				
}		

int DABC::SetBCGateContract(list<CmdParseBuf>::iterator pcmd)
{
	int flag = 0;
	list<ArgStruct>::iterator  parg;
	
	char *Identifier;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	  if(!strcmp(parg->label,"ID"))
	        Identifier=parg->argvalue.sval;
	        
	for(int i=0;i<bc_array.size();i++)
	{	
		if(!strcmp(bc_array[i].psegment->label,Identifier))	
		{
			flag = 1;
			printf("\nBC[%d] label \"%s\" init as GateContract...done\n",i+1,Identifier);
			bc_array[i].BCType = GateContract;
			for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
			{
	        		if(!strcmp(parg->label,"Res"))
				        bc_array[i].R=parg->argvalue.dval;
				if(!strcmp(parg->label,"Cap"))
				        bc_array[i].C=parg->argvalue.dval;
				if(!strcmp(parg->label,"Ind"))
				        bc_array[i].L=parg->argvalue.dval;
				if(!strcmp(parg->label,"WorkFunction"))
				        bc_array[i].WorkFunction=parg->argvalue.dval;        
			}
		}

	}
	
	if(!flag){
		printf("error: BC label \"%s\" not matched\n",Identifier);
		return 1;
	}
	return 0;
				
}			

int DABC::SetBCSchottkyContract(list<CmdParseBuf>::iterator pcmd)
{
	int flag = 0;
	list<ArgStruct>::iterator  parg;
	
	char *Identifier;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	  if(!strcmp(parg->label,"ID"))
	        Identifier=parg->argvalue.sval;
	        
	for(int i=0;i<bc_array.size();i++)
	{	
		if(!strcmp(bc_array[i].psegment->label,Identifier))	
		{
			flag = 1;
			printf("BC[%d] label \"%s\" init as Schottky...\n",i+1,Identifier);
			bc_array[i].BCType = SchottkyContract;
			for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
			{
	        		if(!strcmp(parg->label,"Res"))
				        bc_array[i].R=parg->argvalue.dval;
				if(!strcmp(parg->label,"Cap"))
				        bc_array[i].C=parg->argvalue.dval;
				if(!strcmp(parg->label,"Ind"))
				        bc_array[i].L=parg->argvalue.dval;
				if(!strcmp(parg->label,"Vbarrier"))
				        bc_array[i].Vbarrier=parg->argvalue.dval;        
			}
		
		}
		
		
	}
	
	if(!flag){
		printf("error: BC label \"%s\" not matched\n",Identifier);
		return 1;
	}
	return 0;

}

int DABC::SetBCInsulatorInterface(list<CmdParseBuf>::iterator pcmd) 
{
	int flag = 0;
	list<ArgStruct>::iterator  parg;
	
	char *Identifier;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	  if(!strcmp(parg->label,"ID"))
	        Identifier=parg->argvalue.sval;
	        
	for(int i=0;i<bc_array.size();i++)
	{	
		if(!strcmp(bc_array[i].psegment->label,Identifier))	
		{
			flag = 1;
			printf("\nBC[%d] label \"%s\" init as InsulatorInterface...done\n",i+1,Identifier);
			bc_array[i].BCType = InsulatorInterface;
			for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	        		if(!strcmp(parg->label,"QF"))
				        bc_array[i].QF=parg->argvalue.dval;
		
		}
	}
	
	if(!flag)
	{
		printf("error: BC label \"%s\" not matched\n",Identifier);
		return 1;
	}
	return 0;
}

int DABC::SetBCInsulatorContract(list<CmdParseBuf>::iterator pcmd)
{
	int flag = 0;
	list<ArgStruct>::iterator  parg;
	
	char *Identifier;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	  if(!strcmp(parg->label,"ID"))
	        Identifier=parg->argvalue.sval;
	        
	for(int i=0;i<bc_array.size();i++)
	{	
		if(!strcmp(bc_array[i].psegment->label,Identifier))	
		{
			flag = 1;
			printf("\nBC[%d] label \"%s\" init as InsulatorContract...\n",i+1,Identifier);
			bc_array[i].BCType = InsulatorContract;
			for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
			{
	        		if(!strcmp(parg->label,"Res"))
				        bc_array[i].R=parg->argvalue.dval;
				if(!strcmp(parg->label,"Cap"))
				        bc_array[i].C=parg->argvalue.dval;
				if(!strcmp(parg->label,"Ind"))
				        bc_array[i].L=parg->argvalue.dval;
				if(!strcmp(parg->label,"QF"))
				        bc_array[i].QF=parg->argvalue.dval;
				if(!strcmp(parg->label,"Thick"))
				        bc_array[i].Thick=parg->argvalue.dval*mesh_scale;  //scale it              
			}
			
		}

	}
	
	if(!flag){
		printf("error: BC label \"%s\" not matched\n",Identifier);
		return 1;
	}
		
	return 0;
				
}		
