#include "ctrl.h"
#include "xgraph.h"
#include "grafix3d.h"

int  SolveControl::init_solver()
{

	Nt_DDM_Solver::init_solver();
	Ex_HDM_Solver::init_solver();
	Im_HDM_Solver::init_solver();
	return 0;
}

int  SolveControl::destroy_solver()
{

	Nt_DDM_Solver::destroy_solver();
	Ex_HDM_Solver::destroy_solver();
	Im_HDM_Solver::destroy_solver();
	return 0;
}

int  SolveControl::run_control(list<CmdParseBuf>& cmdlist)
{
	list<CmdParseBuf>::iterator  pcmd;
 	list<ArgStruct>::iterator    parg;

 	//init mesh first
	//set_mesh_scale(1e6); //scale mesh
	if(import_cgns(MeshFile)) return 1;
	build_zone();
	zone_to_field();
	if(setup_bc(cmdlist))   return 1;
 	if(build_zonedata())    return 1;
 	if(setup_init_data(MeshFile,LatticeTemp)) return 1;
 	//reorder the mesh, important for implicit solver
	reorder();

 	solve_define.Solver = DDM;           //the default solver
	solve_define.Scheme = DDM_Newton;

	//search the CmdBuf
 	for(pcmd=cmdlist.begin();pcmd!=cmdlist.end();pcmd++)
 	{
 	   if(!strcmp(pcmd->KeyWord,"METHOD"))
 	   {
 	   	if(set_method(pcmd)) return 1;
 	   	if(do_method()) return 1;
 	   }

 	   if(!strcmp(pcmd->KeyWord,"SOLVE"))
 	   {
 	   	if(set_solve(pcmd)) return 1;
 	   	if(do_solve()) return 1;
 	   }

 	   if(!strcmp(pcmd->KeyWord,"EXTRACT"))
 	   {
 	   	if(set_extract(pcmd)) return 1;
 	   	if(do_extract()) return 1;
 	   }

 	   if(!strcmp(pcmd->KeyWord,"IMPORT"))
 	   {
 	   	if(set_import(pcmd)) return 1;
 	   	if(do_import()) return 1;
 	   }

 	   if(!strcmp(pcmd->KeyWord,"ATTACH"))
 	   {
 	   	if(set_attach(pcmd)) return 1;
 	   	if(do_attach()) return 1;
 	   }
 	   if(!strcmp(pcmd->KeyWord,"REFINE"))
 	   {
 	   	if(set_refine(pcmd)) return 1;
 	   	if(do_refine()) return 1;
 	   }

 	   if(!strcmp(pcmd->KeyWord,"PLOT"))
 	   {
 	   	if(set_plot(pcmd)) return 1;
 	   	if(do_plot()) return 1;
 	   }

 	}

 	//
 	//------------------------------------------------------------
   	return 0;
}


int  SolveControl::do_method()
{
	return 0;
}

int SolveControl::do_solve()
{
	init_solver();
	if(solve_define.Scheme == DDM_Newton)   Nt_DDM_Solver::do_solve(solve_define);
	if(solve_define.Scheme == HDM_explicit) Ex_HDM_Solver::do_solve(solve_define);
	if(solve_define.Scheme == HDM_implicit) Im_HDM_Solver::do_solve(solve_define);
	destroy_solver();
	return 0;
}

int  SolveControl::do_refine()
{
	printf("Refine mesh...\n");
	//delete old solution data
	bc.clear();
	BSolver::refine(refine_define);
	reorder();
 	return 0;
}

int SolveControl::do_extract()
{
	if(extract_define.core)
	{	printf("Extract CoreData to %s...",extract_define.CoreFile);
		//export mesh
		export_mesh(extract_define.CoreFile);
		//export data
		for(int z=0;z<zone_num;z++)
	  	if(zonedata[z]->material_type==semiconductor)
   	  	{
   	  		SMCZone *pzonedata = dynamic_cast< SMCZone * >(zonedata[z]);
   	   		pzonedata->export_doping(extract_define.CoreFile,mesh_scale);
   			pzonedata->export_solution(extract_define.CoreFile,"solution",mesh_scale);
   	  	}
   	  	else if(zonedata[z]->material_type==Insulator)
   	  	{
   	  		ISZone *pzonedata = dynamic_cast< ISZone * >(zonedata[z]);
   	   		pzonedata->export_solution(extract_define.CoreFile,"solution",mesh_scale);
   	  	}
   	  	else if(zonedata[z]->material_type==metal)
   	  	{
   	  		ElZone *pzonedata = dynamic_cast< ElZone * >(zonedata[z]);
   	   		pzonedata->export_solution(extract_define.CoreFile,"solution",mesh_scale);
   	 	 }
   	  	printf("done\n\n");
   	 }
   	 if(extract_define.ascii)
   	 {
   	 	printf("Extract Data to Ascii File %s...",extract_define.AscFile);
   	   	FILE *fp=fopen(extract_define.AscFile,"w");
   	   	fprintf(fp,"# export data as: x y Nd Na n p Pontential \n");
   		for(int z=0;z<zone_num;z++)
		{
	  	  BZoneData *pzonedata = zonedata[z];
	  	  if(zonedata[z]->material_type == semiconductor)
	  	  	fprintf(fp,"# semiconductor zone\n");
	  	  if(zonedata[z]->material_type == Insulator)
	  	  	fprintf(fp,"# Insulator zone\n");
	  	  if(zonedata[z]->material_type == metal)
	  	  	fprintf(fp,"# Electrode zone\n");
   	  	  for(int i=0; i<pzonedata->node_num;i++)
            		fprintf(fp,"%e\t%e\t%e\t%e\t%e\t%e\t%e\n",
        	  		pzonedata->pzone->danode[i].x/mesh_scale,
        	  		pzonedata->pzone->danode[i].y/mesh_scale,
        	  		pzonedata->fs[i].Nd*pow(mesh_scale,3),
        	  		pzonedata->fs[i].Na*pow(mesh_scale,3),
        	  		pzonedata->fs[i].n*pow(mesh_scale,3),
        	  		pzonedata->fs[i].p*pow(mesh_scale,3),
        	  		pzonedata->fs[i].P);
         	}
        	fclose(fp);
        	printf("done\n\n");
   	 }
   	 return 0;

}

int  SolveControl::do_import()
{
	printf("Import CoreData from %s...\n",import_define.CoreFile);
	//delete old solution data
	bc.clear();
	clear_data();
	clear_mesh();

	//init new solver
	if(import_cgns(import_define.CoreFile)) return 1;
	build_zone();
	zone_to_field();
	if(setup_bc(cmdlist))   return 1;
 	if(build_zonedata())    return 1;

	//import data
	for(int z=0;z<zone_num;z++)
	  if(zonedata[z]->material_type==semiconductor)
   	  {
   	  	SMCZone *pzonedata = dynamic_cast< SMCZone * >(zonedata[z]);
   	   	if(pzonedata->import_doping(import_define.CoreFile,mesh_scale)) return 1;
   		if(pzonedata->import_solution(import_define.CoreFile,"solution",mesh_scale)) return 1;
   	  }
   	  else if(zonedata[z]->material_type==Insulator)
   	  {
   	  	ISZone *pzonedata = dynamic_cast< ISZone * >(zonedata[z]);
   	   	if(pzonedata->import_solution(import_define.CoreFile,"solution",mesh_scale)) return 1;
   	  }
   	  else if(zonedata[z]->material_type==metal)
   	  {
   	  	ElZone *pzonedata = dynamic_cast< ElZone * >(zonedata[z]);
   	   	if(pzonedata->import_solution(import_define.CoreFile,"solution",mesh_scale)) return 1;
   	  }

   	 //reorder the mesh, important for implicit solver
	 reorder();
 	 printf("\n");
      	 return 0;
}

int  SolveControl::do_attach()
{
	bc[attach_define.bc_index].vsrc.clear();
	for(int i=0;i<attach_define.vsrc_index.size();i++)
	{
		int vsrc_index=attach_define.vsrc_index[i];
		bc[attach_define.bc_index].vsrc.push_back(vsrc[vsrc_index]);
	 	printf("Vsource %s attached to bc %s\n",vsrc[vsrc_index]->label,attach_define.Electrode);
	}
	return 0;
}

int  SolveControl::do_plot()
{
	if(plot_define.MeshOnly)
	{
		if(plot_define.GeneratePSFile)
			draw_mesh_ps(plot_define.PSFileName);
		else
			draw_mesh_screen(&plot_define);
	}
	else
		plot_data(plot_define);
	return 0;
}

int  SolveControl::set_method(list<CmdParseBuf>::iterator  pcmd)
{
	//do clear
	solve_define.Solver = 0;
	solve_define.Scheme = 0;
	solve_define.FluxFunc  = AUSM;
        solve_define.SNES = LineSearchCubic;

	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Type"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d METHOD: Type syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"DDM"))
				solve_define.Solver = DDM;
			else if(!strcmp(parg->argvalue.sval,"HDM"))
				solve_define.Solver = HDM;
			else
				{printf("line %d METHOD: Type syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"Scheme"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d METHOD: Scheme syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"Newton"))
				solve_define.Scheme = DDM_Newton;
			else if(!strcmp(parg->argvalue.sval,"Gummel"))
				solve_define.Scheme = DDM_Gummel;
			else if(!strcmp(parg->argvalue.sval,"Explicit"))
				solve_define.Scheme = HDM_explicit;
			else if(!strcmp(parg->argvalue.sval,"Implicit"))
				solve_define.Scheme = HDM_implicit;
			else
				{printf("line %d METHOD: Scheme syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"SNES"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d METHOD: SNES syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"LineSearchCubic"))
				solve_define.SNES = LineSearchCubic;
			else if(!strcmp(parg->argvalue.sval,"LineSearchQuadratic"))
				solve_define.SNES = LineSearchQuadratic;
			else if(!strcmp(parg->argvalue.sval,"LineSearchNo"))
				solve_define.SNES = LineSearchNo;
			else if(!strcmp(parg->argvalue.sval,"TrustRegion"))
				solve_define.SNES = TrustRegion;
			else
				{printf("line %d METHOD: SNES syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"TStep"))
		{
			if(parg->argtype==DOUBLE)
				solve_define.TStep = parg->argvalue.dval;
			else {printf("line %d METHOD: TStep syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"CFL"))
		{
			if(parg->argtype==DOUBLE)
				solve_define.CFL = parg->argvalue.dval;
			else {printf("line %d METHOD: CFL syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"FluxFunc"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d METHOD: FluxFunc syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"AUSM"))
				solve_define.FluxFunc = AUSM;
			else if(!strcmp(parg->argvalue.sval,"Roe"))
				solve_define.FluxFunc = Roe;
			else if(!strcmp(parg->argvalue.sval,"VanLeer"))
				solve_define.FluxFunc = VanLeer;
			else
				{printf("line %d METHOD: FluxFunc syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"Reconstruct"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d METHOD: Reconstruct syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"FirstOrder"))
				solve_define.Reconstruct = FirstOrder;
			else if(!strcmp(parg->argvalue.sval,"SecondOrder"))
				solve_define.Reconstruct = SecondOrder;
			else
				{printf("line %d METHOD: Reconstruct syntax error!\n",pcmd->lineno);return 1;}
		}
		else {printf("line %d METHOD: %s no such command!\n",pcmd->lineno,parg->label);return 1;}
	}
	switch(solve_define.Solver)
	{
		case HDM: printf("Solver      : HDM\n");break;
		case DDM: printf("Solver      : DDM\n");break;
	}
	switch(solve_define.Scheme)
	{
		case DDM_Newton:   printf("DDM scheme  : Newton\n");break;
		case HDM_explicit: printf("HDM scheme  : explicit\n");break;
		case HDM_implicit: printf("HDM scheme  : implicit\n");break;
	}
	switch(solve_define.SNES)
	{
		case LineSearchCubic:       printf("Newton method  : LineSearchCubic\n");break;
		case LineSearchQuadratic:   printf("Newton method  : LineSearchQuadratic\n");break;
		case LineSearchNo:          printf("Newton method  : LineSearchNo\n");break;
		case TrustRegion:           printf("Newton method  : TrustRegion\n");break;
	}
	switch(solve_define.FluxFunc)
	{
		case VanLeer: printf("Flux  type  : VanLeer\n");break;
		case Roe:     printf("Flux  type  : Roe\n");    break;
		case AUSM:    printf("Flux  type  : AUSM\n");   break;
	}
	return 0;
}



int  SolveControl::set_solve(list<CmdParseBuf>::iterator  pcmd)
{
	//do clear
	solve_define.Type  = 0;

	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Type"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d SOLVE: Type syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"EQUILIBRIUM"))
				solve_define.Type = EQUILIBRIUM;
			else if(!strcmp(parg->argvalue.sval,"STEADYSTATE"))
				solve_define.Type = STEADYSTATE;
			else if(!strcmp(parg->argvalue.sval,"DCSWEEP"))
				solve_define.Type = DCSWEEP;
			else if(!strcmp(parg->argvalue.sval,"TRANSIENT"))
				solve_define.Type = TRANSIENT;
			else
				{printf("line %d SOLVE: Type syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"VScan"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d SOLVE: VScan syntax error!\n",pcmd->lineno);return 1;}
			solve_define.Electrode_VScan=bc.Get_bc_index(parg->argvalue.sval);
			if(solve_define.Electrode_VScan==-1)
				{printf("line %d SOLVE:Can't find VScan Electrode!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"IVFile"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d SOLVE: IVFile syntax error!\n",pcmd->lineno);return 1;}
			strcpy(solve_define.IVFile,parg->argvalue.sval);
		}
		else if(!strcmp(parg->label,"IVRecord"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d SOLVE: IVRecord syntax error!\n",pcmd->lineno);return 1;}
			solve_define.Electrode_Record=bc.Get_bc_index(parg->argvalue.sval);
			if(solve_define.Electrode_Record==-1)
				{printf("line %d SOLVE:Can't find IVRecord Electrode!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"VStart"))
		{
			 if(parg->argtype==DOUBLE)
				solve_define.VStart = parg->argvalue.dval;
			 else {printf("line %d SOLVE: VStart syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"VStep"))
		{
			if(parg->argtype==DOUBLE)
				solve_define.VStep = parg->argvalue.dval;
			else {printf("line %d SOLVE: VStep syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"VStop"))
		{
			if(parg->argtype==DOUBLE)
				solve_define.VStop = parg->argvalue.dval;
			else {printf("line %d SOLVE: VStop syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"TStart"))
		{
			 if(parg->argtype==DOUBLE)
				solve_define.TStart = parg->argvalue.dval;
			 else {printf("line %d SOLVE: TStart syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"TStop"))
		{
			if(parg->argtype==DOUBLE)
				solve_define.TStop = parg->argvalue.dval;
			else {printf("line %d SOLVE: TStop syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"AUTOSAVE"))
		{
			if(parg->argtype==DOUBLE)
				solve_define.AutoSave = parg->argvalue.dval;
			else {printf("line %d SOLVE: AUTOSAVE syntax error!\n",pcmd->lineno);return 1;}
		}

		else {printf("line %d SOLVE: %s no such command!\n",pcmd->lineno,parg->label);return 1;}
	}
	return 0;
}


int  SolveControl::set_extract(list<CmdParseBuf>::iterator  pcmd)
{
	//do clear
	extract_define.core = 0;
	extract_define.ascii = 0;
	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{

		if(!strcmp(parg->label,"CoreFile"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d EXTRACT: CoreFile syntax error!\n",pcmd->lineno);return 1;}
			extract_define.core = 1;
			strcpy(extract_define.CoreFile,parg->argvalue.sval);
		}
		else if(!strcmp(parg->label,"AscFile"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d EXTRACT: AscFile syntax error!\n",pcmd->lineno);return 1;}
			extract_define.ascii = 1;
			strcpy(extract_define.AscFile,parg->argvalue.sval);
		}
		else {printf("line %d EXTRACT: %s no such command!\n",pcmd->lineno,parg->label);return 1;}

	}
	return 0;
}

int  SolveControl::set_refine(list<CmdParseBuf>::iterator  pcmd)
{

	refine_define.Measure = Linear;
	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Variable"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d REFINE: Variable syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"Potential"))
				refine_define.Variable = Potential;
			else if(!strcmp(parg->argvalue.sval,"Doping"))
				refine_define.Variable = Doping;
			else 	{printf("line %d REFINE: Variable syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"Measure"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d REFINE: Measure syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"Linear"))
				refine_define.Measure = Linear;
			else if(!strcmp(parg->argvalue.sval,"SignedLog"))
				refine_define.Measure = SignedLog;
			else 	{printf("line %d REFINE: Measure syntax error!\n",pcmd->lineno);return 1;}
		}
		else if(!strcmp(parg->label,"Dispersion"))
		{
			if(parg->argtype==DOUBLE)
				refine_define.Dispersion = parg->argvalue.dval;
			else
				{printf("line %d REFINE: Dispersion syntax error!\n",pcmd->lineno);return 1;}
		}
		else {printf("line %d REFINE: %s no such command!\n",pcmd->lineno,parg->label);return 1;}

	}
	return 0;
}

int  SolveControl::set_import(list<CmdParseBuf>::iterator  pcmd)
{
	//do clear
	import_define.flag = 0;

	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"CoreFile"))
		{	if(parg->argtype!=CHARS)
				{printf("line %d IMPORT: CoreFile syntax error!\n",pcmd->lineno);return 1;}
			import_define.flag = 1;
			strcpy(import_define.CoreFile,parg->argvalue.sval);
		}
		else {printf("line %d IMPORT: %s no such command!\n",pcmd->lineno,parg->label);return 1;}
	}
	return 0;
}

int  SolveControl::set_attach(list<CmdParseBuf>::iterator  pcmd)
{
	//do clear
	attach_define.vsrc_name.clear();
	attach_define.vsrc_index.clear();

	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Electrode"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d ATTACH: Electrode syntax error!\n",pcmd->lineno);return 1;}
			strcpy(attach_define.Electrode,parg->argvalue.sval);
			attach_define.bc_index=bc.Get_bc_index(attach_define.Electrode);
			if(attach_define.bc_index==-1)
				{printf("line %d ATTACH:Can't find Electrode %s!\n",pcmd->lineno,attach_define.Electrode);return 1;}
		}

		else if(!strcmp(parg->label,"VApp"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d ATTACH: Vapp syntax error!\n",pcmd->lineno);return 1;}
			string vsrc_name=parg->argvalue.sval;
			attach_define.vsrc_name.push_back(vsrc_name);
			int flag=0;
			for(int j=0;j<vsrc.size();j++)
	  		  if(!strcmp(vsrc_name.c_str(),vsrc[j]->label))
	  		  {
	  		    attach_define.vsrc_index.push_back(j);
	  		    flag=1;
	  		  }
	  		if(flag==0)
				{printf("line %d ATTACH:Can't find VApp %s!\n",pcmd->lineno,parg->argvalue.sval);return 1;}

		}

		else {printf("line %d ATTACH: %s no such command!\n",pcmd->lineno,parg->label);return 1;}
	}
	return 0;

}


int  SolveControl::set_plot(list<CmdParseBuf>::iterator  pcmd)
{
	//set default value
	plot_define.MeshOnly = 0;
	plot_define.Resolution = RES_800x600;
	plot_define.Measure = Linear;
	plot_define.GeneratePSFile = 0;
	plot_define.Style = F_COLOR;
	plot_define.az = 240;
	plot_define.el = 60;
	plot_define.Persp = 0;
	plot_define.ztick = -1;


	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Variable"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d PLOT: Variable syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"Potential"))
				plot_define.Variable = Potential;
			else if(!strcmp(parg->argvalue.sval,"Na"))
				plot_define.Variable = Na;
			else if(!strcmp(parg->argvalue.sval,"Nd"))
				plot_define.Variable = Nd;
			else if(!strcmp(parg->argvalue.sval,"ElecDensity"))
				plot_define.Variable = ElecDensity;
			else if(!strcmp(parg->argvalue.sval,"HoleDensity"))
				plot_define.Variable = HoleDensity;
			else if(!strcmp(parg->argvalue.sval,"Temperature"))
				plot_define.Variable = Temperature;
			else if(!strcmp(parg->argvalue.sval,"EFieldX"))
				plot_define.Variable = EFieldX;
			else if(!strcmp(parg->argvalue.sval,"EFieldY"))
				plot_define.Variable = EFieldY;
			else if(!strcmp(parg->argvalue.sval,"Mesh"))
				plot_define.MeshOnly = MeshObject;
			else 	{printf("line %d PLOT: Variable syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"Resolution"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d PLOT: Resolution syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"Low"))
				plot_define.Resolution = RES_640x480;
			else if(!strcmp(parg->argvalue.sval,"Middle"))
				plot_define.Resolution = RES_800x600;
			else if(!strcmp(parg->argvalue.sval,"High"))
				plot_define.Resolution = RES_1024x768;
			else	{printf("line %d PLOT: Resolution syntax error!\n",pcmd->lineno);return 1;}

		}

		else if(!strcmp(parg->label,"PSFile"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d PLOT: PSFile syntax error!\n",pcmd->lineno);return 1;}
			plot_define.GeneratePSFile = 1;
			strcpy(plot_define.PSFileName,parg->argvalue.sval);
	  	}

	  	else if(!strcmp(parg->label,"Measure"))
		{
			if(parg->argtype!=CHARS)
				{printf("line %d PLOT: Measure syntax error!\n",pcmd->lineno);return 1;}
			if(!strcmp(parg->argvalue.sval,"Linear"))
				plot_define.Measure = Linear;
			else if(!strcmp(parg->argvalue.sval,"SignedLog"))
				plot_define.Measure = SignedLog;
			else 	{printf("line %d PLOT: Measure syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"AzAngle"))
		{
			 if(parg->argtype==DOUBLE)
				plot_define.az = parg->argvalue.dval;
			 else {printf("line %d PLOT: AzAngle syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"ElAngle"))
		{
			 if(parg->argtype==DOUBLE)
				plot_define.el = parg->argvalue.dval;
			 else {printf("line %d PLOT: ElAngle syntax error!\n",pcmd->lineno);return 1;}
		}

		else if(!strcmp(parg->label,"Style"))
		{
			 if(parg->argtype!=CHARS)
				{printf("line %d PLOT: Style syntax error!\n",pcmd->lineno);return 1;}
			 if(!strcmp(parg->argvalue.sval,"Scale"))
				plot_define.Style = F_SCALE;
			 else if(!strcmp(parg->argvalue.sval,"Color"))
				plot_define.Style = F_COLOR;
			 else if(!strcmp(parg->argvalue.sval,"GrayLevel"))
				plot_define.Style = F_GRAYLEVEL;
			 else 	{printf("line %d PLOT: Style syntax error!\n",pcmd->lineno);return 1;}
		}

		else {printf("line %d PLOT: %s no such command!\n",pcmd->lineno,parg->label);return 1;}
	}
	return 0;
}



