#include "mathfunc.h"
#include "ntddm.h"

double 	SMCZone::Jn_drift(int i,int j,double ilen)
{
	double mu1 =  mt->mu_elec(fs[i].Na,fs[i].Nd,fs[i].T);
	double mu2 =  mt->mu_elec(fs[j].Na,fs[j].Nd,fs[j].T);
	double mu  =  (mu1+mu2)/2.0;
	double V1 = fs[i].P;
	double V2 = fs[j].P;
	double E  = -(V2-V1)/ilen;
	double n1 = fs[i].n;
	double n2 = fs[j].n;
	double Vt = mt->k*(fs[i].T+fs[j].T)/2.0/mt->e;
	double n  = n1*aux2((V1-V2)/(2*Vt)) + n2*aux2((V2-V1)/(2*Vt));
	return mu*E*n;
	
}	

double 	SMCZone::Jn_diffuse(int i,int j,double ilen)
{
	double mu1 =  mt->mu_elec(fs[i].Na,fs[i].Nd,fs[i].T);
	double mu2 =  mt->mu_elec(fs[j].Na,fs[j].Nd,fs[j].T);
	double mu  =  (mu1+mu2)/2.0;
	double V1 = fs[i].P;
	double V2 = fs[j].P;
	double n1 = fs[i].n;
	double n2 = fs[j].n;
	double Vt = mt->k*(fs[i].T+fs[j].T)/2.0/mt->e;
	double dndx = aux1((V1-V2)/(2*Vt))*(n2-n1)/ilen;
	return mu*Vt*dndx;
}	

double 	SMCZone::Jp_drift(int i,int j,double ilen)
{
	double mu1 =  mt->mu_hole(fs[i].Na,fs[i].Nd,fs[i].T);
	double mu2 =  mt->mu_hole(fs[j].Na,fs[j].Nd,fs[j].T);
	double mu =  (mu1+mu2)/2.0;
	double V1 = fs[i].P;
	double V2 = fs[j].P;
	double E  = -(V2-V1)/ilen;
	double p1 = fs[i].p;
	double p2 = fs[j].p;
	double Vt = mt->k*(fs[i].T+fs[j].T)/2.0/mt->e;
	double p  = p1*aux2((V2-V1)/(2*Vt)) + p2*aux2((V1-V2)/(2*Vt));
	return mu*E*p;
}	

double 	SMCZone::Jp_diffuse(int i,int j,double ilen)
{
	double mu1 =  mt->mu_hole(fs[i].Na,fs[i].Nd,fs[i].T);
	double mu2 =  mt->mu_hole(fs[j].Na,fs[j].Nd,fs[j].T);
	double mu  =  (mu1+mu2)/2.0;
	double V1 = fs[i].P;
	double V2 = fs[j].P;
	double p1 = fs[i].p;
	double p2 = fs[j].p;
	double Vt = mt->k*(fs[i].T+fs[j].T)/2.0/mt->e;
	double dpdx = aux1((V1-V2)/(2*Vt))*(p2-p1)/ilen;
	return mu*Vt*dpdx;
}




int   SMCZone::get_electrode_current(DABC &bc,int bc_index,double deviceW,double &current)
{
	if(bc[bc_index].BCType==OhmicContract)
	{   
 	   //summer
 	   current=0;
	   for(int i=0;i<bc[bc_index].psegment->node_array.size();i++)
	   {
	     	int node=bc[bc_index].psegment->node_array[i];
	     	const VoronoiCell* pcell = pzone->davcell.GetPointer(node);
	     	for(int j=0;j<pcell->nb_num;j++)
	     		current += deviceW*pcell->elen[j]*
	     			  (Jn_drift(node,pcell->nb_array[j],pcell->ilen[j])
	     			  +Jn_diffuse(node,pcell->nb_array[j],pcell->ilen[j])
	     		          +Jp_drift(node,pcell->nb_array[j],pcell->ilen[j])
	     		          -Jp_diffuse(node,pcell->nb_array[j],pcell->ilen[j]));
	   } 
	   return 0;  		     
	}    
	return 1;
}
