#ifndef _jflux_h_
#define _jflux_h_
inline double 	Jn(double mu,double Vt,double V1,double V2,double n1,double n2,double h)
{
	double E  = -(V2-V1)/h;
	double n  = n1*aux2((V1-V2)/(2*Vt)) + n2*aux2((V2-V1)/(2*Vt));
	double dndx = aux1((V1-V2)/(2*Vt))*(n2-n1)/h;
	return mu*(E*n+Vt*dndx);
}	

inline double 	Jp(double mu,double Vt,double V1,double V2,double p1,double p2,double h)
{
	double E  = -(V2-V1)/h;
	double p  = p1*aux2((V2-V1)/(2*Vt)) + p2*aux2((V1-V2)/(2*Vt));
	double dpdx = aux1((V1-V2)/(2*Vt))*(p2-p1)/h;
	return mu*(E*p-Vt*dpdx);
}	

inline double 	dJn_dVi(double mu,double Vt,double V1,double V2,double n1,double n2,double h)
{
	double E  = -(V2-V1)/h;
	double n  = n1*aux2((V1-V2)/(2*Vt)) + n2*aux2((V2-V1)/(2*Vt));
	double dE_dVi =  1.0/h;
	double dn_dVi =  n1*pd1aux2((V1-V2)/(2*Vt))/(2*Vt) - n2*pd1aux2((V2-V1)/(2*Vt))/(2*Vt);
	double ddndx_dVi =  pd1aux1((V1-V2)/(2*Vt))/(2*Vt)*(n2-n1)/h;
	return mu*(dE_dVi*n+E*dn_dVi + Vt*ddndx_dVi);
}	

inline double 	dJn_dVd(double mu,double Vt,double V1,double V2,double n1,double n2,double h)
{
	double E  = -(V2-V1)/h;
	double n  = n1*aux2((V1-V2)/(2*Vt)) + n2*aux2((V2-V1)/(2*Vt));
	double dE_dVd =  -1.0/h;
	double dn_dVd =  -n1*pd1aux2((V1-V2)/(2*Vt))/(2*Vt) + n2*pd1aux2((V2-V1)/(2*Vt))/(2*Vt);
	double ddndx_dVd =  -pd1aux1((V1-V2)/(2*Vt))/(2*Vt)*(n2-n1)/h;
	return mu*(dE_dVd*n+E*dn_dVd + Vt*ddndx_dVd);
}	

inline double 	dJn_dni(double mu,double Vt,double V1,double V2,double n1,double n2,double h)
{
	double E  = -(V2-V1)/h;
	double dn_dni =     aux2((V1-V2)/(2*Vt));
	double ddndx_dni = -aux1((V1-V2)/(2*Vt))/h;
	return mu*(E*dn_dni + Vt*ddndx_dni);
}	

inline double 	dJn_dnd(double mu,double Vt,double V1,double V2,double n1,double n2,double h)
{
	double E  = -(V2-V1)/h;
	double dn_dnd =     aux2((V2-V1)/(2*Vt));
	double ddndx_dnd =  aux1((V1-V2)/(2*Vt))/h;
	return mu*(E*dn_dnd + Vt*ddndx_dnd);
}	

inline double 	dJp_dVi(double mu,double Vt,double V1,double V2,double p1,double p2,double h)
{
	double E  = -(V2-V1)/h;
	double p  =  p1*aux2((V2-V1)/(2*Vt)) + p2*aux2((V1-V2)/(2*Vt));
	double dE_dVi =  1.0/h;
	double dp_dVi    =  -p1*pd1aux2((V2-V1)/(2*Vt))/(2*Vt) + p2*pd1aux2((V1-V2)/(2*Vt))/(2*Vt);
	double ddpdx_dVi =  pd1aux1((V1-V2)/(2*Vt))/(2*Vt)*(p2-p1)/h;
	return mu*(dE_dVi*p+E*dp_dVi - Vt*ddpdx_dVi);
}	

inline double 	dJp_dVd(double mu,double Vt,double V1,double V2,double p1,double p2,double h)
{
	double E  = -(V2-V1)/h;
	double p  =  p1*aux2((V2-V1)/(2*Vt)) + p2*aux2((V1-V2)/(2*Vt));
	double dE_dVd = -1.0/h;
	double dp_dVd    =  p1*pd1aux2((V2-V1)/(2*Vt))/(2*Vt) - p2*pd1aux2((V1-V2)/(2*Vt))/(2*Vt);
	double ddpdx_dVd =  -pd1aux1((V1-V2)/(2*Vt))/(2*Vt)*(p2-p1)/h;
	return mu*(dE_dVd*p+E*dp_dVd - Vt*ddpdx_dVd);
}	

inline double 	dJp_dpi(double mu,double Vt,double V1,double V2,double p1,double p2,double h)
{
	double E  = -(V2-V1)/h;
	double dp_dpi =     aux2((V2-V1)/(2*Vt));
	double ddpdx_dpi = -aux1((V1-V2)/(2*Vt))/h;
	return mu*(E*dp_dpi - Vt*ddpdx_dpi);
}	

inline double 	dJp_dpd(double mu,double Vt,double V1,double V2,double p1,double p2,double h)
{
	double E  = -(V2-V1)/h;
	double dp_dpd =     aux2((V1-V2)/(2*Vt));
	double ddpdx_dpd =  aux1((V1-V2)/(2*Vt))/h;
	return mu*(E*dp_dpd - Vt*ddpdx_dpd);
}	

#endif
