#include "mathfunc.h"
#include "ntddm.h"
#include "jflux.h"




/* ----------------------------------------------------------------------------
 * form_function_pn:  This function setup DDM equation F(x)=0
 */
PetscErrorCode form_function_pn(SNES snes,Vec x,Vec f,void *dummy)
{

   int size = 0;
   int equ_num = 0;
   PetscScalar    *xx,*ff;
   Nt_DDM_Solver *ps = (Nt_DDM_Solver*)dummy;
   double dt = ps->dt;

   //Get pointers to vector data.
   VecGetArray(x,&xx);
   VecGetArray(f,&ff);

   for(int z=0;z<ps->zone_num;z++)
   {
   	 //semiconductor zone
   	 if(ps->zonedata[z]->material_type==semiconductor)
   	 {
   	   equ_num = 3;
   	   size = ps->zone[z].davcell.size();
   	   SMCZone *pzonedata= dynamic_cast< SMCZone * >(ps->zonedata[z]);

   	   double k = ps->zonedata[z]->mt->k;
   	   double e = ps->zonedata[z]->mt->e;
   	   double eps = ps->zonedata[z]->mt->eps;

   	   for(int i=0;i<size;i++)
   	   {
   	   	const VoronoiCell* pcell = ps->zone[z].davcell.GetPointer(i);

   	   	if(!pcell->bc_index||
   	   	    ps->bc[pcell->bc_index-1].BCType==NeumannBoundary||
   	   	    ps->bc[pcell->bc_index-1].BCType==InsulatorContract||
   	   	    ps->bc[pcell->bc_index-1].BCType==InsulatorInterface)
   	   	{
   	   		double Na = pzonedata->fs[i].Na;
   	   		double Nd = pzonedata->fs[i].Nd;
   	   		double T  = pzonedata->fs[i].T;
   	   		double Vt = k*T/e;

   	   		double Vi = xx[ps->zofs[z]+3*i+0];     //potential of node i
   	   		double ni = xx[ps->zofs[z]+3*i+1];     //electron density of node i
   	   		double pi = xx[ps->zofs[z]+3*i+2];     //hole density of node i
   	   		double mun =  ps->zonedata[z]->mt->mu_elec(Na,Nd,T);
   	   		double mup =  ps->zonedata[z]->mt->mu_hole(Na,Nd,T);

   	   		double R   =  pzonedata->mt->recombination(pi,ni,Na,Nd,T);
   	   		double area = pcell->area;
   	   		double grad_div_P = 0;
   	   		double grad_P = 0;
   	   		double Fn = 0;
   	   		double Fp = 0;

   			for(int j=0;j<pcell->nb_num;j++)
   			{
   				int    nb = pcell->nb_array[j];
   				double Nar = pzonedata->fs[nb].Na;
   	   			double Ndr = pzonedata->fs[nb].Nd;
   	   			double Tr  = pzonedata->fs[nb].T;

   				double Vr = xx[ps->zofs[z]+3*nb+0];     //potential of nb node
   				double nr = xx[ps->zofs[z]+3*nb+1];     //electron density of nb node
   				double pr = xx[ps->zofs[z]+3*nb+2];     //hole density of nb node
   				double munr =  pzonedata->mt->mu_elec(Nar,Ndr,Tr);
   	   			double mupr =  pzonedata->mt->mu_hole(Nar,Ndr,Tr);
   				double hmunr =  (mun+munr)/2;
   				double hmupr =  (mup+mupr)/2;

   				grad_div_P += pcell->elen[j]/pcell->ilen[j]*(Vr-Vi)/area;

   				if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==InsulatorContract)
   				  if(j==0||j==pcell->nb_num-1)
   				  {
   					double vapp = ps->bc[pcell->bc_index-1].V;
 	        			//double q = e*sigma; //sigma is the surface change density
 	        			double q = 0;
 	               	 		double eps_1 = pzonedata->mt->eps;
        		 		double eps_2 = pzonedata->mt->eps_SiO2;
        		 		double Thick = ps->bc[pcell->bc_index-1].Thick;
 	          	 		double r=q/eps_1 + eps_2/eps_1/Thick*vapp;
 	          	 		double s=eps_2/eps_1/Thick;
 	          	 		grad_div_P += 0.5*pcell->ilen[j]*(r-0.25*s*(3*Vi+Vr))/area;
   				  }

   				Fn += Jn(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[j])*pcell->elen[j]/area;
   				Fp += Jp(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[j])*pcell->elen[j]/area;
   			}

   			if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==InsulatorInterface)
   			{
   				//first get norm vector to interface
              			double nx,ny;
              			for(int k=0;k<pcell->mirror_num;k++)
              			{
              				nx = pcell->mirror[k].nx;
              				ny = pcell->mirror[k].ny;
              				if(fabs(nx)+fabs(ny)>0.5) break;
              			}

              			//use lease-square technology to get derivative
              			double a=0,b=0,c=0,d=0,e=0,w=0;
        			const VoronoiCell *pcell = ps->zone[z].davcell.GetPointer(i);
        			for(int k=0;k<pcell->nb_num;k++)
        			{
        				const VoronoiCell *ncell = ps->zone[z].davcell.GetPointer(pcell->nb_array[k]);
        				double dx = ncell->x - pcell->x;
        				double dy = ncell->y - pcell->y;
        				double du = xx[ps->zofs[z]+3*pcell->nb_array[k]] - xx[ps->zofs[z]+3*i];
        				w=1.0/sqrt(dx*dx+dy*dy);
        				a+=w*w*dx*dx;
        				b+=w*w*dx*dy;
        	 			c+=w*w*dy*dy;
        	 			d+=w*w*du*dx;
        	 			e+=w*w*du*dy;
        			}
        	 		double ux=(c*d-b*e)/(a*c-b*b);
        	 		double uy=(a*e-b*d)/(a*c-b*b);

        			int donorzone,donornode; //donorzone : Insulator zone
            			ps->zinterface.Find_donor(z,i,donorzone,donornode);
            			const VoronoiCell *dcell = ps->zone[donorzone].davcell.GetPointer(donornode);
            			a=0,b=0,c=0,d=0,e=0,w=0;
        			for(int k=0;k<dcell->nb_num;k++)
        			{
        				const VoronoiCell *ncell = ps->zone[donorzone].davcell.GetPointer(dcell->nb_array[k]);
        				double dx = ncell->x - dcell->x;
        				double dy = ncell->y - dcell->y;
        				double du = xx[ps->zofs[donorzone]+dcell->nb_array[k]] - xx[ps->zofs[donorzone]+donornode];
        				w=1.0/sqrt(dx*dx+dy*dy);
        				a+=w*w*dx*dx;
        				b+=w*w*dx*dy;
        	 			c+=w*w*dy*dy;
        	 			d+=w*w*du*dx;
        	 			e+=w*w*du*dy;
        			}
        			double uxd=(c*d-b*e)/(a*c-b*b);
        	 		double uyd=(a*e-b*d)/(a*c-b*b);
        	 		grad_P = eps*(ux*nx+uy*ny) - ps->zonedata[donorzone]->mt->eps*(uxd*nx+uyd*ny);

        	 		ff[ps->zofs[z]+3*i+0] =  grad_P;
   				ff[ps->zofs[z]+3*i+1] = -(ni-pzonedata->fs[i].n)/dt + Fn - R;
   				ff[ps->zofs[z]+3*i+2] = -(pi-pzonedata->fs[i].p)/dt - Fp - R;
        	  	}
        	  	else
   			{
   				ff[ps->zofs[z]+3*i+0] =  grad_div_P + e/eps*(pi-ni+Nd-Na);
   				ff[ps->zofs[z]+3*i+1] = -(ni-pzonedata->fs[i].n)/dt + Fn - R;
   				ff[ps->zofs[z]+3*i+2] = -(pi-pzonedata->fs[i].p)/dt - Fp - R;
   			}
   			continue;
   	   	}

   	   	if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==OhmicContract)
   	   	{
   	   		double Na = pzonedata->fs[i].Na;
   	   		double Nd = pzonedata->fs[i].Nd;
   	   		double T  = pzonedata->fs[i].T;
   	   		double nie = pzonedata->mt->ni(T);
   	   		double Vbi = pzonedata->mt->Vbi(Na,Nd,T);
   	   		double electron_density,hole_density;
   	   		int    om_equ;
   	   		for(int j=0;j<pzonedata->electrode.size();j++)
   	   			if(pzonedata->electrode[j]==pcell->bc_index-1)	{om_equ=j;break;}

   	   		ff[ps->zofs[z]+3*i+0] = xx[ps->zofs[z]+3*i+0]-Vbi-xx[ps->zofs[z]+equ_num*size+om_equ];

   			if(Na>Nd)   //p-type
   			{
   				hole_density = (fabs(Na)+sqrt(Na*Na+4*nie))/2.0;
   				electron_density = nie*nie/hole_density;
   			}
   	        	else        //n-type
   	        	{
   	        		electron_density = (fabs(Nd)+sqrt(Nd*Nd+4*nie))/2.0;
   				hole_density = nie*nie/electron_density;
   			}
   	        	ff[ps->zofs[z]+3*i+1] = xx[ps->zofs[z]+3*i+1] - electron_density;  //electron density
   	        	ff[ps->zofs[z]+3*i+2] = xx[ps->zofs[z]+3*i+2] - hole_density;      //hole density
   	        	continue;
   		}

   	   	if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==SchottkyContract)
   	   	{
   	   		continue;
   	   	}

    	   }

    	   for(int i=0;i<pzonedata->electrode.size();i++)
    	   {
    	   	int bc_index = pzonedata->electrode[i];
    	   	double R = ps->bc[bc_index].R;
    	   	double C = ps->bc[bc_index].C;
    	   	double L = ps->bc[bc_index].L;
    	   	double V = ps->bc[bc_index].V;
    	   	double In = ps->bc[bc_index].I;
    	   	double Icn = ps->bc[bc_index].Ic;
    	   	double Pn = ps->bc[bc_index].P;
    	   	double P = xx[ps->zofs[z]+equ_num*size+i];

 	   	double current=0;
	   	for(int j=0;j<ps->bc[bc_index].psegment->node_array.size();j++)
	   	{
	     	  int node=ps->bc[bc_index].psegment->node_array[j];
	     	  const VoronoiCell* pcell = ps->zone[z].davcell.GetPointer(node);

   	   	  double T  = pzonedata->fs[node].T;
   	   	  double Vt = k*T/e;

   	   	  double Vi = xx[ps->zofs[z]+3*node+0];     //potential of node i
   	   	  double ni = xx[ps->zofs[z]+3*node+1];     //electron density of node i
   	   	  double pi = xx[ps->zofs[z]+3*node+2];     //hole density of node i
   	   	  double mun =  ps->zonedata[z]->mt->mu_elec(pzonedata->fs[node].Na,pzonedata->fs[node].Nd,T);
   	   	  double mup =  ps->zonedata[z]->mt->mu_hole(pzonedata->fs[node].Na,pzonedata->fs[node].Nd,T);

	     	  for(int k=0;k<pcell->nb_num;k++)
	     	  {
	     	  	int    nb = pcell->nb_array[k];
	     	  	double Vr = xx[ps->zofs[z]+3*nb+0];     //potential of nb node
   			double nr = xx[ps->zofs[z]+3*nb+1];     //electron density of nb node
   			double pr = xx[ps->zofs[z]+3*nb+2];     //hole density of nb node
   			double munr =  pzonedata->mt->mu_elec(pzonedata->fs[nb].Na,pzonedata->fs[nb].Nd,pzonedata->fs[nb].T);
   	   		double mupr =  pzonedata->mt->mu_hole(pzonedata->fs[nb].Na,pzonedata->fs[nb].Nd,pzonedata->fs[nb].T);
   			double hmunr =  (mun+munr)/2;
   			double hmupr =  (mup+mupr)/2;
   			current += ps->DeviceDepth*pcell->elen[k]*
   					(Jn(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[k])
   				        +Jp(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[k]));
	       	  }
	  	}

    	   	ff[ps->zofs[z]+equ_num*size+i]=(L/dt+R)*current-V+(1+(L/dt+R)*C/dt)*P-(L/dt+R)*C/dt*Pn-L/dt*(In+Icn);

    	   }


   	 }

   	 // Insulator zone
   	 if(ps->zonedata[z]->material_type==Insulator)
   	 {
   	   equ_num = 1;
   	   size = ps->zone[z].davcell.size();
   	   for(int i=0;i<size;i++)
   	   {
   	   	const VoronoiCell* pcell = &ps->zone[z].davcell[i];
   	   	if(!pcell->bc_index||ps->bc[pcell->bc_index-1].BCType==NeumannBoundary)
               	{
               		double Vi = xx[ps->zofs[z]+i];
               		double grad_div_P = 0;
               		for(int j=0;j<pcell->nb_num;j++)
   			{
   				int    nb = pcell->nb_array[j];
   				double Vr = xx[ps->zofs[z]+nb];     //potential of nb node
   				grad_div_P += pcell->elen[j]/pcell->ilen[j]*(Vr-Vi)/pcell->area;
   			}
   			ff[ps->zofs[z]+i] =  grad_div_P;
   			continue;
     		}

                if(ps->bc[pcell->bc_index-1].BCType==InsulatorInterface)
                {
                	int donorzone,donornode;
            		ps->zinterface.Find_donor(z,i,donorzone,donornode);
                	ff[ps->zofs[z]+i] = xx[ps->zofs[z]+i] - xx[ps->zofs[donorzone]+3*donornode];
                	continue;
                }

                if(ps->bc[pcell->bc_index-1].BCType==GateContract)
                {
                	ff[ps->zofs[z]+i] = xx[ps->zofs[z]+i] - ps->bc[pcell->bc_index-1].V;
                	continue;
                }
   	   }
   	 }

   }

   //Restore vectors
   VecRestoreArray(x,&xx);
   VecRestoreArray(f,&ff);

   VecNorm(f,NORM_1,&ps->norm);
   printf("  iterative %d\tnorm = %e\n",ps->its++,ps->norm);
   return 0;
}



/* ----------------------------------------------------------------------------
 * form_jacobian_pn:  This function setup jacobian matrix F'(x) of DDM equation F(x)=0
 * dual carrier edition
 */
PetscErrorCode form_jacobian_pn(SNES snes,Vec x,Mat *jac,Mat *B,MatStructure *flag,void *dummy)
{
   PetscScalar    *xx,A[9];
   PetscInt       index[3],col[3];

   Nt_DDM_Solver *ps = (Nt_DDM_Solver*)dummy;
   //Get pointer to vector data
   VecGetArray(x,&xx);

   double dt = ps->dt;
   int equ_num = 0;
   int size = 0;
   MatZeroEntries(*jac);	//clear old matrix
   //Compute Jacobian entries and insert into matrix.
   for(int z=0;z<ps->zone_num;z++)
   {
   	 if(ps->zonedata[z]->material_type==semiconductor)
   	 {
   	   equ_num = 3;
   	   size = ps->zone[z].davcell.size();
   	   SMCZone *pzonedata= dynamic_cast< SMCZone * >(ps->zonedata[z]);
   	   double k = ps->zonedata[z]->mt->k;
   	   double e = ps->zonedata[z]->mt->e;
   	   double eps = ps->zonedata[z]->mt->eps;

   	   for(int i=0;i<size;i++)
   	   {
   	   	const VoronoiCell* pcell = ps->zone[z].davcell.GetPointer(i);

   	   	if(!pcell->bc_index||
   	   	    ps->bc[pcell->bc_index-1].BCType==NeumannBoundary||
   	   	    ps->bc[pcell->bc_index-1].BCType==InsulatorContract||
   	   	    ps->bc[pcell->bc_index-1].BCType==InsulatorInterface)
		{
   	   		double Na = pzonedata->fs[i].Na;
   	   		double Nd = pzonedata->fs[i].Nd;
   	   		double T  = pzonedata->fs[i].T;
   	   		double Vt = k*T/e;
   	   		double Vi = xx[ps->zofs[z]+3*i+0];     //potential of node i
   	   		double ni = xx[ps->zofs[z]+3*i+1];     //electron density of node i
   	   		double pi = xx[ps->zofs[z]+3*i+2];     //hole density of node i
   	   		double mun =  ps->zonedata[z]->mt->mu_elec(Na,Nd,T);
   	   		double mup =  ps->zonedata[z]->mt->mu_hole(Na,Nd,T);
   	   		double area = pcell->area;
   	   		double d_grad_div_P_dVi = 0;
   	   		double d_Fn_dni = 0;
   	   		double d_Fn_dVi = 0;
   	   		double d_Fp_dpi = 0;
   	   		double d_Fp_dVi = 0;
   	   		//--------------------------------
   	   		index[0] = ps->zofs[z]+3*i+0;
   			index[1] = ps->zofs[z]+3*i+1;
   			index[2] = ps->zofs[z]+3*i+2;
   			//--------------------------------
   	   		for(int j=0;j<pcell->nb_num;j++)
   			{
   				int    nb = pcell->nb_array[j];
   				double Nar = pzonedata->fs[nb].Na;
   	   			double Ndr = pzonedata->fs[nb].Nd;
   	   			double Tr  = pzonedata->fs[nb].T;

   				double Vr = xx[ps->zofs[z]+3*nb+0];     //potential of nb node
   				double nr = xx[ps->zofs[z]+3*nb+1];     //electron density of nb node
   				double pr = xx[ps->zofs[z]+3*nb+2];     //hole density of nb node
   				double munr =  pzonedata->mt->mu_elec(Nar,Ndr,Tr);
   	   			double mupr =  pzonedata->mt->mu_hole(Nar,Ndr,Tr);
   				double hmunr =  (mun+munr)/2;
   				double hmupr =  (mup+mupr)/2;

   				//-------------------------------------
   				d_grad_div_P_dVi += -pcell->elen[j]/pcell->ilen[j]/area;
   				d_Fn_dVi += dJn_dVi(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[j])*pcell->elen[j]/area;
   				d_Fn_dni += dJn_dni(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[j])*pcell->elen[j]/area;
   				d_Fp_dVi += dJp_dVi(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[j])*pcell->elen[j]/area;
   				d_Fp_dpi += dJp_dpi(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[j])*pcell->elen[j]/area;
   				//-------------------------------------
   				//df(x)i/dx(r)
   				A[0] =  pcell->elen[j]/pcell->ilen[j]/area;      //dfun(0)/dP(r)
   				A[1] =  0;              			 //dfun(0)/dn(r)
   				A[2] =  0;              			 //dfun(0)/dp(r)

   				A[3] =  dJn_dVd(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[j])*pcell->elen[j]/area;    //dfun(1)/dP(r)
   				A[4] =  dJn_dnd(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[j])*pcell->elen[j]/area;    //dfun(1)/dn(r)
   				A[5] =   0;              		        		   	     //dfun(1)/dp(r)

   				A[6] = -dJp_dVd(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[j])*pcell->elen[j]/area;    //dfun(2)/dP(r)
   				A[7] =   0;              		        		             //dfun(2)/dn(r)
   				A[8] = -dJp_dpd(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[j])*pcell->elen[j]/area;    //dfun(2)/dp(r)

   				if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==InsulatorContract)
   				  if(j==0||j==pcell->nb_num-1)
   				  {
   					double eps_1 = ps->zonedata[z]->mt->eps;
        		 		double eps_2 = ps->zonedata[z]->mt->eps_SiO2;
        		 		double Thick = ps->bc[pcell->bc_index-1].Thick;
 	          			double s=eps_2/eps_1/Thick;
 	          			d_grad_div_P_dVi += -0.5*pcell->ilen[j]*0.25*s*3/area;
   					A[0]+= -0.5*pcell->ilen[j]*0.25*s/area;
   				  }

   				col[0] = ps->zofs[z]+3*nb+0;
   				col[1] = ps->zofs[z]+3*nb+1;
   				col[2] = ps->zofs[z]+3*nb+2;

   				MatSetValues(*jac,3,index,3,col,A,INSERT_VALUES);

   			}
   			//fun(0) is the poisson's equation
   			//fun(1) is the continuous equation of electron
   			//fun(2) is the continuous equation of hole

   			//df(x)i/dx(i)
   			if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==InsulatorInterface)
   			{
   				A[0] = 0;    //dfun(0)/dP(i)
   				A[1] = 0;    //dfun(0)/dn(i)
   				A[2] = 0;    //dfun(0)/dp(i)
   			}
   			else
   			{
   				A[0] =  d_grad_div_P_dVi;    //dfun(0)/dP(i)
   				A[1] =  -e/eps;        	     //dfun(0)/dn(i)
   				A[2] =   e/eps;              //dfun(0)/dp(i)
   			}
   			A[3] =  d_Fn_dVi;            //dfun(1)/dP(i)
   			A[4] =  -1/dt + d_Fn_dni;    //dfun(1)/dn(i)
   			A[5] =  0;             	     //dfun(1)/dp(i)

   			A[6] =  -d_Fp_dVi;           //dfun(2)/dP(i)
   			A[7] =  0;             	     //dfun(2)/dn(i)
   			A[8] =  -1/dt - d_Fp_dpi;    //dfun(2)/dp(i)

   			MatSetValues(*jac,3,index,3,index,A,INSERT_VALUES);

   			if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==InsulatorInterface)
               		{

              			//first get norm vector to interface
              			double nx,ny;
              			for(int k=0;k<pcell->mirror_num;k++)
              			{
              				nx = pcell->mirror[k].nx;
              				ny = pcell->mirror[k].ny;
              				if(fabs(nx)+fabs(ny)>0.5) break;
              			}
              			//use lease-square technology to get derivative
              			double coefficient=0;
        			for(int k=0;k<pcell->nb_num;k++)
        			{
        				const VoronoiCell *ncell = ps->zone[z].davcell.GetPointer(pcell->nb_array[k]);
        				int ofs = ps->zone[z].offset;
        				double dx = ncell->x - pcell->x;
        				double dy = ncell->y - pcell->y;
        				double w=1.0/sqrt(dx*dx+dy*dy);
        				double a=ps->a1[i+ofs];
        				double b=ps->a2[i+ofs];
        				double c=ps->a3[i+ofs];
        				double ay = 1/(a*c-b*b)*(a*w*w*dy-b*w*w*dx);
        				double ax = 1/(a*c-b*b)*(c*w*w*dx-b*w*w*dy);
        				double m = eps*(ax*nx + ay*ny);
        				MatSetValue(*jac,ps->zofs[z]+3*i,ps->zofs[z]+3*pcell->nb_array[k],m,INSERT_VALUES);
        				coefficient+=-m;
        				//cout<<coefficient<<endl;
        			}
        	  		MatSetValue(*jac,ps->zofs[z]+3*i,ps->zofs[z]+3*i,coefficient,INSERT_VALUES);

        			int donorzone,donornode;
            			ps->zinterface.Find_donor(z,i,donorzone,donornode);
            			const VoronoiCell *dcell = ps->zone[donorzone].davcell.GetPointer(donornode);
            			coefficient=0;
        			for(int k=0;k<dcell->nb_num;k++)
        			{
        				const VoronoiCell *ncell = ps->zone[donorzone].davcell.GetPointer(dcell->nb_array[k]);
        				int ofs = ps->zone[donorzone].offset;
        				double dx = ncell->x - dcell->x;
        				double dy = ncell->y - dcell->y;
        				double w=1.0/sqrt(dx*dx+dy*dy);
        				double a=ps->a1[donornode+ofs];
        				double b=ps->a2[donornode+ofs];
        				double c=ps->a3[donornode+ofs];
        				double ay = 1/(a*c-b*b)*(a*w*w*dy-b*w*w*dx);
        				double ax = 1/(a*c-b*b)*(c*w*w*dx-b*w*w*dy);
        				double eps = ps->zonedata[donorzone]->mt->eps;
        				double m = eps*(ax*nx + ay*ny);
        				MatSetValue(*jac,ps->zofs[z]+3*i,ps->zofs[donorzone]+dcell->nb_array[k],-m,INSERT_VALUES);
        				coefficient+=-m;
        	  		}
        	  		MatSetValue(*jac,ps->zofs[z]+3*i,ps->zofs[donorzone]+donornode,-coefficient,INSERT_VALUES);
        	  	}

   			continue;

   		}

   		if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==OhmicContract)
   	   	{
   	   		A[0] = 1;  A[1] = 0;  A[2] = 0;
   			A[3] = 0;  A[4] = 1;  A[5] = 0;
   	        	A[6] = 0;  A[7] = 0;  A[8] = 1;
   	        	index[0] = ps->zofs[z]+3*i+0;
   			index[1] = ps->zofs[z]+3*i+1;
   			index[2] = ps->zofs[z]+3*i+2;

   			MatSetValues(*jac,3,index,3,index,A,INSERT_VALUES);

   			int    om_equ;
   	   		for(int j=0;j<pzonedata->electrode.size();j++)
   	   			if(pzonedata->electrode[j]==pcell->bc_index-1)	{om_equ=j;break;}
   			MatSetValue(*jac,ps->zofs[z]+3*i,ps->zofs[z]+equ_num*size+om_equ,-1,INSERT_VALUES);

   			continue;
   		}

   		if(pcell->bc_index&&ps->bc[pcell->bc_index-1].BCType==SchottkyContract)
   		{
   			continue;
   		}

      	   }

    	   for(int i=0;i<pzonedata->electrode.size();i++)
    	   {
    	   	int bc_index = pzonedata->electrode[i];
    	   	int matrix_row = ps->zofs[z]+equ_num*size+i;
    	   	double R = ps->bc[bc_index].R;
    	   	double C = ps->bc[bc_index].C;
    	   	double L = ps->bc[bc_index].L;
	   	MatAssemblyBegin(*jac,MAT_FLUSH_ASSEMBLY);
                MatAssemblyEnd(*jac,MAT_FLUSH_ASSEMBLY);
		for(int j=0;j<ps->bc[bc_index].psegment->node_array.size();j++)
	   	{
	     	  int node=ps->bc[bc_index].psegment->node_array[j];
	     	  const VoronoiCell* pcell = ps->zone[z].davcell.GetPointer(node);

   	   	  double T  = pzonedata->fs[node].T;
   	   	  double Vt = k*T/e;

   	   	  double Vi = xx[ps->zofs[z]+3*node+0];     //potential of node i
   	   	  double ni = xx[ps->zofs[z]+3*node+1];     //electron density of node i
   	   	  double pi = xx[ps->zofs[z]+3*node+2];     //hole density of node i
   	   	  double mun =  ps->zonedata[z]->mt->mu_elec(pzonedata->fs[node].Na,pzonedata->fs[node].Nd,T);
   	   	  double mup =  ps->zonedata[z]->mt->mu_hole(pzonedata->fs[node].Na,pzonedata->fs[node].Nd,T);

	     	  for(int k=0;k<pcell->nb_num;k++)
	     	  {
	     	  	int    nb = pcell->nb_array[k];
	     	  	double Vr = xx[ps->zofs[z]+3*nb+0];     //potential of nb node
   			double nr = xx[ps->zofs[z]+3*nb+1];     //electron density of nb node
   			double pr = xx[ps->zofs[z]+3*nb+2];     //hole density of nb node
   			double munr =  pzonedata->mt->mu_elec(pzonedata->fs[nb].Na,pzonedata->fs[nb].Nd,pzonedata->fs[nb].T);
   	   		double mupr =  pzonedata->mt->mu_hole(pzonedata->fs[nb].Na,pzonedata->fs[nb].Nd,pzonedata->fs[nb].T);
   			double hmunr =  (mun+munr)/2;
   			double hmupr =  (mup+mupr)/2;

   			double dJ_dVi = dJn_dVi(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[k])*pcell->elen[k];
   			               +dJp_dVi(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[k])*pcell->elen[k];
   			double dJ_dni = dJn_dni(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[k])*pcell->elen[k];
   			double dJ_dpi = dJp_dpi(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[k])*pcell->elen[k];

   			double dJ_dVr = dJn_dVd(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[k])*pcell->elen[k];
   			               +dJp_dVd(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[k])*pcell->elen[k];
   			double dJ_dnr = dJn_dnd(hmunr,Vt,Vi,Vr,ni,nr,pcell->ilen[k])*pcell->elen[k];
   			double dJ_dpr = dJp_dpd(hmupr,Vt,Vi,Vr,pi,pr,pcell->ilen[k])*pcell->elen[k];

   			MatSetValue(*jac,matrix_row,ps->zofs[z]+3*node+0,ps->DeviceDepth*dJ_dVi*(L/dt+R),ADD_VALUES);
	     	 	MatSetValue(*jac,matrix_row,ps->zofs[z]+3*node+1,ps->DeviceDepth*dJ_dni*(L/dt+R),ADD_VALUES);
	     	 	MatSetValue(*jac,matrix_row,ps->zofs[z]+3*node+2,ps->DeviceDepth*dJ_dpi*(L/dt+R),ADD_VALUES);

   			MatSetValue(*jac,matrix_row,ps->zofs[z]+3*nb+0,ps->DeviceDepth*dJ_dVr*(L/dt+R),ADD_VALUES);
	     	  	MatSetValue(*jac,matrix_row,ps->zofs[z]+3*nb+1,ps->DeviceDepth*dJ_dnr*(L/dt+R),ADD_VALUES);
	     	  	MatSetValue(*jac,matrix_row,ps->zofs[z]+3*nb+2,ps->DeviceDepth*dJ_dpr*(L/dt+R),ADD_VALUES);
   		  }
                  MatAssemblyBegin(*jac,MAT_FLUSH_ASSEMBLY);
                  MatAssemblyEnd(*jac,MAT_FLUSH_ASSEMBLY);
	  	}

		MatSetValue(*jac,matrix_row,matrix_row,1+(L/dt+R)*C/dt,INSERT_VALUES); //dJ/dP
    	   }

   	 }

   	 if(ps->zonedata[z]->material_type==Insulator)
   	 {
   	   equ_num = 1;
   	   size = ps->zone[z].davcell.size();
   	   for(int i=0;i<size;i++)
   	   {
   	   	const VoronoiCell* pcell = &ps->zone[z].davcell[i];
   	   	//inner point or neumann bc type
         	if(!pcell->bc_index||ps->bc[pcell->bc_index-1].BCType==NeumannBoundary)
                 {
                 	double coefficient=0;
                       	for(int k=0;k<pcell->nb_num;k++)
                  	{
                  		double a=pcell->elen[k]/pcell->ilen[k]/pcell->area;
                  		MatSetValue(*jac,ps->zofs[z]+i,ps->zofs[z]+pcell->nb_array[k],a,INSERT_VALUES);
        			coefficient+=-a;
        		}
        		MatSetValue(*jac,ps->zofs[z]+i,ps->zofs[z]+i,coefficient,INSERT_VALUES);
     			continue;
                 }

        	if(ps->bc[pcell->bc_index-1].BCType==InsulatorInterface)
              	{
              		int donorzone,donornode;
            		ps->zinterface.Find_donor(z,i,donorzone,donornode);
            		MatSetValue(*jac,ps->zofs[z]+i,ps->zofs[z]+i,1,INSERT_VALUES);
            		MatSetValue(*jac,ps->zofs[z]+i,ps->zofs[donorzone]+3*donornode,-1,INSERT_VALUES);
            		continue;
            	}

            	if(ps->bc[pcell->bc_index-1].BCType==GateContract)
              	{MatSetValue(*jac,ps->zofs[z]+i,ps->zofs[z]+i,1,INSERT_VALUES);continue;}
   	   }
   	 }

   }

   *flag = SAME_NONZERO_PATTERN;

   //Restore vector
   VecRestoreArray(x,&xx);

   //Assemble matrix
   MatAssemblyBegin(*jac,MAT_FINAL_ASSEMBLY);
   MatAssemblyEnd(*jac,MAT_FINAL_ASSEMBLY);
   //MatView(*jac,PETSC_VIEWER_DRAW_WORLD);
   //getchar();
   return 0;
}



/* ----------------------------------------------------------------------------
 * Nt_DDM_Solver::init_solver:  This function do initial setup for nonlinear solver
 */
int Nt_DDM_Solver::init_solver()
{

  printf("DDM solver init...");

  // compute the scale of problem
  zofs.resize(zone_num+1);
  zofs[0] = 0;
  for(int i=0;i<zone_num;i++)
  {
     if(zonedata[i]->material_type==semiconductor) //semiconductor zone
     {
     	SMCZone *pzonedata = dynamic_cast< SMCZone * >(zonedata[i]);
     	N+= 3*pzonedata->pzone->davcell.size();
       	N+=pzonedata->electrode.size();  //additional equation for electrode
       	zofs[i+1] = N;
     }
     else if(zonedata[i]->material_type==Insulator) //Insulator zone
     {
     	N+= zone[i].davcell.size();
     	zofs[i+1] = zone[i].davcell.size();
     }
     total_nodes += zonedata[i]->pzone->davcell.size();
  }

  //init least-squares parameters
  a1  = new double[total_nodes];
  a2  = new double[total_nodes];
  a3  = new double[total_nodes];
  for(int i=0;i<total_nodes;i++)   	a1[i]=a2[i]=a3[i]=0;

  for(int z=0;z<zone_num;z++)
  {
    int ofs = zone[z].offset;
    for(int i=0; i<zone[z].davcell.size();i++)
    {
        const VoronoiCell *pcell = zone[z].davcell.GetPointer(i);
        for(int k=0;k<pcell->nb_num;k++)
       	{
          const VoronoiCell *ncell = zone[z].davcell.GetPointer(pcell->nb_array[k]);
          double dx = ncell->x - pcell->x;
          double dy = ncell->y - pcell->y;
          double w  = 1.0/sqrt(dx*dx+dy*dy);
          a1[i+ofs]+=w*w*dx*dx;
          a2[i+ofs]+=w*w*dx*dy;
          a3[i+ofs]+=w*w*dy*dy;
        }
    }
  }
  // Create nonlinear solver context
  SNESCreate(PETSC_COMM_WORLD,&snes);
  VecCreateSeq(PETSC_COMM_SELF,N,&x);
  VecDuplicate(x,&r);

  // Create Jacobian matrix data structure
  // pre-alloc approximate memory
  int *nnz = new int[N];
  int *p = nnz;
  for(int i=0;i<zone_num;i++)
  {
    //semiconductor zone
    if(zonedata[i]->material_type==semiconductor)
    {
     SMCZone *pzonedata = dynamic_cast< SMCZone * >(zonedata[i]);
     for(int j=0;j<pzonedata->pzone->davcell.size();j++)
     {
     	*p++ = 3*(pzonedata->pzone->davcell[j].nb_num+1);
     	*p++ = 3*(pzonedata->pzone->davcell[j].nb_num+1);
     	*p++ = 3*(pzonedata->pzone->davcell[j].nb_num+1);
     }

     for(int j=0;j<pzonedata->electrode.size();j++)
     	*p++ = bc[pzonedata->electrode[j]].psegment->node_array.size()+1;

    }
    //Insulator zones, poisson equation only
    else if(zonedata[i]->material_type==Insulator)
    	 for(int j=0;j<zone[i].davcell.size();j++)
 		*p++ = 1*(zone[i].davcell[j].nb_num+1);
   }

   MatCreateSeqAIJ(PETSC_COMM_SELF,N,N,0,nnz,&J);
   MatSetFromOptions(J);
   delete [] nnz;

   // Set function evaluation routine and vector.
   // Set Jacobian matrix data structure and Jacobian evaluation routine

   SNESSetFunction(snes,r,form_function_pn,this);
   SNESSetJacobian(snes,J,J,form_jacobian_pn,this);

   //Set linear solver defaults for this problem. By extracting the
   //KSP, KSP, and PC contexts from the SNES context, we can then
   //directly call any KSP, KSP, and PC routines to set various options.

   SNESGetKSP(snes,&ksp);
   KSPGetPC(ksp,&pc);
   PCSetType(pc,PCLU);
   //SNESSetTrustRegionTolerance(snes,1e-12*N);

   SNESSetTolerances(snes,1e-12*N,1e-8,1e-8/N,1000,5000);
   SNESSetType(snes,SNESLS);

   //Evaluate initial guess,importnat for newton solver

   double init_value[3];
   int    index[3];
   int    offset=0;
   for(int z=0;z<zone_num;z++)
   {
   	 if(zonedata[z]->material_type==semiconductor)
   	 {
   	     SMCZone *pzonedata = dynamic_cast< SMCZone * >(zonedata[z]);
   	     for(int i=0;i<zone[z].davcell.size();i++)
   	     {
   	   	index[0] = offset+0;
   		index[1] = offset+1;
   		index[2] = offset+2;
   		init_value[0] = zonedata[z]->fs[i].P;
   		init_value[1] = zonedata[z]->fs[i].n;
   		init_value[2] = zonedata[z]->fs[i].p;
   		VecSetValues(x,3,index,init_value,INSERT_VALUES);
   		offset += 3;
   	     }
   	     for(int j=0;j<pzonedata->electrode.size();j++)
    	     {
    	     	VecSetValue(x,offset,0,INSERT_VALUES);
    	     	offset += 1;
    	     }
   	 }

   	 if(zonedata[z]->material_type==Insulator)
   	   for(int i=0;i<zone[z].davcell.size();i++)
   	   {
   	   	VecSetValue(x,offset,0,INSERT_VALUES);
   	   	offset += 1;
   	   }
   }
   //
   VecAssemblyBegin(x);
   VecAssemblyEnd(x);

   printf("done\n");
}



/* ----------------------------------------------------------------------------
 * Nt_DDM_Solver::do_solve:  This function solve the problem
 * current solution data(n p and P) is in fs
 * previous solution data(n and p) is in aux
 */
int Nt_DDM_Solver::do_solve(SolveDefine &sv)
{
   // reset newton method
   SNESSetType(snes,SNESLS);  //default method
   if(sv.SNES==LineSearchCubic)
   	SNESLineSearchSet(snes,SNESLineSearchCubic,PETSC_NULL);
   else if(sv.SNES==LineSearchQuadratic)
   	SNESLineSearchSet(snes,SNESLineSearchQuadratic,PETSC_NULL);
   else if(sv.SNES==LineSearchNo)
   	SNESLineSearchSet(snes,SNESLineSearchNo,PETSC_NULL);
   else if(sv.SNES==TrustRegion)
   	SNESSetType(snes,SNESTR);
   SNESSetFromOptions(snes);

   if(sv.Type==EQUILIBRIUM)
   {
   	printf("Compute equilibrium \n");
   	bc.Clear_Vapp();
   	dt = 1e38; its = 0;
   	SNESSolve(snes,PETSC_NULL,x);
   	solution_update(x);

   	SNESGetConvergedReason(snes,&reason);
   	VecNorm(r,NORM_1,&norm);
   	printf("residual norm = %e ConvergedReason %d\n\n",norm,reason);
   	return 0;
   }

   if(sv.Type==STEADYSTATE)
   {
   	printf("Compute steady-state \n");
   	bc.Set_Vapp(0);
   	dt = 1e38; its = 0;
   	SNESSolve(snes,PETSC_NULL,x);
   	solution_update(x);

   	SNESGetConvergedReason(snes,&reason);
   	VecNorm(r,NORM_1,&norm);
   	printf("residual norm = %e ConvergedReason %d\n\n",norm,reason);
   	return 0;
   }

   SMCZone *pzonedata;
	for(int i=0;i<zone_num;i++)
	  if(zonedata[i]->material_type == semiconductor)
	 	pzonedata = dynamic_cast< SMCZone * >(zonedata[i]);

   if(sv.Type==DCSWEEP)
   {
   	dt = 1e38;
   	double I=0;
   	bc.Set_Vapp(0);
   	printf("DC scan from %e step %e to %e\n",sv.VStart,sv.VStep,sv.VStop);
   	FILE *fiv=fopen(sv.IVFile,"w");
   	for(double V=sv.VStart;V<sv.VStop;V+=sv.VStep)
   	{
   		its = 0;
   		printf("DC Scan: V(%s) = %e\n",bc[sv.Electrode_VScan].psegment->label,V);
   		bc[sv.Electrode_VScan].V=V;
   		SNESSolve(snes,PETSC_NULL,x);
   		solution_update(x);
   		SNESGetConvergedReason(snes,&reason);
   		VecNorm(r,NORM_1,&norm);
   		printf("residual norm = %e ConvergedReason %d\n\n",norm,reason);
   		pzonedata->get_electrode_current(bc,sv.Electrode_Record,DeviceDepth,I);
   		fprintf(fiv,"%e\t%15.13e\n",bc[sv.Electrode_Record].P,I);
   	}
   	fclose(fiv);
   	return 0;
   }


   if(sv.Type==TRANSIENT)
   {
   	 clock = sv.TStart;
   	 dt = sv.TStep;
   	 double I = 0;
   	 printf("Transient compute from %e step %e to %e\n",sv.TStart,sv.TStep,sv.TStop);
   	 FILE *fiv=fopen(sv.IVFile,"w");
   	 while(clock<sv.TStop)
   	 {
 		printf("t = %e\n",clock);
 		bc.Set_Vapp(clock);
 		its = 0;
 		SNESSolve(snes,PETSC_NULL,x);
 		solution_update(x);
		clock+=dt;
		SNESGetConvergedReason(snes,&reason);
   		VecNorm(r,NORM_1,&norm);
   		printf("residual norm = %e ConvergedReason %d\n\n",norm,reason);
   		pzonedata->get_electrode_current(bc,sv.Electrode_Record,DeviceDepth,I);
   		fprintf(fiv,"%e\t%15.13e\n",bc[sv.Electrode_Record].P,I);
   	 }
   	 fclose(fiv);
   	 return 0;
   }

   //solve finished, do data synchronization
   //get_electrode_current should in class bc or class ntddm
   solution_update(x);

}



/* ----------------------------------------------------------------------------
 * Nt_DDM_Solver::destroy_solver:  This function do destroy the nonlinear solver
 */
int Nt_DDM_Solver::destroy_solver()
{
   // Free work space
   N = 0;
   total_nodes = 0;
   delete [] a1;
   delete [] a2;
   delete [] a3;
   zofs.clear();
   VecDestroy(x); VecDestroy(r);
   MatDestroy(J); SNESDestroy(snes);

}



void Nt_DDM_Solver::solution_update(Vec& x)
{
//------------------------------------------------------------
   PetscScalar    *xx;
   VecGetArray(x,&xx);
   int    offset=0;
   for(int z=0;z<zone_num;z++)
   {
   	 if(zonedata[z]->material_type==semiconductor)
   	 {
   	     SMCZone *pzonedata = dynamic_cast< SMCZone * >(zonedata[z]);
   	     for(int i=0;i<zone[z].davcell.size();i++)
   	     {
   		zonedata[z]->fs[i].P = xx[offset+0];
   		zonedata[z]->fs[i].n = xx[offset+1];
   		zonedata[z]->fs[i].p = xx[offset+2];
   		offset += 3;
      	     }
      	     for(int i=0;i<pzonedata->electrode.size();i++)
    	     {
    	     	double P = xx[offset];
    	     	double Pn = bc[pzonedata->electrode[i]].P;
    	     	double C = bc[pzonedata->electrode[i]].C;
    	     	double I;
    	     	pzonedata->get_electrode_current(bc,pzonedata->electrode[i],DeviceDepth,I);
    	     	bc[pzonedata->electrode[i]].Ic = C*(P-Pn)/dt;
    	     	bc[pzonedata->electrode[i]].I = I;
    	     	bc[pzonedata->electrode[i]].P = P;
    	     	offset += 1;
    	     }
   	 }

   	 if(zonedata[z]->material_type==Insulator)
   	   for(int i=0;i<zone[z].davcell.size();i++)
   	   {
   	   	zonedata[z]->fs[i].P = xx[offset];
   	   	offset += 1;
   	   }
   }

   VecRestoreArray(x,&xx);
}
