#include "parser.tab.h"
#include "vsource.h"
#include "cmdbuf.h"
#include "solver.h"
#include <string.h>
#include <stdio.h>


VSource* SetVDC(list<CmdParseBuf>::iterator  pcmd)  	  	
{
	VSource* vsrc;
	char   *ID;
	double Tdelay=0,v=0;
	list<ArgStruct>::iterator parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Vconst"))
			v=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Tdelay"))
			Tdelay=parg->argvalue.dval;
		else if(!strcmp(parg->label,"ID"))
			ID=parg->argvalue.sval;		
	}			
	
 	vsrc=new VDC(Tdelay,v);
 	strcpy(vsrc->label,ID);
 	printf("\nVDC    : %s \ttd=%.2e v=%.2e\n",ID,Tdelay,v);
 	return vsrc;
}

VSource* SetVSIN(list<CmdParseBuf>::iterator  pcmd)  	  	
{
	VSource* vsrc;
	char   *ID;
	double Tdelay=0,vamp=0,fenq=0;
	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Vamp"))
			vamp=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Tdelay"))
			Tdelay=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Freq"))
			fenq=parg->argvalue.dval;
		else if(!strcmp(parg->label,"ID"))
			ID=parg->argvalue.sval;			
	}			
	
 	vsrc=new VSIN(Tdelay,vamp,fenq);
 	strcpy(vsrc->label,ID);
 	printf("\nVSIN   : %s \ttd=%.2e vamp=%.2e fenq=%.2e\n",ID,Tdelay,vamp,fenq);
 	return vsrc;
}	

VSource* SetVEXP(list<CmdParseBuf>::iterator  pcmd)  	  	
{
	VSource* vsrc;
	char   *ID;
	double Tdelay=0,trc=0,tfd=0,tfc=0,vlo=0,vhi=0;	
	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Tdelay"))
			Tdelay=parg->argvalue.dval;
		else if(!strcmp(parg->label,"TRC"))
			trc=parg->argvalue.dval;
		else if(!strcmp(parg->label,"TFD"))
			tfd=parg->argvalue.dval;
		else if(!strcmp(parg->label,"TFC"))
			tfc=parg->argvalue.dval;	
		else if(!strcmp(parg->label,"Vlo"))
			vlo=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Vhi"))
			vhi=parg->argvalue.dval;
		else if(!strcmp(parg->label,"ID"))
			ID=parg->argvalue.sval;	
	
	}			
	
 	vsrc=new VEXP(Tdelay,vlo,vhi,trc,tfd,tfc);
 	strcpy(vsrc->label,ID);
 	printf("\nVEXP   : %s \ttd=%.2e trc=%.2e tfd=%.2e tfc=%.2e\n\t\t vlo=%.2e vhi=%.2e\n",
 		ID,Tdelay,trc,tfd,tfc,vlo,vhi);
 	return vsrc;	
}	

VSource* SetVPULSE(list<CmdParseBuf>::iterator  pcmd)  	  	
{
	VSource* vsrc;
	char   *ID;
	double Tdelay=0, tr=0, tf=0, pw=0, pr=10, vlo=0, vhi=0;	
	list<ArgStruct>::iterator  parg;
	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
	{
		if(!strcmp(parg->label,"Tdelay"))
			Tdelay=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Tr"))
			tr=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Tf"))
			tf=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Pw"))
			pw=parg->argvalue.dval;	
		else if(!strcmp(parg->label,"Pr"))
			pr=parg->argvalue.dval;	
		else if(!strcmp(parg->label,"Vlo"))
			vlo=parg->argvalue.dval;
		else if(!strcmp(parg->label,"Vhi"))
			vhi=parg->argvalue.dval;
		else if(!strcmp(parg->label,"ID"))
			ID=parg->argvalue.sval;		
	}			
	
 	vsrc=new VPULSE(Tdelay,vlo,vhi,tr,tf,pw,pr);
 	strcpy(vsrc->label,ID);
 	printf("\nVPULSE : %s \ttd=%.2e tr=%.2e tf=%.2e pw=%.2e pr=%.2e\n\t\t vlo=%.2e vhi=%.2e\n",
 		ID,Tdelay,tr,tf,pw,pr,vlo,vhi);
 	return vsrc;	
}


//simple error hanfle function	
inline int read_config_err_handle(char *s)
{
	printf("%s\n",s);
	return 1;
}

/* ----------------------------------------------------------------------------
 * BSolver::static_config:  This function get global config data from CmdParseBuf  
 */	
int BSolver::static_config(list<CmdParseBuf> &cmdlist)
{
	list<CmdParseBuf>::iterator  pcmd;
 	list<ArgStruct>::iterator    parg;
 		
 	//search the CmdParseBuf 
 	for(pcmd=cmdlist.begin();pcmd!=cmdlist.end();pcmd++)
 	{ 
 	   if(!strcmp(pcmd->KeyWord,"set"))   //it is a set structure 
 	     for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
		{
			if(!strcmp(parg->label,"MeshFile"))  strcpy(MeshFile,parg->argvalue.sval);
 	    		if(!strcmp(parg->label,"ModelFile")) strcpy(ModelFile,parg->argvalue.sval);	
 	   
 	   		if(!strcmp(parg->label,"Carrier"))	
 	   			if(!strcmp(parg->argvalue.sval,"pn"))	     Carrier=PN_Both;
 	      	      		else if(!strcmp(parg->argvalue.sval,"p"))    Carrier=P_Type;
 	   	      		else if(!strcmp(parg->argvalue.sval,"n"))    Carrier=N_Type;	
 	   			else return read_config_err_handle("Unrecognized carrier type");
 				
 			if(!strcmp(parg->label,"LatticeTemp"))    LatticeTemp=parg->argvalue.dval;
 	   
 	       		if(!strcmp(parg->label,"DeviceDepth"))    DeviceDepth=parg->argvalue.dval;
 	       		
 	       		if(!strcmp(parg->label,"MeshScale"))      mesh_scale=1.0/parg->argvalue.dval;
 	       		
 	       	}	
        }
        config_info();
        printf("\nSetting each Vsource here...\n");
        for(pcmd=cmdlist.begin();pcmd!=cmdlist.end();pcmd++)
         if(!strcmp(pcmd->KeyWord,"vsource"))   //it is a vsource structure 
 	   {
 	   	for(parg=pcmd->arglist.begin();parg!=pcmd->arglist.end();parg++)
 		 if(!strcmp(parg->label,"Type"))
 		{
 			if(!strcmp(parg->argvalue.sval,"VDC"))
 				{vsrc.push_back(SetVDC(pcmd));} 
 			if(!strcmp(parg->argvalue.sval,"VSIN"))
 				{vsrc.push_back(SetVSIN(pcmd));}
 			if(!strcmp(parg->argvalue.sval,"VEXP"))
 				{vsrc.push_back(SetVEXP(pcmd));} 
 			if(!strcmp(parg->argvalue.sval,"VPULSE"))
 				{vsrc.push_back(SetVPULSE(pcmd));} 
 		} 
           }   
	
	return 0;
}

/* ----------------------------------------------------------------------------
 * BSolver::config_info:  This function print global config data to screen  
 * mainly for debug
 */	
void BSolver::config_info()
{
	
	printf("\n--------------------------------------------------\n");
	printf("cgns file   : %s\n",MeshFile);
	printf("model file  : %s\n",ModelFile);
				
	switch(Carrier)
	{
		case N_Type:  printf("carrier     : n  only\n");break;
		case P_Type:  printf("carrier     : p  only\n");break;
		case PN_Both: printf("carrier     : pn both\n");break;
	}
		
	printf("device depth init as %e \n",DeviceDepth);
	printf("lattice temprature init as %e K\n",LatticeTemp);
				
}	

