#include <math.h>
#include <stdio.h>
#include "exhdm.h"


//-------------------------------------------------------------------------------
inline vector_double  minmod(vector_double a,vector_double b)
{
	vector_double c,zero;
	Set_Vector_zero(zero);
	c=vsign(a)*max(zero,min(vabs(a),vsign(a)*b));
	return c;
}

inline vector_double Albada(vector_double a,vector_double b)
{
	vector_double c,eps;
	Set_Vector(eps,1e-38);
	c=0.5*(1+vsign(a*b))*((a*a+eps)*b+(b*b+eps)*a)/(a*a+b*b+2*eps);
	return c;
}


void SMCZone::first_order_reconst(DABC &bc,int carrier)
{
	if(carrier==P_Type||carrier==PN_Both)
	  for(int i=0;i<node_num;i++)
	   for(int j=0;j<pzone->davcell[i].nb_num;j++)
	  	aux[i].Qp_r[j]=fs[i].Qp;
	
	   
	if(carrier==N_Type||carrier==PN_Both)
	  for(int i=0;i<node_num;i++)
	   for(int j=0;j<pzone->davcell[i].nb_num;j++)
	  	aux[i].Qn_r[j]=fs[i].Qn;
	
}

void SMCZone::MUSCL_reconst(DABC &bc,int carrier)
{
  	 
  vector_double a1,b1,c1,a2,b2,c2;
  vector_double yn1,yn2,yp1,yp2;
  
  vector_double dQn,dQn_max;
  vector_double dQp,dQp_max;
  
  for(int i=0;i<node_num;i++)
  {
        const VoronoiCell* pcell = pzone->davcell.GetPointer(i);
       	Node        node = pzone->danode[i];
       	
       	Set_Vector_zero(yn1);  	Set_Vector_zero(yn2);
       	Set_Vector_zero(yp1);  	Set_Vector_zero(yp2);
      	
       	Set_Vector_zero(a1);   	Set_Vector_zero(b1);       Set_Vector_zero(c1); 
        Set_Vector_zero(a2);   	Set_Vector_zero(b2);       Set_Vector_zero(c2); 
        Set_Vector_zero(dQn_max); 
        Set_Vector_zero(dQp_max); 
        double h=sqrt(2*pcell->area);
        for(int j=0;j<pcell->nb_num;j++)
        {
        	dQn = vabs(fs[pcell->nb_array[j]].Qn-fs[i].Qn)+1e-38;
        	dQn_max=max(dQn,dQn_max);
        	dQp = vabs(fs[pcell->nb_array[j]].Qp-fs[i].Qp)+1e-38;
        	dQp_max=max(dQp,dQp_max);
        }
        
        for(int j=0;j<pcell->nb_num;j++)
        {
        	double dx = pzone->danode[pcell->nb_array[j]].x - node.x;
        	double dy = pzone->danode[pcell->nb_array[j]].y - node.y;
        	double dl = sqrt(dx*dx+dy*dy);
        	
        	if(carrier==N_Type||carrier==PN_Both)
        	{
        		dQn = fs[pcell->nb_array[j]].Qn-fs[i].Qn;
        		vector_double eno = vabs(dQn)/dQn_max+dl/h;
        	 	vector_double w=1.0/(dl*dl)/eno/eno;
        	 	//double w = 1.0/dl;
        	 	a1 = a1 + w*w*dx*dx;
        		b1 = b1 + w*w*dx*dy;
        		c1 = c1 + w*w*dy*dy;             
        		yn1 = yn1 + w*w*dQn*dx;
        		yn2 = yn2 + w*w*dQn*dy;
        	}
        	if(carrier==P_Type||carrier==PN_Both)	
        	{
        		dQp = fs[pcell->nb_array[j]].Qp-fs[i].Qp;
        		vector_double eno = vabs(dQp)/dQp_max+dl/h;
        	 	vector_double w=1.0/(dl*dl)/eno/eno;
        	 	//double w = 1.0/dl;
        	 	a2 = a2 + w*w*dx*dx;
        		b2 = b2 + w*w*dx*dy;
        		c2 = c2 + w*w*dy*dy;             
        		yp1 = yp1 + w*w*dQp*dx;
        		yp2 = yp2 + w*w*dQp*dy;
        	}
        }
              
       //boundary point: process ghost cell
       if(pcell->bc_index&&
          (bc[pcell->bc_index-1].BCType==NeumannBoundary||
           bc[pcell->bc_index-1].BCType==InsulatorContract||
           bc[pcell->bc_index-1].BCType==InsulatorInterface))
	{
	
		for (int j=0;j<pcell->mirror_num; j++)
	 	{     
	 		int c=pcell->mirror[j].index;
	 		int iedge=pcell->mirror[j].nbindex;
	 		int cedge=pcell->inb_array[iedge];
	 		double nx = pcell->mirror[j].nx;
	 		double ny = pcell->mirror[j].ny;
	 		
	 		double dx = pcell->mirror[j].x - node.x;
        		double dy = pcell->mirror[j].y - node.y;
        		double dl = sqrt(dx*dx+dy*dy);
        	
	 		vector_double ghost; 
	 		matrix_double M;
	 		Set_Matrix_zero(M);
	 		M.m[0] = 1; 
	 		M.m[5] = 1-2*nx*nx; M.m[6] = -2*nx*ny;
	 		M.m[9] = -2*nx*ny;  M.m[10] = 1-2*ny*ny;
	 		M.m[15] = 1;
	 		 
	 		if(carrier==N_Type||carrier==PN_Both)
	 		{
	 		  ghost = M*fs[c].Qn;
	 		  dQn = ghost-fs[i].Qn;
        		  vector_double eno = vabs(dQn)/dQn_max+dl/h;
        	 	  //vector_double w=1.0/(dl*dl)/eno/eno;
        	 	  double w = 1.0/dl;
        	 	  a1 = a1 + w*w*dx*dx;
        		  b1 = b1 + w*w*dx*dy;
        		  c1 = c1 + w*w*dy*dy;             
        		  yn1 = yn1 + w*w*dQn*dx;
        		  yn2 = yn2 + w*w*dQn*dy;
			}	  
		        if(carrier==P_Type||carrier==PN_Both)
		        {
		          ghost = M*fs[c].Qp;
		          dQp = ghost-fs[i].Qp;
        		  vector_double eno = vabs(dQp)/dQp_max+dl/h;
        	 	  //vector_double w=1.0/(dl*dl)/eno/eno;
        	 	  double w = 1.0/dl;
        	 	  a2 = a2 + w*w*dx*dx;
        		  b2 = b2 + w*w*dx*dy;
        		  c2 = c2 + w*w*dy*dy;             
        		  yp1 = yp1 + w*w*dQp*dx;
        		  yp2 = yp2 + w*w*dQp*dy;
			}	  
		}		  
	  }	        
		   
        aux[i].Qnx = (c1*yn1-b1*yn2)/(a1*c1-b1*b1);
        aux[i].Qny = (a1*yn2-b1*yn1)/(a1*c1-b1*b1);
        aux[i].Qpx = (c2*yp1-b2*yp2)/(a2*c2-b2*b2);
        aux[i].Qpy = (a2*yp2-b2*yp1)/(a2*c2-b2*b2);
  }

  for(int i=0;i<pzone->davedge.size();i++)
  {
	 const VoronoiCell *cell1 = pzone->davcell.GetPointer(pzone->davedge[i].cell1);
	 const VoronoiCell *cell2 = pzone->davcell.GetPointer(pzone->davedge[i].cell2);
	 int cell1_index = pzone->davedge[i].cell1;
	 int cell2_index = pzone->davedge[i].cell2;	
	 int c1_nb =  pzone->davedge[i].c1_nb;		//cell2 index in cell1's nb_array
	 int c2_nb =  pzone->davedge[i].c2_nb;		//cell1 index in cell2's nb_array
	 
	 double angle = pzone->davcell[cell1_index].angle[c1_nb];
	 double ilen  = pzone->davcell[cell1_index].ilen[c1_nb];	 
	
	if(carrier==N_Type||carrier==PN_Both)
	{
	  vector_double dQidr = (aux[cell1_index].Qnx*cos(angle)+aux[cell1_index].Qny*sin(angle))*ilen;
	  vector_double dQjdr = (aux[cell2_index].Qnx*cos(angle)+aux[cell2_index].Qny*sin(angle))*ilen;
	  vector_double di1 = fs[cell2_index].Qn - fs[cell1_index].Qn;

	  aux[cell1_index].Qn_r[c1_nb] = fs[cell1_index].Qn + Albada(di1,dQidr)*0.5*ilen;
	  aux[cell2_index].Qn_r[c2_nb] = fs[cell2_index].Qn - Albada(dQjdr,di1)*0.5*ilen;	       	    
        }
        
        if(carrier==P_Type||carrier==PN_Both)
        {
          vector_double dQidr = (aux[cell1_index].Qpx*cos(angle)+aux[cell1_index].Qpy*sin(angle))*ilen;
	  vector_double dQjdr = (aux[cell2_index].Qpx*cos(angle)+aux[cell2_index].Qpy*sin(angle))*ilen;
	  vector_double di1 = fs[cell2_index].Qp - fs[cell1_index].Qp;

	  aux[cell1_index].Qp_r[c1_nb] = fs[cell1_index].Qp + Albada(di1,dQidr)*0.5*ilen;
	  aux[cell2_index].Qp_r[c2_nb] = fs[cell2_index].Qp - Albada(dQjdr,di1)*0.5*ilen;	       	    		  
	}	  
   }
}


/*
void SMCZone::least_squares_reconst(int carrier)
{
  	 
  	 vector_double y1,y2; 
         vector_double dQn,dQn_max;
         vector_double dQp,dQp_max;
  	 
  	 for(int i=0;i<node_num;i++)
        {
        	vector_double a,b,c;
        	double dl;
        	const VoronoiCell* pcell = pzone->davcell.GetPointer(i);
        	Node        node = pzone->danode[i];
               	Set_Vector_zero(dQn_max); 
               	Set_Vector_zero(dQp_max); 
        	double h=sqrt(pcell->area);
        	for(int j=0;j<pcell->nb_num;j++)
        	{
        		dQn = vabs(fs[pcell->nb_array[j]].Qn-fs[i].Qn)+1e-20;
               		dQn_max=max(dQn,dQn_max);
               		dQp = vabs(fs[pcell->nb_array[j]].Qp-fs[i].Qp)+1e-20;
               		dQp_max=max(dQp,dQp_max);
              	}
              	if(carrier==N_Type||carrier==PN_Both)
              	{	
        		Set_Vector_zero(a);   	Set_Vector_zero(b);
               		Set_Vector_zero(c); 
        		Set_Vector_zero(y1);   	Set_Vector_zero(y2);
        		for(int j=0;j<pcell->nb_num;j++)
        		{
        			double dx = pzone->danode[pcell->nb_array[j]].x - node.x;
        			double dy = pzone->danode[pcell->nb_array[j]].y - node.y;
        			dQn = fs[pcell->nb_array[j]].Qn-fs[i].Qn;
        			dl=sqrt(dx*dx+dy*dy);
        	 		vector_double eno = vabs(dQn)/dQn_max+dl/h;
        	 		vector_double w=1.0/(dl*dl)/eno/eno;
        	 	 	a=a+w*w*dx*dx;
        			b=b+w*w*dx*dy;
        	 		c=c+w*w*dy*dy;
        	 		y1=y1+w*w*dQn*dx;
        	 		y2=y2+w*w*dQn*dy;
               	 	}
        		vector_double nx=(c*y1-b*y2)/(a*c-b*b);
        		vector_double ny=(a*y2-b*y1)/(a*c-b*b);
        		for(int j=0;j<pcell->nb_num;j++)
        		{
        			double angle = pcell->angle[j];
        			double ilen = pcell->ilen[j];
        			aux[i].Qn_r[j]=fs[i].Qn 
        				+ minmod((nx*cos(angle)+ny*sin(angle))*0.5*ilen,0.5*(fs[j].Qn-fs[i].Qn));
        		}		
        	}
        	if(carrier==P_Type||carrier==PN_Both)
        	{
        		Set_Vector_zero(a);   	Set_Vector_zero(b);
               		Set_Vector_zero(c); 
        		Set_Vector_zero(y1);   	Set_Vector_zero(y2);
        		for(int j=0;j<pcell->nb_num;j++)
        		{
        			double dx = pzone->danode[pcell->nb_array[j]].x - node.x;
        			double dy = pzone->danode[pcell->nb_array[j]].y - node.y;
        			dQp = fs[pcell->nb_array[j]].Qp-fs[i].Qp;
        			dl=sqrt(dx*dx+dy*dy);
        			vector_double eno = vabs(dQp)/dQp_max+dl/h;
        	  		vector_double w=1.0/(dl*dl)/eno/eno;
        			a=a+w*w*dx*dx;
        			b=b+w*w*dx*dy;
        	 		c=c+w*w*dy*dy;
        	 		y1=y1+w*w*dQp*dx;
        	 		y2=y2+w*w*dQp*dy;
               	 	}
               		vector_double px=(c*y1-b*y2)/(a*c-b*b);
        		vector_double py=(a*y2-b*y1)/(a*c-b*b);
        	        	
        		for(int j=0;j<pcell->nb_num;j++)
        		{
        			double angle = pcell->angle[j];
        			double ilen = pcell->ilen[j];
        			aux[i].Qp_r[j]=fs[i].Qp
        		        	   + minmod((px*cos(angle)+py*sin(angle))*0.5*ilen,0.5*(fs[j].Qp-fs[i].Qp));
          		}
    		}
	}	 	
}
*/

//-------------------------------------------------------------------------------
inline double MPolynomial2plus(double M)
{    return 0.25*(M+1)*(M+1);  }
           
inline double MPolynomial2neg(double M)
{    return -0.25*(M-1)*(M-1);  }
       
inline double MPolynomial4plus(double M)
{
	if(fabs(M)>=1) 
	   return 0.5*(M+fabs(M));
	else
           return MPolynomial2plus(M)*(1-2*MPolynomial2neg(M));
}

inline double MPolynomial4neg(double M)
{
	if(fabs(M)>=1) 
	   return 0.5*(M-fabs(M));
	else
           return MPolynomial2neg(M)*(1+2*MPolynomial2plus(M));
}           

inline double PPolynomial5plus(double M)
{
	if(fabs(M)>=1)
	  return 1/M*0.5*(M+fabs(M));
	else
	  return MPolynomial2plus(M)*(2-M-3*M*MPolynomial2neg(M));
}

inline double PPolynomial5neg(double M)
{
	if(fabs(M)>=1)
	  return 1/M*0.5*(M-fabs(M));
	else
	  return MPolynomial2neg(M)*(-2-M+3*M*MPolynomial2plus(M));
}


vector_double AUSM_if_flux(const vector_double &sf1,const vector_double &sf2,
			   double elen,double ilen, double angle,double m1,double m2, double &CFL)
{
	double a1,a2,a;
	double M1,M2,M,Mavg,massflow;
	double n1,v1,p1,n2,v2,p2,p;
	double H1,H2;
        vector_double flux;	 
         //
        n1 = sf1.v[0];  //
	v1 = sqrt(sf1.v[1]*sf1.v[1]+sf1.v[2]*sf1.v[2])/(n1*m1);
	p1 = 2.0/3.0*(sf1.v[3] - 0.5*m1*n1*v1*v1);
	a1 = sqrt(5.0/3.0*p1/(n1*m1));
	H1 = sf1.v[3]/n1+p1/n1;


	n2 = sf2.v[0];  //
	v2 = sqrt(sf2.v[1]*sf2.v[1]+sf2.v[2]*sf2.v[2])/(n2*m2);
	p2 = 2.0/3.0*(sf2.v[3] - 0.5*m2*n2*v2*v2);
	a2 = sqrt(5.0/3.0*p2/(n2*m2));
	H2 = sf2.v[3]/n2+p2/n2;
	
	a  = sqrt(a1*a2);
	 
	M1 = (sf1.v[1]*cos(angle)+sf1.v[2]*sin(angle))/n1/m1/a;
	M2 = (sf2.v[1]*cos(angle)+sf2.v[2]*sin(angle))/n2/m2/a;
	
	 
	Mavg = 0.5*(M1*M1+M2*M2);
        M  = MPolynomial4plus(M1) + MPolynomial4neg(M2)-max(1-0.8*Mavg,0)*(p2-p1)/(p2+p1);
        massflow  = M>0? a*M*n1: a*M*n2;
	
        if(massflow > 0)
        {
	 	flux.v[0] = 1;
	 	flux.v[1] = sf1.v[1]/n1;
	 	flux.v[2] = sf1.v[2]/n1;
	 	flux.v[3] = H1;
	 	flux=flux*massflow;
	}
	else
	{
		flux.v[0] = 1;
	 	flux.v[1] = sf2.v[1]/n2;
	 	flux.v[2] = sf2.v[2]/n2;
	 	flux.v[3] = H2;
	 	flux=flux*massflow;
	}

	p =  PPolynomial5plus(M1)*p1 + PPolynomial5neg(M2)*p2
	     -PPolynomial5plus(M1)*PPolynomial5neg(M2)*sqrt(n1*m1*n2*m2)*a*a*(M2-M1);
	flux.v[1] += p*cos(angle);  //	
	flux.v[2] += p*sin(angle);  //	
	if(CFL>ilen/(a+max(v1,v2))) CFL=ilen/(a+max(v1,v2));

	return flux*elen;
}


//-------------------------------------------------------------------------------
inline vector_double Fx(const vector_double &U,double m)
{
	vector_double F;
	double n = U.v[0];
	double vx= U.v[1]/(n*m);
	double vy= U.v[2]/(n*m);
	double p = 2.0/3.0*(U.v[3] - 0.5*m*n*(vx*vx+vy*vy));
	double E = U.v[3]/n;
	
	F.v[0] = n*vx;
	F.v[1] = m*n*vx*vx+p;
	F.v[2] = m*n*vx*vy;
	F.v[3] = n*vx*(E+p/n);
	
	return F; 
}
	
inline vector_double Fy(const vector_double &U,double m)
{
	vector_double F;
	double n = U.v[0];
	double vx= U.v[1]/(n*m);
	double vy= U.v[2]/(n*m);
	double p = 2.0/3.0*(U.v[3] - 0.5*m*n*(vx*vx+vy*vy));
	double E = U.v[3]/n;
	
	F.v[0] = n*vy;
	F.v[1] = m*n*vx*vy;
	F.v[2] = m*n*vy*vy+p;
	F.v[3] = n*vy*(E+p/n);
	
	return F; 
}

inline matrix_double A_Roe(const vector_double &U,double angle,double m)
{
	matrix_double R,L,D; 
	double nx = cos(angle),ny = sin(angle);
	double n = U.v[0];
	double vx= U.v[1]/(n*m);
	double vy= U.v[2]/(n*m);
	double p = 2.0/3.0*(U.v[3] - 0.5*m*n*(vx*vx+vy*vy));
	double c  = sqrt(5.0/3.0*p/(n*m));
	
	R.m[0] = 0;   R.m[1] = 1/m;  R.m[2] = 1/m;      R.m[3] = 1/m;
	R.m[4] = -ny; R.m[5] = vx;   R.m[6] = vx-c*nx;  R.m[7] = vx+c*nx;  
	R.m[8] = nx;  R.m[9] = vy;   R.m[10]= vy-c*ny;  R.m[11]= vy+c*ny;
	R.m[12] = vy*nx-vx*ny; 
	R.m[13] = 0.5*(vx*vx+vy*vy);  
	R.m[14] = 0.5*(3*c*c+vx*vx+vy*vy-2*c*vx*nx-2*c*vy*ny);    
	R.m[15] = 0.5*(3*c*c+vx*vx+vy*vy+2*c*vx*nx+2*c*vy*ny);

	L.m[0] = -m*vy*nx+m*vx*ny; L.m[1] = -ny; L.m[2] = nx; L.m[3] = 0;
	L.m[4] = m*(3*c*c-vx*vx-vy*vy)/(3*c*c);
	L.m[5] = 2*vx/(3*c*c);
	L.m[6] = 2*vy/(3*c*c);
	L.m[7] = -2.0/(3*c*c);
	
	L.m[8] = m*(vx*vx+vy*vy+3*c*vx*nx+3*c*vy*ny)/(6*c*c);
	L.m[9] = -(3*c*nx+2*vx)/(6*c*c);
	L.m[10] =-(3*c*ny+2*vy)/(6*c*c);
	L.m[11] = 1.0/(3*c*c);
	
	L.m[12] = m*(vx*vx+vy*vy-3*c*vx*nx-3*c*vy*ny)/(6*c*c);
	L.m[13] = (3*c*nx-2*vx)/(6*c*c);
	L.m[14] = (3*c*ny-2*vy)/(6*c*c);
	L.m[15] = 1.0/(3*c*c);
	
	Set_Matrix_zero(D);
	
	D.m[0] = fabs(vx*nx+vy*ny);
	D.m[5] = fabs(vx*nx+vy*ny);
	D.m[10]= fabs(vx*nx+vy*ny-c);
	D.m[15]= fabs(vx*nx+vy*ny+c);
	
	return R*D*L;
	
}	



vector_double Roe_if_flux(const vector_double &sf1,const vector_double &sf2,
			   double elen,double ilen, double angle,double m1,double m2, double &CFL)
{
	vector_double U;
	
	double n1 = sf1.v[0];  
	double v1x= sf1.v[1]/(n1*m1);
	double v1y= sf1.v[2]/(n1*m1);
	double p1 = 2.0/3.0*(sf1.v[3] - 0.5*m1*n1*(v1x*v1x+v1y*v1y));
	double al = sqrt(5.0/3.0*p1/(n1*m1));
	double H1 = sf1.v[3]/n1+p1/n1;
		
	double n2 = sf2.v[0];  
	double v2x= sf2.v[1]/(n2*m2);
	double v2y= sf2.v[2]/(n2*m2);
	double p2 = 2.0/3.0*(sf2.v[3] - 0.5*m2*n2*(v2x*v2x+v2y*v2y));
	double a2 = sqrt(5.0/3.0*p2/(n2*m2));
	double H2 = sf2.v[3]/n2+p2/n2;
	 	
	 
	double ni = pow((sqrt(n1)+sqrt(n2))/2.0,2);
	double mi = (sqrt(n1)*m1+sqrt(n2)*m2)/(sqrt(n1)+sqrt(n2));
	double vix = (sqrt(n1)*v1x+sqrt(n2)*v2x)/(sqrt(n1)+sqrt(n2));
	double viy = (sqrt(n1)*v1y+sqrt(n2)*v2y)/(sqrt(n1)+sqrt(n2));
	double Hi = (sqrt(n1)*H1+sqrt(n2)*H2)/(sqrt(n1)+sqrt(n2));
	double ci = sqrt(2.0/3.0*(Hi/mi-0.5*(vix*vix+viy*viy)));
	double pi = 3.0/5.0*ni*ci*ci*mi;
	double Ei = Hi-pi/ni;
	
	if(CFL>ilen/(ci+sqrt(vix*vix+viy*viy))) CFL=ilen/(ci+sqrt(vix*vix+viy*viy));
	
	U.v[0] = ni;
	U.v[1] = mi*ni*vix;
	U.v[2] = mi*ni*viy;
	U.v[3] = ni*Ei;
	vector_double F_x,F_y;
	F_x = (Fx(sf1,m1)+Fx(sf2,m2))*cos(angle);
	F_y = (Fy(sf1,m1)+Fy(sf2,m2))*sin(angle);
	return 0.5*elen*(F_x+ F_y - A_Roe(U,angle,mi)*(sf2-sf1));
				   
}

//-------------------------------------------------------------------------------
int SMCZone::set_flux_function(int flux_type)
{
	if(flux_type == AUSM)
		{flux_function = AUSM_if_flux; return 0;}
	if(flux_type == Roe)
		{flux_function = Roe_if_flux; return 0;}
	printf("error, no flux function specified\n");
	return 1;			
}

void SMCZone::Get_jacobian_matrix(const vector_double &Q,matrix_double &Ax,matrix_double &Ay,int carrier)
{
	double n,m,vx,vy,p,c;
	
	if(carrier == N_Type)
		m = mt->mass_elec();
	else if(carrier == P_Type)
		m = mt->mass_hole();
	
	n  = Q.v[0];  //
	vx = Q.v[1]/(n*m);
	vy = Q.v[2]/(n*m);
	p  = 2.0/3.0*(Q.v[3] - 0.5*m*n*(vx*vx+vy*vy));
	c  = sqrt(5.0/3.0*p/(n*m));
		
	Ax.m[0] = 0;
	Ax.m[1] = 1.0/m;	
	Ax.m[2] = 0;	
	Ax.m[3] = 0;
	Ax.m[4] = 1.0/3*m*(-2*vx*vx+vy*vy); 
	Ax.m[5] = 4.0/3*vx; 
	Ax.m[6] = -2.0/3*vy;	
	Ax.m[7] = 2.0/3;
	Ax.m[8] = -m*vx*vy;	
	Ax.m[9] = vy;	
	Ax.m[10]= vx;	
	Ax.m[11]= 0;
	Ax.m[12]= -1.0/6*m*vx*(9*c*c+vx*vx+vy*vy);
	Ax.m[13]= 1.0/6*(9*c*c-vx*vx+3*vy*vy);
	Ax.m[14]= -2.0/3*vx*vy;
	Ax.m[15]= 5.0/3*vx;
	
	Ay.m[0] = 0;	
	Ay.m[1] = 0;	
	Ay.m[2] = 1.0/m;	
	Ay.m[3] = 0;
	Ay.m[4] = -m*vx*vy; 
	Ay.m[5] = vy; 
	Ay.m[6] = vx;	
	Ay.m[7] = 0;
	Ay.m[8] = 1.0/3*m*(vx*vx-2*vy*vy);	
	Ay.m[9] = -2.0/3*vx;	
	Ay.m[10]= 4.0/3*vy;	
	Ay.m[11]= 2.0/3;
	Ay.m[12]= -1.0/6*m*vy*(9*c*c+vx*vx+vy*vy);
	Ay.m[13]= -2.0/3*vx*vy;
	Ay.m[14]= 1.0/6*(9*c*c+3*vx*vx-vy*vy);
	Ay.m[15]= 5.0/3*vy;
	
}


void SMCZone::form_matrix(const vector_double &Qi,const vector_double &Qj,double angle,double elen,
			  matrix_double& Ai,matrix_double&Aj,int carrier)
{
	matrix_double I,Jx,Jy;
        Set_Matrix_I(I);
        double m;
        if(carrier == N_Type)
		m = mt->mass_elec();
	else if(carrier == P_Type)
		m = mt->mass_hole();    	

       	double ni =  Qi.v[0];  //
       	double nj =  Qj.v[0];  //
       	double vxi = Qi.v[1]/(ni*m);
	double vyi = Qi.v[2]/(ni*m);
        double vxj = Qj.v[1]/(nj*m);
	double vyj = Qj.v[2]/(nj*m);
	double pi = 2.0/3.0*(Qi.v[3] - 0.5*m*ni*(vxi*vxi+vyi*vyi));
	double ci = sqrt(5.0/3.0*pi/(ni*m)); 
	double pj = 2.0/3.0*(Qj.v[3] - 0.5*m*nj*(vxj*vxj+vyj*vyj));
	double cj = sqrt(5.0/3.0*pj/(nj*m));
	double a1 = sqrt(ni),a2 = sqrt(nj);
	double vxij = (a1*vxi + a2*vxj)/(a1+a2);
       	double vyij = (a1*vyi + a2*vyj)/(a1+a2);
       	double cij =  (a1*ci  + a2*cj)/(a1+a2);
       	double l   = fabs(vxij*cos(angle)+ vyij*sin(angle))+cij;   
       	if(carrier == N_Type)
       	{
       		Get_jacobian_matrix(Qi,Jx,Jy,N_Type);
       		Ai = 0.5*elen*(Jx*cos(angle) + Jy*sin(angle)+ l*I);
       		Get_jacobian_matrix(Qj,Jx,Jy,N_Type);
       		Aj = 0.5*elen*(Jx*cos(angle) + Jy*sin(angle)- l*I);
       	}
       	else if(carrier == P_Type)
       	{
       		Get_jacobian_matrix(Qi,Jx,Jy,P_Type);
       		Ai = 0.5*elen*(Jx*cos(angle) + Jy*sin(angle)+ l*I);
       		Get_jacobian_matrix(Qj,Jx,Jy,P_Type);
       		Aj = 0.5*elen*(Jx*cos(angle) + Jy*sin(angle)- l*I);
       	}
}


void SMCZone::build_right_const(int carrier)
{
	int size = pzone->davcell.size();
	matrix_double S;
	if(carrier==PN_Both||carrier==N_Type)
	  for(int i=0;i<size;i++)
	 {
		get_source_matrix(S,i,N_Type);
		aux[i].Qn_old = fs[i].Qn;
		aux[i].Rn_old =	-0.5*aux[i].Qn_flux+0.5*S*fs[i].Qn;
	 }

	if(carrier==PN_Both||carrier==P_Type)
	  for(int i=0;i<size;i++)
	 {
		get_source_matrix(S,i,P_Type);
		aux[i].Qp_old = fs[i].Qp;
		aux[i].Rp_old =	-0.5*aux[i].Qp_flux+0.5*S*fs[i].Qp;
	 }
}

void SMCZone::build_implicit_equation(SolveDefine &sv,Mat& A,Vec &b,DABC &bc,double dt,int carrier)
{
	int size = pzone->davcell.size();
	int index[4],col[4];
	double dtao;
	matrix_double I,J,Jr,Ji,S,M;
	Set_Matrix_I(I);
	Set_Matrix_zero(M);
	vector_double right;
	
	if(sv.Reconstruct==FirstOrder)
		first_order_reconst(bc,carrier);
	if(sv.Reconstruct==SecondOrder)
	 	MUSCL_reconst(bc,carrier);
	if(sv.Type==TRANSIENT) 
		dtao = 0.1*build_flux(bc,carrier);
	else 
		dtao = 1e38*build_flux(bc,carrier);
	for(int i=0;i<size;i++)
	{
		const VoronoiCell* pcell = pzone->davcell.GetPointer(i);
		index[0] = 4*i+0;   		index[1] = 4*i+1;
   		index[2] = 4*i+2;		index[3] = 4*i+3;
   		
   		//first type boundary point
   		if(pcell->bc_index&&bc[pcell->bc_index-1].BCType==OhmicContract)
   		{
   			Set_Matrix_I(J);
   			Set_Vector_zero(right);
   			VecSetValues(b,4,index,right.v,INSERT_VALUES);
   			MatSetValues(A,4,index,4,index,J.m,INSERT_VALUES);
   			continue;
   		}
   		
   		//inner point	    
   		
   		double V = pcell->area + pcell->garea;
   		   			
   		get_source_matrix(S,i,carrier);
   		J = V*(1/dt+1/dtao)*I-0.5*S;
   		if(carrier == N_Type)
			right = -V*(fs[i].Qn-aux[i].Qn_old)/dt
				+aux[i].Rn_old
				-0.5*aux[i].Qn_flux+0.5*S*fs[i].Qn;
		if(carrier == P_Type)
			right = -V*(fs[i].Qp-aux[i].Qp_old)/dt
				+aux[i].Rp_old 
				-0.5*aux[i].Qp_flux+0.5*S*fs[i].Qp;	
   		VecSetValues(b,4,index,right.v,INSERT_VALUES);
   		for(int j=0;j<pcell->nb_num;j++)
		{
			int nb = pcell->nb_array[j];
			if(carrier == N_Type)
				form_matrix(fs[i].Qn,fs[nb].Qn,
					       pcell->angle[j],pcell->elen[j],Ji,Jr,N_Type);
			if(carrier == P_Type)
				form_matrix(fs[i].Qp,fs[nb].Qp,
					       pcell->angle[j],pcell->elen[j],Ji,Jr,P_Type);
			J = J + 0.5*Ji;
			col[0] = 4*nb+0;	col[1] = 4*nb+1;
   			col[2] = 4*nb+2;	col[3] = 4*nb+3;
			Jr = 0.5*Jr;
   			MatSetValues(A,4,index,4,col,Jr.m,INSERT_VALUES);
   		}

   		//no-flux boundary point: process ghost cell
       		if(pcell->bc_index&&
          	   (bc[pcell->bc_index-1].BCType==NeumannBoundary||
           	    bc[pcell->bc_index-1].BCType==InsulatorContract||
           	    bc[pcell->bc_index-1].BCType==InsulatorInterface||
           	    bc[pcell->bc_index-1].BCType==SchottkyContract))
		  for (int j=0;j<pcell->mirror_num; j++)
	 	  {     
	 		int c = pcell->mirror[j].index;
	 		int nbindex = pcell->mirror[j].nbindex;
	 		double nx = pcell->mirror[j].nx;
	 		double ny = pcell->mirror[j].ny;
	 		vector_double ghost;

	 		M.m[0] = 1.0;
	 		M.m[5] = 1.0-2.0*nx*nx; M.m[6] = -2.0*nx*ny;
	 		M.m[9] = -2.0*nx*ny;  M.m[10] = 1.0-2.0*ny*ny;
	 		M.m[15] = 1.0;
	 		
	 		if(carrier==N_Type)
	 		{
	 		  ghost = M*fs[c].Qn;
	 		  matrix_double Jb;
	 		  form_matrix(fs[i].Qn,fs[c].Qn,pcell->angle[nbindex],
	 		  	      pcell->elen[nbindex],Ji,Jr,N_Type);
	 		  Jb = 0.5*Jr;
	 		  form_matrix(fs[i].Qn,ghost,pcell->mirror[j].angle,
	 		              pcell->mirror[j].elen,Ji,Jr,N_Type);
	 		  Jb = Jb + 0.5*Jr*M;
	 		  col[0] = 4*c+0;	col[1] = 4*c+1;
   			  col[2] = 4*c+2;	col[3] = 4*c+3;
	 		  MatSetValues(A,4,index,4,col,Jb.m,INSERT_VALUES);              
			}
			
		        if(carrier==P_Type)
		        {
		          ghost = M*fs[c].Qp;
			  matrix_double Jb;
			  form_matrix(fs[i].Qp,fs[c].Qp,pcell->angle[nbindex],
			              pcell->elen[nbindex],Ji,Jr,P_Type);
			  Jb = 0.5*Jr; 
			  form_matrix(fs[i].Qp,ghost,pcell->mirror[j].angle,
	 		              pcell->mirror[j].elen,Ji,Jr,P_Type);
	 		  Jb = Jb + 0.5*Jr*M;
	 		  col[0] = 4*c+0;	col[1] = 4*c+1;
   			  col[2] = 4*c+2;	col[3] = 4*c+3;   
   			  MatSetValues(A,4,index,4,col,Jb.m,INSERT_VALUES);       
			}	  
			J = J + 0.5*Ji;
	          }
	      	
	       MatSetValues(A,4,index,4,index,J.m,INSERT_VALUES);				  
   	}
   	VecAssemblyBegin(b);
        VecAssemblyEnd(b);
   	MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);
  	MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);
}   


void SMCZone::solution_update(Vec &x,int carrier)
{
	int size = pzone->davcell.size();
	PetscScalar    *data;
	VecGetArray(x,&data);
	if(carrier == N_Type)
	  for(int i=0;i<size;i++)
	 {

		fs[i].Qn.v[0]+=data[4*i+0];
		fs[i].Qn.v[1]+=data[4*i+1];
		fs[i].Qn.v[2]+=data[4*i+2];
		fs[i].Qn.v[3]+=data[4*i+3];
	 }

	if(carrier == P_Type)
	  for(int i=0;i<size;i++)
	 {
		fs[i].Qp.v[0]+=data[4*i+0];
		fs[i].Qp.v[1]+=data[4*i+1];
		fs[i].Qp.v[2]+=data[4*i+2];
		fs[i].Qp.v[3]+=data[4*i+3];
	 }
	VecRestoreArray(x,&data);
}


double SMCZone::build_flux(DABC &bc,int carrier)
{
  double CFL=1e38;
  vector_double sf_flux_n,sf_flux_p;
  for(int i=0;i<pzone->davedge.size();i++)
  {
	 const VoronoiCell *cell1 = pzone->davcell.GetPointer(pzone->davedge[i].cell1);
	 const VoronoiCell *cell2 = pzone->davcell.GetPointer(pzone->davedge[i].cell2);
	 int cell1_index = pzone->davedge[i].cell1;
	 int cell2_index = pzone->davedge[i].cell2;	
	 int c1_nb =  pzone->davedge[i].c1_nb;		//cell2 index in cell1's nb_array
	 int c2_nb =  pzone->davedge[i].c2_nb;		//cell1 index in cell2's nb_array
	 	 
	if(carrier==N_Type||carrier==PN_Both)
	{
	  double me1=mt->mass_elec(), me2=mt->mass_elec();
	  sf_flux_n = flux_function(aux[cell1_index].Qn_r[c1_nb],aux[cell2_index].Qn_r[c2_nb],
		       	    cell1->elen[c1_nb],cell1->ilen[c1_nb],cell1->angle[c1_nb],me1,me2,CFL);
	  aux[cell1_index].Qn_s[c1_nb] = sf_flux_n;
	  aux[cell2_index].Qn_s[c2_nb] = -1.0*sf_flux_n;	       	    
        }
        if(carrier==P_Type||carrier==PN_Both)
        {
          double mh1=mt->mass_hole(), mh2=mt->mass_hole();	
          sf_flux_p = flux_function(aux[cell1_index].Qp_r[c1_nb],aux[cell2_index].Qp_r[c2_nb],
			    cell1->elen[c1_nb],cell1->ilen[c1_nb],cell1->angle[c1_nb],mh1,mh2,CFL); 
	  aux[cell1_index].Qp_s[c1_nb] = sf_flux_p;
	  aux[cell2_index].Qp_s[c2_nb] = -1.0*sf_flux_p;	   		  
	}	  
   }
   
   for(int i=0;i<node_num;i++)
   {
       Set_Vector_zero(aux[i].Qn_flux);
       Set_Vector_zero(aux[i].Qp_flux);
       const VoronoiCell* pcell = pzone->davcell.GetPointer(i);
       //inside point
       for (int j=0;j<pcell->nb_num; j++)
       {       
		if(carrier==N_Type||carrier==PN_Both)
			aux[i].Qn_flux = aux[i].Qn_flux + aux[i].Qn_s[j];
	
	 	if(carrier==P_Type||carrier==PN_Both)
			aux[i].Qp_flux = aux[i].Qp_flux + aux[i].Qp_s[j];
       }
       
       //boundary point: process ghost cell
       if(pcell->bc_index&&
          (bc[pcell->bc_index-1].BCType==NeumannBoundary||
           bc[pcell->bc_index-1].BCType==InsulatorContract||
           bc[pcell->bc_index-1].BCType==InsulatorInterface||
           bc[pcell->bc_index-1].BCType==SchottkyContract))
	{
	
		for (int j=0;j<pcell->mirror_num; j++)
	 	{     
	 		int c=pcell->mirror[j].index;
	 		int iedge=pcell->mirror[j].nbindex;
	 		int cedge=pcell->inb_array[iedge];
	 		double nx = pcell->mirror[j].nx;
	 		double ny = pcell->mirror[j].ny;
	 		vector_double ghost; 
	 		matrix_double M;
	 		Set_Matrix_zero(M);
	 		M.m[0] = 1; 
	 		M.m[5] = 1-2*nx*nx; M.m[6] = -2*nx*ny;
	 		M.m[9] = -2*nx*ny;  M.m[10] = 1-2*ny*ny;
	 		M.m[15] = 1;
	 		 
	 		if(carrier==N_Type||carrier==PN_Both)
	 		{
	 		  double me1=mt->mass_elec(), me2=mt->mass_elec();
	 		  ghost = M*aux[c].Qn_r[cedge];	
	 		  aux[i].Qn_flux =aux[i].Qn_flux+flux_function(aux[i].Qn_r[iedge],ghost,
				  pcell->mirror[j].elen,pcell->mirror[j].ilen,
				  pcell->mirror[j].angle,me1,me2,CFL);
			}	  
		        if(carrier==P_Type||carrier==PN_Both)
		        {
		          double mh1=mt->mass_hole(), mh2=mt->mass_hole();
		          ghost = M*aux[c].Qp_r[cedge];		
	 		  aux[i].Qp_flux =aux[i].Qp_flux+flux_function(aux[i].Qp_r[iedge],ghost,
				  pcell->mirror[j].elen,pcell->mirror[j].ilen,
				  pcell->mirror[j].angle,mh1,mh2,CFL);
			}	  
		}		  
	  }	        
	 	
  }
  return CFL;
}	 
	   

/*
//-------------------------------------------------------------------------------
double SMCZone::build_flux(DABC &bc,int carrier)
{
	double CFL=1e38;
	
	for(int i=0;i<node_num;i++)
	{
	
	 VoronoiCell pcell = pzone->davcell[i];
	 Set_Vector_zero(aux[i].Qn_flux);
	 Set_Vector_zero(aux[i].Qp_flux);
	 if(!pcell.bc_index || bc[pcell.bc_index-1].BCType==OhmicContract)
	 for (int j=0;j<pcell.nb_num; j++)
	 	{       
	 		
	 		int c = pcell.nb_array[j];
	 		int cedge = pcell.inb_array[j];
	 		if(carrier==N_Type||carrier==PN_Both)
	 		{
	 		  double me1=mt->mass_elec(), me2=mt->mass_elec();
	 		  aux[i].Qn_flux = aux[i].Qn_flux+flux_function(aux[i].Qn_r[j],aux[c].Qn_r[cedge],
			       	  pcell.elen[j],pcell.ilen[j],pcell.angle[j],me1,me2,CFL);
		        }
		        if(carrier==P_Type||carrier==PN_Both)
		        {
		          double mh1=mt->mass_hole(), mh2=mt->mass_hole();	
		          aux[i].Qp_flux = aux[i].Qp_flux+flux_function(aux[i].Qp_r[j],aux[c].Qp_r[cedge],
				  pcell.elen[j],pcell.ilen[j],pcell.angle[j],mh1,mh2,CFL); 
			}	  
		}
				  
	  else if(bc[pcell.bc_index-1].BCType==InsulatorInterface||
	          bc[pcell.bc_index-1].BCType==NeumannBoundary)
	  {
	  	for (int j=0;j<pcell.nb_num; j++)
	 	{       
	 		int c = pcell.nb_array[j];
	 		int cedge = pcell.inb_array[j];
	 		if(carrier==N_Type||carrier==PN_Both)
	 		{
	 		  double me1=mt->mass_elec(), me2=mt->mass_elec();
	 		  aux[i].Qn_flux = aux[i].Qn_flux+flux_function(aux[i].Qn_r[j],aux[c].Qn_r[cedge],
			       	  pcell.elen[j],pcell.ilen[j],pcell.angle[j],me1,me2,CFL);
		        }
		        if(carrier==P_Type||carrier==PN_Both)
		        {
		          double mh1=mt->mass_hole(), mh2=mt->mass_hole();	
		          aux[i].Qp_flux = aux[i].Qp_flux+flux_function(aux[i].Qp_r[j],aux[c].Qp_r[cedge],
				  pcell.elen[j],pcell.ilen[j],pcell.angle[j],mh1,mh2,CFL); 
			}	  
		}		  
		for (int j=0;j<pcell.mirror_num; j++)
	 	{     
	 		int c=pcell.mirror[j].index;
	 		int iedge=pcell.mirror[j].nbindex;
	 		int cedge=pcell.inb_array[iedge];
	 		vector_double ghost;
	 		if(carrier==N_Type||carrier==PN_Both)
	 		{
	 		  double me1=mt->mass_elec(), me2=mt->mass_elec();
	 		  ghost.v[0]=aux[c].Qn_r[cedge].v[0];
	 		  ghost.v[1]=aux[c].Qn_r[cedge].v[1]-2*(aux[c].Qn_r[cedge].v[1]*pcell.mirror[j].nx+
	 			aux[c].Qn_r[cedge].v[2]*pcell.mirror[j].ny)*pcell.mirror[j].nx;
	 		  ghost.v[2]=aux[c].Qn_r[cedge].v[2]-2*(aux[c].Qn_r[cedge].v[1]*pcell.mirror[j].nx+
	 			aux[c].Qn_r[cedge].v[2]*pcell.mirror[j].ny)*pcell.mirror[j].ny;
	 		  ghost.v[3]=aux[c].Qn_r[cedge].v[3];	
	 			
	 		  aux[i].Qn_flux =aux[i].Qn_flux+flux_function(aux[i].Qn_r[iedge],ghost,
				  pcell.mirror[j].elen,pcell.mirror[j].ilen,
				  pcell.mirror[j].angle,me1,me2,CFL);
			}	  
		        if(carrier==P_Type||carrier==PN_Both)
		        {
		          double mh1=mt->mass_hole(), mh2=mt->mass_hole();
		          ghost.v[0]=aux[c].Qp_r[cedge].v[0];
	 		  ghost.v[1]=aux[c].Qp_r[cedge].v[1]-2*(aux[c].Qp_r[cedge].v[1]*pcell.mirror[j].nx+
	 			aux[c].Qp_r[cedge].v[2]*pcell.mirror[j].ny)*pcell.mirror[j].nx;
	 		  ghost.v[2]=aux[c].Qp_r[cedge].v[2]-2*(aux[c].Qp_r[cedge].v[1]*pcell.mirror[j].nx+
	 			aux[c].Qp_r[cedge].v[2]*pcell.mirror[j].ny)*pcell.mirror[j].ny;
	 		  ghost.v[3]=aux[c].Qp_r[cedge].v[3];	
	 			
	 		  aux[i].Qp_flux =aux[i].Qp_flux+flux_function(aux[i].Qp_r[iedge],ghost,
				  pcell.mirror[j].elen,pcell.mirror[j].ilen,
				  pcell.mirror[j].angle,mh1,mh2,CFL);
			}	  
		}		  
	  }	        

	 }
	 return CFL;
}
*/

void SMCZone::Flux_explicit_update(DABC &bc,int carrier,double dt)
{
	 for(int i=0;i<node_num;i++)
	  if(pzone->davcell[i].bc_index==0||(pzone->davcell[i].bc_index&&
              (bc[pzone->davcell[i].bc_index-1].BCType==NeumannBoundary||
               bc[pzone->davcell[i].bc_index-1].BCType==InsulatorContract||
               bc[pzone->davcell[i].bc_index-1].BCType==InsulatorInterface||
               bc[pzone->davcell[i].bc_index-1].BCType==SchottkyContract)))
	  {
	 	double S = pzone->davcell[i].area + pzone->davcell[i].garea;
	 	if(carrier==N_Type||carrier==PN_Both)
	 		fs[i].Qn=fs[i].Qn-dt*aux[i].Qn_flux/S;
		if(carrier==P_Type||carrier==PN_Both)
			fs[i].Qp=fs[i].Qp-dt*aux[i].Qp_flux/S;
	 }
}

void SMCZone::set_ohmic_boundary(DABC &bc,int carrier)
{
	int size = pzone->davcell.size();
	for(int i=0;i<size;i++)
	{
	  const VoronoiCell* pcell = pzone->davcell.GetPointer(i);
	
   	  //first type boundary point
   	  if(pcell->bc_index&&bc[pcell->bc_index-1].BCType==OhmicContract)
   	    
   	  {
   		 double nie = mt->ni(fs[i].T);
	     	 if((fs[i].Nd-fs[i].Na)>0)   //n-type
		 {
	  		fs[i].n=fs[i].Qn.v[0]=((fs[i].Nd-fs[i].Na)+sqrt(pow(fs[i].Nd-fs[i].Na,2)+4*nie*nie))/2;
			fs[i].p=fs[i].Qp.v[0]=nie*nie/fs[i].Qn.v[0];
	 	 }
	 	 else		       	     //p-type
	 	 {
	 		fs[i].p=fs[i].Qp.v[0]=((fs[i].Na-fs[i].Nd)+sqrt(pow(fs[i].Na-fs[i].Nd,2)+4*nie*nie))/2;
	 		fs[i].n=fs[i].Qn.v[0]=nie*nie/fs[i].Qp.v[0];
	 	 }
	 	
	  	//fs[i].Qn.v[2]=sqrt(fs[i].Qn.v[1]*fs[i].Qn.v[1]+fs[i].Qn.v[2]*fs[i].Qn.v[2]);	 	
	 	//fs[i].Qn.v[1]=0.0;
	 	fs[i].Qn.v[3]=1.5*fs[i].Qn.v[0]*mt->k*fs[i].T+
	 	           0.5*(fs[i].Qn.v[1]*fs[i].Qn.v[1]+fs[i].Qn.v[2]*fs[i].Qn.v[2])/mt->mass_elec()/fs[i].Qn.v[0];
	 	
	 	//fs[i].Qp.v[2]=sqrt(fs[i].Qp.v[1]*fs[i].Qp.v[1]+fs[i].Qp.v[2]*fs[i].Qp.v[2]);	 
	 	//fs[i].Qp.v[1]=0;
	 	fs[i].Qp.v[3]=1.5*fs[i].Qp.v[0]*mt->k*fs[i].T+
	 	           0.5*(fs[i].Qp.v[1]*fs[i].Qp.v[1]+fs[i].Qp.v[2]*fs[i].Qp.v[2])/mt->mass_hole()/fs[i].Qp.v[0];
	  }
	}	 	
}
