#include "imhdm.h"
#include <assert.h>

int Im_HDM_Solver::init_solver()
{
	printf("HDM solver init...");
	int smcz_num = 0;
	for(int i=0;i<zone_num;i++)
	  if(zonedata[i]->material_type == semiconductor)
	  {
	  	pzonedata = dynamic_cast< SMCZone * >(zonedata[i]);
	  	smcz_num++;
	  }
	if(smcz_num !=1)
		{printf("Implicit HDM mthod can only deal with one semiconductor zone\n");return 1;}

	pzone = pzonedata->pzone;

	N = 4*pzone->davcell.size();

	// pre-alloc approximate memory
	int *nnz = new int[N];
  	int *p = nnz;
	for(int i=0;i<pzone->davcell.size();i++)
        {
        	*p++ = 4*(pzone->davcell[i].nb_num+1);
        	*p++ = 4*(pzone->davcell[i].nb_num+1);
        	*p++ = 4*(pzone->davcell[i].nb_num+1);
        	*p++ = 4*(pzone->davcell[i].nb_num+1);
        }

        VecCreate(PETSC_COMM_WORLD,&x);
        VecSetSizes(x,PETSC_DECIDE,N);
        VecSetType(x, VECSEQ);
        VecDuplicate(x,&b);
        MatCreateSeqAIJ(PETSC_COMM_SELF,N,N,0,nnz,&A);
        MatSetFromOptions(A);

        KSPCreate(PETSC_COMM_WORLD,&ksp);
	KSPSetOperators(ksp,A,A,SAME_NONZERO_PATTERN);
	KSPSetType(ksp,KSPBCGS);
	KSPGetPC(ksp,&pc);
	PCSetType(pc,PCILU);
        KSPSetTolerances(ksp,PETSC_DEFAULT,1e-12*N,PETSC_DEFAULT,200);
        KSPSetFromOptions(ksp);
        delete [] nnz;
        printf("done\n");
        return 0;
}

int Im_HDM_Solver::destroy_solver()
{
	VecDestroy(b);
	VecDestroy(x);
	MatDestroy(A);
	KSPDestroy(ksp);
	return 0;
}





int Im_HDM_Solver::do_solve(SolveDefine &sv)
{
	PetscReal  residual_norm,total_norm,init_norm;
	PetscInt   its,total_its=0;
	PetscErrorCode ierr;
	pzonedata->set_flux_function(sv.FluxFunc);

	init_poisson();

	if(sv.Type!=TRANSIENT)
	{
		printf("HDM mthod can only do TRANSIENT computing\n");
		return 1 ;
	}

	if(sv.Type==TRANSIENT)
	{
		bc.Set_Vapp(0);

		while(clock<sv.TStop)
		{
	 		printf("t = %e\n",clock);

	 		bc.Set_Vapp(clock);
	 		if(sv.Reconstruct==FirstOrder)
	 			pzonedata->first_order_reconst(bc,Carrier);
	 		if(sv.Reconstruct==SecondOrder)
	 			pzonedata->MUSCL_reconst(bc,Carrier);
	  		dt = sv.CFL*pzonedata->build_flux(bc,Carrier);
	   		do_poisson();

	 		pzonedata->build_right_const(Carrier);

	 		total_its = 0;
	 		init_norm = 0.0;
			// main its loop
       			do{
       			   do_poisson();
       			   total_norm = 0.0;
       			   if(Carrier==N_Type||Carrier==PN_Both)
       			   {
       				pzonedata->build_implicit_equation(sv,A,b,bc,dt,N_Type);
       				//VecNorm(b,NORM_1,&residual_norm);
       				//printf("%e ",residual_norm);
       				KSPSolve(ksp,b,x);
				pzonedata->solution_update(x,N_Type);
				KSPGetIterationNumber(ksp,&its);
				printf("\t elec marching: inner iteration %d\t",its);
	       			VecNorm(x,NORM_1,&residual_norm);
	       			if(total_its==0) init_norm += residual_norm;
				total_norm+=residual_norm;
				printf("norm = %e\n",residual_norm);
			   }
			   //do_poisson();
			   if(Carrier==P_Type||Carrier==PN_Both)
       			   {
       			 	pzonedata->build_implicit_equation(sv,A,b,bc,dt,P_Type);
       			 	//VecNorm(b,NORM_1,&residual_norm);
       				//printf("%e ",residual_norm);
       				KSPSolve(ksp,b,x);
				pzonedata->solution_update(x,P_Type);
				KSPGetIterationNumber(ksp,&its);
				printf("\t hole marching: inner iteration %d\t",its);
	       			VecNorm(x,NORM_1,&residual_norm);
	       			if(total_its==0) init_norm += residual_norm;
				total_norm+=residual_norm;
				printf("norm = %e\n",residual_norm);
			   }

			   pzonedata->set_ohmic_boundary(bc,Carrier);
        		   total_its++;

       			} while(total_norm>1e-6*init_norm&&total_its<300);
       			printf("IterationNumber %d, residual norm %e\n\n",total_its,total_norm);
			clock+=dt;
		}

	}

	destroy_poisson();
	return 0;

}
