#include "solver.h"
#include <assert.h>
#include <cgnslib.h>

//simple error handle function
inline int cgns_error_handle(char *s,int fn)
{
	printf("%s\n",s);
	cg_close(fn);
        return 1;
}



int SMCZone::import_doping(char *cgnsfile,double mesh_scale)
{
	int  Z = zone_index+1,fn,B=1;
	int  isize[3],nsol;
	char zonelabel[32],solname[32];
	GridLocation_t location;

	int node_num = pzone->davcell.size();

	double *Na,*Nd;
	Na = new double[node_num];
	Nd = new double[node_num];

	cg_open(cgnsfile,MODE_READ,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);

	printf("read Doping...");
	//---------------------------------------------------------------------------------------
	//how many fieldsol node?
	assert(!cg_nsols(fn,B,Z,&nsol));
	if(nsol == 0)
		return cgns_error_handle("no fieldsol node,not a supported cgns file",fn);
	//search for the "doping" FlowSolution_t
	int flag = 0;
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,solname,&location));
	   if(!strcmp(solname,"Doping"))
	   {    int imin = 1;
	        flag = 1;
	        printf("done\n");
	      	assert(!cg_field_read(fn,B,Z,i,"Nd",RealDouble,&imin,&isize[0],Nd));
                assert(!cg_field_read(fn,B,Z,i,"Na",RealDouble,&imin,&isize[0],Na));
           }
        }

 	if(flag == 0 )
 		return cgns_error_handle("no doping found",fn);
 	else
 		cg_close(fn);

 	for(int i=0;i<node_num;i++)
        {	//also,doping will be scaled. nagtive value convert to positive
        	fs[i].Nd = fabs(Nd[i])/pow(mesh_scale,3);
        	fs[i].Na = fabs(Na[i])/pow(mesh_scale,3);
        }

        delete [] Na;
	delete [] Nd;
	return 0;

}


int SMCZone::export_doping(char *cgnsfile,double mesh_scale)
{
	int  Z = zone_index+1,fn,B=1,SOL,F;
	int  isize[3],nsol;
	char zonelabel[32],solname[32];
	GridLocation_t location;

	int node_num = pzone->davcell.size();

	double *Na,*Nd;
	Na = new double[node_num];
	Nd = new double[node_num];

	for(int i=0;i<node_num;i++)
        {
        	Nd[i] = fs[i].Nd*pow(mesh_scale,3);
        	Na[i] = fs[i].Na*pow(mesh_scale,3);
        }

	cg_open(cgnsfile,MODE_MODIFY,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);

	//---------------------------------------------------------------------------------------
	//search for FlowSolution_t if the "Doping" is exit
	assert(!cg_nsols(fn,B,Z,&nsol));
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,solname,&location));
	   if(!strcmp(solname,"Doping"))
	   {
	   	printf("waring !!! doping exit,delete it\n");
	   	assert(!cg_goto(fn,B,"Zone_t",Z,"end"));
	   	assert(!cg_delete_node("Doping"));
	   }
        }

 	//write field solution

 	assert(!cg_sol_write(fn,B,Z,"Doping",Vertex,&SOL));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"Na",Na,&F));
  	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"Nd",Nd,&F));

 	cg_close(fn);

        delete [] Na;
	delete [] Nd;
	return 0;

}


int SMCZone::export_solution(char *cgnsfile,char *solname,double mesh_scale)
{
	int  Z = zone_index+1,fn,B=1,SOL,F;
	int  isize[3],nsol;
	char zonelabel[32],sol_exist[32];
	GridLocation_t location;
	double me=mt->mass_elec(), mh=mt->mass_hole();
	int node_num = pzone->davcell.size();

	double *n,*p,*P,*T,*Vxn,*Vyn,*Vxp,*Vyp,*En,*Ep;

	n  = new double[node_num];
	p  = new double[node_num];
	Vxn = new double[node_num];
	Vyn = new double[node_num];
	Vxp = new double[node_num];
	Vyp = new double[node_num];
	P  = new double[node_num];
	En = new double[node_num];
	Ep = new double[node_num];
	T  = new double[node_num];

	for(int i=0;i<node_num;i++)
        {
        	n[i]   = fs[i].Qn.v[0]*pow(mesh_scale,3);		     //density unit cm-3
        	p[i]   = fs[i].Qp.v[0]*pow(mesh_scale,3);
        	P[i]   = fs[i].P;
		T[i]   = fs[i].T*300.0;

		En[i]  = fs[i].Qn.v[3];///n[i];      //energy eV
		Ep[i]  = fs[i].Qp.v[3];///p[i];      //energy eV
        	Vxn[i] = fs[i].Qn.v[1];///(n[i]*me)*1e8;  //velocity unit cm/s
		Vyn[i] = fs[i].Qn.v[2];///(n[i]*me)*1e8;
		Vxp[i] = fs[i].Qp.v[1];///(p[i]*mh)*1e8;
		Vyp[i] = fs[i].Qp.v[2];///(p[i]*mh)*1e8;
        }

	cg_open(cgnsfile,MODE_MODIFY,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);
	//------------------------------------------------------------
	//search for FlowSolution_t if the "solution" is exit
	assert(!cg_nsols(fn,B,Z,&nsol));
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,sol_exist,&location));
	   if(!strcmp(sol_exist,solname))
	   {
	   	printf("waring !!! Solution exit,delete it\n");
	   	assert(!cg_goto(fn,B,"Zone_t",Z,"end"));
	   	assert(!cg_delete_node(solname));
	   }
        }
        //-----------------------------------------------------------
 	assert(!cg_sol_write(fn,B,Z,solname,Vertex,&SOL));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"elec_density",n,&F));
  	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"hole_density",p,&F));
  	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"potential",P,&F));
  	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"lattice_temperature",T,&F));
  	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"elec_velocity_x",Vxn,&F));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"elec_velocity_y",Vyn,&F));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"hole_velocity_x",Vxp,&F));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"hole_velocity_y",Vyp,&F));

 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"elec_energy",En,&F));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"hole_energy",Ep,&F));


  	cg_close(fn);

        delete [] n;	delete [] p;
	delete [] Vxn;	delete [] Vyn;
	delete [] Vxp;	delete [] Vyp;
	delete [] P;	delete [] En;
	delete [] Ep;	delete [] T;
	return 0;

}

int ElZone::export_solution(char *cgnsfile,char *solname,double mesh_scale)
{
	int  Z = zone_index+1,fn,B=1,SOL,F;
	int  isize[3],nsol;
	char zonelabel[32],sol_exist[32];
	GridLocation_t location;

	int node_num = pzone->davcell.size();

	double *P,*T;

	P  = new double[node_num];
	T  = new double[node_num];

	for(int i=0;i<node_num;i++)
        {
	       	P[i]   = fs[i].P;
		T[i]   = fs[i].T*300.0;
        }

	cg_open(cgnsfile,MODE_MODIFY,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);
	//------------------------------------------------------------
	//search for FlowSolution_t if the "solution" is exit
	assert(!cg_nsols(fn,B,Z,&nsol));
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,sol_exist,&location));
	   if(!strcmp(sol_exist,solname))
	   {
	   	printf("waring !!! Solution exit,delete it\n");
	   	assert(!cg_goto(fn,B,"Zone_t",Z,"end"));
	   	assert(!cg_delete_node(solname));
	   }
        }
        //-----------------------------------------------------------
 	assert(!cg_sol_write(fn,B,Z,solname,Vertex,&SOL));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"potential",P,&F));
  	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"lattice_temperature",T,&F));

  	cg_close(fn);

    	delete [] P;
	delete [] T;
	return 0;

}

int ISZone::export_solution(char *cgnsfile,char *solname,double mesh_scale)
{
	int  Z = zone_index+1,fn,B=1,SOL,F;
	int  isize[3],nsol;
	char zonelabel[32],sol_exist[32];
	GridLocation_t location;

	int node_num = pzone->davcell.size();
	double *P,*T;

	P  = new double[node_num];
	T  = new double[node_num];

	for(int i=0;i<node_num;i++)
        {
	       	P[i]   = fs[i].P;
		T[i]   = fs[i].T*300.0;
        }

	cg_open(cgnsfile,MODE_MODIFY,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);
	//------------------------------------------------------------
	//search for FlowSolution_t if the "solution" is exit
	assert(!cg_nsols(fn,B,Z,&nsol));
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,sol_exist,&location));
	   if(!strcmp(sol_exist,solname))
	   {
	   	printf("waring !!! Solution exit,delete it\n");
	   	assert(!cg_goto(fn,B,"Zone_t",Z,"end"));
	   	assert(!cg_delete_node(solname));
	   }
        }
        //-----------------------------------------------------------
 	assert(!cg_sol_write(fn,B,Z,solname,Vertex,&SOL));
 	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"potential",P,&F));
  	assert(!cg_field_write(fn,B,Z,SOL,RealDouble,"lattice_temperature",T,&F));

  	cg_close(fn);

    	delete [] P;
	delete [] T;
	return 0;

}



int SMCZone::import_solution(char *cgnsfile,char *solname,double mesh_scale)
{
	int  Z = zone_index+1,fn,B=1;
	int  isize[3],nsol;
	char zonelabel[32],sol[32];
	GridLocation_t location;
	double me=mt->mass_elec(), mh=mt->mass_hole();
	int node_num = pzone->davcell.size();

	double *n,*p,*P,*T,*Vxn,*Vyn,*Vxp,*Vyp,*En,*Ep;

	n  = new double[node_num];
	p  = new double[node_num];
	Vxn = new double[node_num];
	Vyn = new double[node_num];
	Vxp = new double[node_num];
	Vyp = new double[node_num];
	P  = new double[node_num];
	En = new double[node_num];
	Ep = new double[node_num];
	T  = new double[node_num];


	cg_open(cgnsfile,MODE_READ,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);

	printf("read Solution...");
	//---------------------------------------------------------------------------------------
	//how many fieldsol node?
	assert(!cg_nsols(fn,B,Z,&nsol));
	if(nsol == 0)
		return cgns_error_handle("no fieldsol node,not a supported CoreFile",fn);
	//search for the "solution" FlowSolution_t
	int flag = 0;
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,sol,&location));
	   if(!strcmp(sol,solname))
	   {    int imin = 1;
	        flag = 1;
		printf("done\n");
  	        assert(!cg_field_read(fn,B,Z,i,"elec_density",RealDouble,&imin,&isize[0],n));
                assert(!cg_field_read(fn,B,Z,i,"hole_density",RealDouble,&imin,&isize[0],p));
                assert(!cg_field_read(fn,B,Z,i,"lattice_temperature",RealDouble,&imin,&isize[0],T));
                assert(!cg_field_read(fn,B,Z,i,"potential",RealDouble,&imin,&isize[0],P));

                assert(!cg_field_read(fn,B,Z,i,"elec_velocity_x",RealDouble,&imin,&isize[0],Vxn));
                assert(!cg_field_read(fn,B,Z,i,"elec_velocity_y",RealDouble,&imin,&isize[0],Vyn));
                assert(!cg_field_read(fn,B,Z,i,"hole_velocity_x",RealDouble,&imin,&isize[0],Vxp));
                assert(!cg_field_read(fn,B,Z,i,"hole_velocity_y",RealDouble,&imin,&isize[0],Vyp));

                assert(!cg_field_read(fn,B,Z,i,"elec_energy",RealDouble,&imin,&isize[0],En));
                assert(!cg_field_read(fn,B,Z,i,"hole_energy",RealDouble,&imin,&isize[0],Ep));

           }
        }
 	for(int i=0;i<node_num;i++)
        {
        	fs[i].n = n[i]/pow(mesh_scale,3);
        	fs[i].p = p[i]/pow(mesh_scale,3);
               	fs[i].P = P[i];
		fs[i].T = T[i]/300.0;

		fs[i].Qn.v[0] = fs[i].n;
		fs[i].Qp.v[0] = fs[i].p;

		fs[i].Qn.v[1] = Vxn[i];//*fs[i].n*me/1e8;
		fs[i].Qn.v[2] = Vyn[i];//*fs[i].n*me/1e8;
		fs[i].Qp.v[1] = Vxp[i];//*fs[i].p*mh/1e8;
		fs[i].Qp.v[2] = Vyp[i];//*fs[i].p*mh/1e8;
		fs[i].Qn.v[3] = En[i];//*fs[i].n;
		fs[i].Qp.v[3] = Ep[i];//*fs[i].p;

        }

        delete [] n;	delete [] p;
	delete [] Vxn;	delete [] Vyn;
	delete [] Vxp;	delete [] Vyp;
	delete [] P;	delete [] En;
	delete [] Ep;	delete [] T;

 	if(flag == 0 )
 		return cgns_error_handle("no Core Data found",fn);
 	else
 		cg_close(fn);

	return 0;
}

int ISZone::import_solution(char *cgnsfile,char *solname,double mesh_scale)
{
	int  Z = zone_index+1,fn,B=1;
	int  isize[3],nsol;
	char zonelabel[32],sol[32];
	GridLocation_t location;
	int node_num = pzone->davcell.size();

	double *n,*p,*P,*T,*Vxn,*Vyn,*Vxp,*Vyp,*En,*Ep;

	P  = new double[node_num];
	T  = new double[node_num];


	cg_open(cgnsfile,MODE_READ,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);

	printf("read Solution...");
	//---------------------------------------------------------------------------------------
	//how many fieldsol node?
	assert(!cg_nsols(fn,B,Z,&nsol));
	if(nsol == 0)
		return cgns_error_handle("no fieldsol node,not a supported CoreFile",fn);
	//search for the "solution" FlowSolution_t
	int flag = 0;
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,sol,&location));
	   if(!strcmp(sol,solname))
	   {    int imin = 1;
	        flag = 1;
		printf("done\n");
  	        assert(!cg_field_read(fn,B,Z,i,"lattice_temperature",RealDouble,&imin,&isize[0],T));
                assert(!cg_field_read(fn,B,Z,i,"potential",RealDouble,&imin,&isize[0],P));
           }
        }
 	for(int i=0;i<node_num;i++)
        {
        	fs[i].Na = 0;
        	fs[i].Nd = 0;
        	fs[i].n = 0;
        	fs[i].p = 0;
               	fs[i].P = P[i];
		fs[i].T = T[i]/300.0;

		Set_Vector_zero(fs[i].Qn);
		Set_Vector_zero(fs[i].Qp);

        }

	delete [] P;
	delete [] T;

 	if(flag == 0 )
 		return cgns_error_handle("no Core Data found",fn);
 	else
 		cg_close(fn);

	return 0;
}

int ElZone::import_solution(char *cgnsfile,char *solname,double mesh_scale)
{

	int  Z = zone_index+1,fn,B=1;
	int  isize[3],nsol;
	char zonelabel[32],sol[32];
	GridLocation_t location;
	int node_num = pzone->davcell.size();

	double *n,*p,*P,*T,*Vxn,*Vyn,*Vxp,*Vyp,*En,*Ep;

	P  = new double[node_num];
	T  = new double[node_num];


	cg_open(cgnsfile,MODE_READ,&fn);
	cg_zone_read(fn,B,Z,zonelabel,isize);

	printf("read Solution...");
	//---------------------------------------------------------------------------------------
	//how many fieldsol node?
	assert(!cg_nsols(fn,B,Z,&nsol));
	if(nsol == 0)
		return cgns_error_handle("no fieldsol node,not a supported CoreFile",fn);
	//search for the "solution" FlowSolution_t
	int flag = 0;
	for(int i=1;i<=nsol;i++)
	{
	   assert(!cg_sol_info(fn,B,Z,i,sol,&location));
	   if(!strcmp(sol,solname))
	   {    int imin = 1;
	        flag = 1;
		printf("done\n");
  	        assert(!cg_field_read(fn,B,Z,i,"lattice_temperature",RealDouble,&imin,&isize[0],T));
                assert(!cg_field_read(fn,B,Z,i,"potential",RealDouble,&imin,&isize[0],P));
           }
        }
 	for(int i=0;i<node_num;i++)
        {
        	fs[i].Na = 0;
        	fs[i].Nd = 0;
        	fs[i].n = 0;
        	fs[i].p = 0;
               	fs[i].P = P[i];
		fs[i].T = T[i]/300.0;

		Set_Vector_zero(fs[i].Qn);
		Set_Vector_zero(fs[i].Qp);

        }

	delete [] P;
	delete [] T;

 	if(flag == 0 )
 		return cgns_error_handle("no Core Data found",fn);
 	else
 		cg_close(fn);

	return 0;
}
