#include "solver.h"
#include "xgraph.h"
#include "grafix3d.h"

static TriPlot3D* triplot;

inline double FunLinear(double value)
{
	return value;
}

inline double FunSignedLog10(double value)
{
	return dsign(value)*log10(1.0+fabs(value));
}


///////////////////////////////////////////////////////////////////////////////
void Redraw()
{
  triplot->PlotGrid();
  triplot->PlotMesh();
  triplot->PlotData(TRUE, TRUE);
}

void TitleEtc ( PlotDefine &plot, int x1, int y1, int ResFactor )
{

  char txt[64];
  int ix;
  int iy = int(CHARPIXELHIGH * ResFactor * 1.2);

  if      (plot.Resolution == RES_640x480)  ix = 56;
  else if (plot.Resolution == RES_1024x768) ix = 64;
  else if (plot.Resolution == RES_800x600)  ix = 96;

  x1 += 4 * ResFactor * CHARPIXELWIDE;
  y1 *= ResFactor;
  sprintf(txt, "GSS 3D graphic output");
  GRText(ix + x1, y1 += iy, txt);
  if(plot.Variable == Potential)
  	sprintf(txt, "Variable : Potential");
  else if(plot.Variable == Na)
  	sprintf(txt, "Variable : Na");
  else if(plot.Variable == Nd)
  	sprintf(txt, "Variable : Nd");
  else if(plot.Variable == ElecDensity)
  	sprintf(txt, "Variable : ElecDensity");
  else if(plot.Variable == HoleDensity)
	sprintf(txt, "Variable : HoleDensity");
  else if(plot.Variable == Temperature)
  	sprintf(txt, "Variable : Temperature");
  else if(plot.Variable == EFieldX)
  	sprintf(txt, "Variable : EFieldX");
  else if(plot.Variable == EFieldY)
  	sprintf(txt, "Variable : EFieldY");
  GRText(ix + x1, y1 += iy, txt);
  sprintf(txt, "AZ = %3.3e, EL = %3.3e", plot.az, plot.el);
  GRText(ix + x1, y1 += iy, txt);
}

/*-----------------------------------------------------------------------
 * Function:	 BSolver::plot_data
 * Scope:	 Public
 * Description:	 This function draws 3D data and mesh on the screen
 */
void BSolver::plot_data(PlotDefine &plot)
{
  int MAXx,MAXy;
  AINODE *aainode,*painode;
  PNT3D *apnt3dInput;

  int nodes=0,triangles=0;
  for(int z=0;z<zone_num;z++)
  {
  	nodes += zone[z].danode.size();
  	triangles += zone[z].datri.size();
  }

  aainode     = new AINODE[triangles];
  apnt3dInput = new PNT3D[nodes];
  painode=aainode;

  double (*Measure) (double);
  if(plot.Measure==Linear)
  	Measure = FunLinear;
  if(plot.Measure==SignedLog)
  	Measure = FunSignedLog10;

  for(int z=0;z<zone_num;z++)
  {
    for(int i=0;i<zone[z].datri.size();i++)
    {
  	(*painode)[0] = zone[z].datri[i].A + zone[z].offset;
   	(*painode)[1] = zone[z].datri[i].B + zone[z].offset;
    	(*painode)[2] = zone[z].datri[i].C + zone[z].offset;
    	if(zone[z].datri[i].D!=-1)
    		(*painode)[3] = zone[z].datri[i].D + zone[z].offset;
    	else
    		(*painode)[3] = zone[z].datri[i].D;
    	painode++;
    }
    for(int j=0;j<zone[z].danode.size();j++)
    {
  	apnt3dInput[j+zone[z].offset].x = zone[z].danode[j].x;
        apnt3dInput[j+zone[z].offset].y = zone[z].danode[j].y;
        if(plot.Variable==Nd)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->fs[j].Nd*pow(mesh_scale,3));
        if(plot.Variable==Na)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->fs[j].Na*pow(mesh_scale,3));
        if(plot.Variable==Potential)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->fs[j].P);
        if(plot.Variable==ElecDensity)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->fs[j].n*pow(mesh_scale,3));
        if(plot.Variable==HoleDensity)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->fs[j].p*pow(mesh_scale,3));
        if(plot.Variable==EFieldX)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->aux[j].Ex*mesh_scale);
        if(plot.Variable==EFieldY)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->aux[j].Ey*mesh_scale);
        if(plot.Variable==Temperature)
        	apnt3dInput[j+zone[z].offset].z = Measure(zonedata[z]->fs[j].T*3e2);

    }
  }

  if(plot.Resolution == RES_640x480)
  	{MAXx = 640; MAXy = 480;}
  else if(plot.Resolution == RES_1024x768)
  	{MAXx = 1024; MAXy = 768;}
  else if(plot.Resolution == RES_800x600)
  	{MAXx = 800; MAXy = 600;}

  triplot = new TriPlot3D(triangles, nodes,
                          aainode, apnt3dInput,
                          plot.az, plot.el,
                          plot.Resolution, plot.Style,
                          50, -1, 0);
  GROpenGraphWin("Rectangular/Triangular Surface Plot", "Plot", 0, 0, MAXx, MAXy, NULL);

  if (plot.GeneratePSFile)
  {
    triplot->nResFactor = 20;
    triplot->cx = 468 * triplot->nResFactor;
    triplot->cy = 468 * triplot->nResFactor;
    triplot->cxLBorder = 72  * triplot->nResFactor;
    triplot->cyLBorder = 162 * triplot->nResFactor;
    triplot->Xform2DData();
    GRPrintOpen(plot.PSFileName, triplot->nResFactor);
    Redraw();
    TitleEtc(plot, 0, 15, triplot->nResFactor);
    GRPrintClose();
  }
  else
  {
  	int fDone = FALSE;
  	short int keycode;
  	int arg1, arg2;
  	while (!fDone)
  	{ switch(getevent(&keycode, &arg1, &arg2))
    	  { case EXPOSE:
                 Redraw();
                 TitleEtc(plot,0, 5, 1);
                 flushdisplay();
            break;

            case CONFIGURENOTIFY:
            break;

            case BUTTONPRESS:
            case KEYPRESS:
                 fDone = TRUE;
            break;
          }
        }
  }

  delete triplot;
  delete [] apnt3dInput;
  delete [] aainode;

}


///////////////////////////////////////////////////////////////////////////////


/*-----------------------------------------------------------------------
 * Function:	 MESH::FillPoly
 * Scope:	 Private
 * Description:	 This function draws and fills a polygon.  The polygon is
 *		 limited to 32 sides
 */
void MESH::FillPoly (int c, double *adnode, int nFColor, int nBColor)
{
  	// return if the number of sides is greater than 32
	if (c > 32) return;

  	// draw and fill the polygon
  	GRSetColor(nFColor);
  	GRSetFillColor(nBColor);
  	int an[64];
  	for(int i = 0; i < c; ++i)
  	{
    		an[2*i]   = xMAP(adnode[2*i]);
    		an[2*i+1] = yMAP(adnode[2*i+1]);
  	}
  	GRFillPoly(c, an);
}



/*-----------------------------------------------------------------------
 * Function:	 MESH::RedrawMesh
 * Scope:	 Private
 * Description:	 This function redraw the screen.
 */
void MESH::RedrawMesh(void *dummy)
{
  PlotDefine *plot = (PlotDefine *)dummy;
  // initialize local variables
  int cte = 0;
  int cre = 0;
  int cObtuse = 0;
  int x,y,d=20;
  char szGlobalBuf[256];
  double adnode[8];

  switch (plot->Resolution)
  {
    case RES_640x480:	x = 480;  y = 10;  break;
    case RES_800x600:	x = 600;  y = 10;  break;
    case RES_1024x768:	x = 768;  y = 10;  break;
  }
  //draw triangle from zone level
  for(int i = 0; i < zone_num; i++)
    for(int itri = 0; itri < zone[i].datri.size(); itri++)
   {
    Tri ptri = zone[i].datri[itri];
    adnode[0] = zone[i].danode[ptri.A].x;
    adnode[1] = zone[i].danode[ptri.A].y;
    adnode[2] = zone[i].danode[ptri.B].x;
    adnode[3] = zone[i].danode[ptri.B].y;
    adnode[4] = zone[i].danode[ptri.C].x;
    adnode[5] = zone[i].danode[ptri.C].y;

    int Fcolor;
    if(i%5==0)  Fcolor= GR_RED;
    if(i%5==1)  Fcolor= GR_BLUE;
    if(i%5==2)  Fcolor= GR_GREEN;
    if(i%5==3)  Fcolor= GR_YELLOW;
    if(i%5==4)  Fcolor= GR_CYAN;

    if (ptri.D == -1)// triangular element
    {
      FillPoly(3, adnode, Fcolor, GR_BLACK);
    }
    else //  rectangular element
    {
      adnode[6] = zone[i].danode[ptri.D].x;
      adnode[7] = zone[i].danode[ptri.D].y;
      FillPoly(4, adnode, Fcolor, GR_BLACK);
    }

  }


  //draw Voronoi Cell
  for(int i = 0; i < zone_num; i++)
  {
    GRSetColor(GR_WHITE);
    GRSetFillColor(GR_BLACK);
    for(int j = 0; j < zone[i].davedge.size(); j++)
    {
      	VEdge pEdge = zone[i].davedge[j];
    	int sx1=xMAP(pEdge.x1);
    	int sy1=yMAP(pEdge.y1);
    	int sx2=xMAP(pEdge.x2);
   	int sy2=yMAP(pEdge.y2);
       	GRLine(sx1,sy1,sx2,sy2);
    }
    GRSetColor(GR_WHITE);
    sprintf(szGlobalBuf, "zone %d lable : %s ",i+1,zone[i].zonename);
    GRText(x+10, y+=d,    szGlobalBuf);
    sprintf(szGlobalBuf, "   %d nodes",int(zone[i].danode.size()));
    GRText(x+10, y+=d,    szGlobalBuf);
    sprintf(szGlobalBuf, "   %d elements",int(zone[i].datri.size()));
    GRText(x+10, y+=d, szGlobalBuf);
    sprintf(szGlobalBuf, "   %d voronoi edges",int(zone[i].davedge.size()));
    GRText(x+10, y+=d, szGlobalBuf);
  }


  //draw segment from zone level
  GRSetColor(GR_BROWN);
  for(int i = 0; i < zone_num; i++)
    for(int j=0;j<zone[i].dasegment.size();j++)
  	for(int k=0;k<zone[i].dasegment[j].edge_num;k++)
  	{
  		int p1=zone[i].dasegment[j].edge_array[2*k];
  		int p2=zone[i].dasegment[j].edge_array[2*k+1];
  		int sx1=xMAP(zone[i].danode[p1].x);
    		int sy1=yMAP(zone[i].danode[p1].y);
    		int sx2=xMAP(zone[i].danode[p2].x);
   		int sy2=yMAP(zone[i].danode[p2].y);
    		GRLine(sx1,sy1,sx2,sy2);
    	}

  GRSetColor(GR_WHITE);
  sprintf(szGlobalBuf, "click to continue");
  GRText(x+10, y+40, szGlobalBuf);

}

/*-----------------------------------------------------------------------
 * Function:	 MESH::draw_mesh_screen
 * Scope:	 public
 * Description:	 This function draw the mesh to screen
 */
void MESH::draw_mesh_screen(void *dummy)
{
	PlotDefine *plot = (PlotDefine *)dummy;
	int fDone = 0;
  	int nx,ny;
  	if(plot->Resolution == RES_640x480)
  		{nx = 640; ny = 480;}
  	else if(plot->Resolution == RES_1024x768)
  		{nx = 1024; ny = 768;}
  	else if(plot->Resolution == RES_800x600)
  		{nx = 800; ny = 600;}

 	short int keycode;
  	int arg1, arg2;
  	xDelta = xMax - xMin;
  	yDelta = yMax - yMin;

  	if (xDelta > yDelta) yDelta = xDelta;
    	else		 xDelta = yDelta;
   	cxBorder = 10;
  	cyBorder = 10;
  	cx = ny - 2*cxBorder - 23;
  	cy = ny - 2*cyBorder - 23;
  	GRInitGraphics();
 	GROpenGraphWin("Mesh Topology", "Mesh", -1, -1, nx, ny, NULL);

  	while (!fDone)
  	{ switch(getevent(&keycode, &arg1, &arg2))
    		{ case EXPOSE:
        		RedrawMesh(plot);
        		flushdisplay();
        		break;

      		  case CONFIGURENOTIFY:
        		break;

      		  case BUTTONPRESS:
      		  case KEYPRESS:
        		fDone = 1;
        	  break;
    		}
  	}
  	GRFreeGraphics();
}


/*-----------------------------------------------------------------------
 * Function:	 MESH::draw_mesh_ps
 * Scope:	 public
 * Description:	 This function generates a postscript file.
 */
void MESH::draw_mesh_ps(char *psfile)
{
  cx = 9360;
  cy = 9360;
  cxBorder = 1440;
  cyBorder = 3240;
  double adnode[8];
  GRPrintOpen(psfile,20);
  GRSetColor(GR_BLACK);

  //draw triangle from zone level
  for(int i = 0; i < zone_num; i++)
    for(int itri = 0; itri < zone[i].datri.size(); itri++)
   {
    Tri ptri = zone[i].datri[itri];
    adnode[0] = zone[i].danode[ptri.A].x;
    adnode[1] = zone[i].danode[ptri.A].y;
    adnode[2] = zone[i].danode[ptri.B].x;
    adnode[3] = zone[i].danode[ptri.B].y;
    adnode[4] = zone[i].danode[ptri.C].x;
    adnode[5] = zone[i].danode[ptri.C].y;

    int Fcolor;
    if(i%4==0)  Fcolor= GR_RED;
    if(i%4==1)  Fcolor= GR_CYAN;
    if(i%4==2)  Fcolor= GR_GREEN;
    if(i%4==3)  Fcolor= GR_YELLOW;

    if (ptri.D == -1)// triangular element
    {
      FillPoly(3, adnode, Fcolor, GR_BLACK);
    }
    else //  rectangular element
    {
      adnode[6] = zone[i].danode[ptri.D].x;
      adnode[7] = zone[i].danode[ptri.D].y;
      FillPoly(4, adnode, Fcolor, GR_BLACK);
    }

  }
  //draw Voronoi Cell
  for(int i = 0; i < zone_num; i++)
  {
    GRSetColor(GR_BLUE);
    GRSetFillColor(GR_BLACK);
    for(int j = 0; j < zone[i].davedge.size(); j++)
    {
      	VEdge pEdge = zone[i].davedge[j];
    	int sx1=xMAP(pEdge.x1);
    	int sy1=yMAP(pEdge.y1);
    	int sx2=xMAP(pEdge.x2);
   	int sy2=yMAP(pEdge.y2);
       	GRLine(sx1,sy1,sx2,sy2);
    }

  }

 //draw segment from zone level
  GRSetColor(GR_BROWN);
  for(int i = 0; i < zone_num; i++)
    for(int j=0;j<zone[i].dasegment.size();j++)
  	for(int k=0;k<zone[i].dasegment[j].edge_num;k++)
  	{
  		int p1=zone[i].dasegment[j].edge_array[2*k];
  		int p2=zone[i].dasegment[j].edge_array[2*k+1];
  		int sx1=xMAP(zone[i].danode[p1].x);
    		int sy1=yMAP(zone[i].danode[p1].y);
    		int sx2=xMAP(zone[i].danode[p2].x);
   		int sy2=yMAP(zone[i].danode[p2].y);
    		GRLine(sx1,sy1,sx2,sy2);
    	}
  GRPrintClose();
}
