#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cgnslib.h>
#include <math.h>
#include <iostream>
#include "solver.h"
#include "geom.h"


#define REAL double

struct triangulateio {
  REAL *pointlist;                                               /* In / out */
  REAL *pointattributelist;                                      /* In / out */
  int *pointmarkerlist;                                          /* In / out */
  int numberofpoints;                                            /* In / out */
  int numberofpointattributes;                                   /* In / out */

  int *trianglelist;                                             /* In / out */
  REAL *triangleattributelist;                                   /* In / out */
  REAL *trianglearealist;                                         /* In only */
  int *neighborlist;                                             /* Out only */
  int numberoftriangles;                                         /* In / out */
  int numberofcorners;                                           /* In / out */
  int numberoftriangleattributes;                                /* In / out */

  int *segmentlist;                                              /* In / out */
  int *segmentmarkerlist;                                        /* In / out */
  int numberofsegments;                                          /* In / out */

  REAL *holelist;                        /* In / pointer to array copied out */
  int numberofholes;                                      /* In / copied out */

  REAL *regionlist;                      /* In / pointer to array copied out */
  int numberofregions;                                    /* In / copied out */

  int *edgelist;                                                 /* Out only */
  int *edgemarkerlist;            /* Not used with Voronoi diagram; out only */
  REAL *normlist;                /* Used only with Voronoi diagram; out only */
  int numberofedges;                                             /* Out only */
};

extern "C"
{
void triangulate(char *, struct triangulateio *, struct triangulateio *,
                 struct triangulateio *);
}

inline double FunLinear(double value)
{
	return value;
}

inline double FunSignedLog(double value)
{
	return dsign(value)*log(1+fabs(value));
}

/* ----------------------------------------------------------------------------
 * BSolver::set_scale  This function compute the refine scale
 */
double  BSolver::set_scale(int A,int B,int C,RefineDefine & ref)
{
	double M1,M2,M3,M,Scale;
	int z = gnode[A].zone_index;
	//a function pointer
  	double (*Measure) (double);
  	if(ref.Measure==Linear)
  		Measure = FunLinear;
  	if(ref.Measure==SignedLog)
  		Measure = FunSignedLog;

	if(ref.Variable ==Potential)
  	{
  		M1 = Measure(zonedata[z]->fs[gnode[A].index].P);
  	 	M2 = Measure(zonedata[z]->fs[gnode[B].index].P);
  	 	M3 = Measure(zonedata[z]->fs[gnode[C].index].P);
  	}
  	if(ref.Variable ==Doping)
  	{
  	 	M1 = Measure(zonedata[z]->fs[gnode[A].index].Na*pow(mesh_scale,3)-
  	 		     zonedata[z]->fs[gnode[A].index].Nd*pow(mesh_scale,3));
  	 	M2 = Measure(zonedata[z]->fs[gnode[B].index].Na*pow(mesh_scale,3)-
  	 		     zonedata[z]->fs[gnode[B].index].Nd*pow(mesh_scale,3));
  	 	M3 = Measure(zonedata[z]->fs[gnode[C].index].Na*pow(mesh_scale,3)-
  	 		     zonedata[z]->fs[gnode[C].index].Nd*pow(mesh_scale,3));
  	}
  	M =  max(max(fabs(M1-M2),fabs(M2-M3)),fabs(M3-M1))+1e-6;
  	if(M>ref.Dispersion)
  		Scale = ref.Dispersion/M;
  	else
  		Scale = 1+1e-6;
  	if(Scale<0.75) Scale = 0.75;
  	return Scale;
}

/* ----------------------------------------------------------------------------
 * BSolver::refine  This function call triangulate to refine the mesh
 *
 */
int BSolver::refine(RefineDefine & ref)
{
  struct triangulateio in, out;

  zone_to_field();
  //save old mesh
  gnode_old = gnode;
  gtri_old = gtri;
  gsegment_old = gsegment;
   //a function pointer
  	double (*Measure) (double);
  	if(ref.Measure==Linear)
  		Measure = FunLinear;
  	if(ref.Measure==SignedLog)
  		Measure = FunSignedLog;

  //do necessarily prepare for call triangulate
  in.numberofpoints = gnode.size();
  in.numberofpointattributes = 2; //Nd and Na

  in.pointattributelist = (REAL *)malloc(in.numberofpoints * in.numberofpointattributes * sizeof(REAL));
  in.pointlist = (REAL *) malloc(in.numberofpoints * 2 * sizeof(REAL));
  in.pointmarkerlist = (int *) malloc(in.numberofpoints * sizeof(int));
  for(int i=0;i<in.numberofpoints;i++)
  {
  	  in.pointlist[2*i] = gnode[i].x;
  	  in.pointlist[2*i+1] = gnode[i].y;
  	  in.pointmarkerlist[i] = gnode[i].bc_index;
  }
  REAL *ppointattributelist = in.pointattributelist;
  for(int i=0;i<in.numberofpoints;i++)
  {
  	int z = gnode[i].zone_index;
  	int n = gnode[i].index;
  	 //*ppointattributelist++ = log(1.0+zonedata[z]->fs[n].Nd*pow(mesh_scale,3));
  	 //*ppointattributelist++ = log(1.0+zonedata[z]->fs[n].Na*pow(mesh_scale,3));
  	 *ppointattributelist++ = zonedata[z]->fs[n].Nd*pow(mesh_scale,3);
  	 *ppointattributelist++ = zonedata[z]->fs[n].Na*pow(mesh_scale,3);
  }

  in.numberoftriangles = gtri.size();
  for(int i=0;i<gtri.size();i++)
  	if(gtri[i].D!=-1)  in.numberoftriangles++;
  in.numberofcorners = 3;
  in.numberoftriangleattributes = 1;
  in.trianglelist =  (int *) malloc(in.numberoftriangles * 3 * sizeof(int));
  in.trianglearealist = (REAL *) malloc(in.numberoftriangles * sizeof(REAL));
  in.triangleattributelist = (REAL *) malloc(in.numberoftriangles * sizeof(REAL));
  int *ptrianglelist = in.trianglelist;
  REAL *ptrianglearealist = in.trianglearealist;
  REAL *ptriangleattributelist = in.triangleattributelist;
  for(int i=0;i<gtri.size();i++)
  {
  	  if(gtri[i].D!=-1)
  	  {
  	  	*ptrianglelist++ = gtri[i].A;
  	  	*ptrianglelist++ = gtri[i].B;
  	  	*ptrianglelist++ = gtri[i].D;
  	  	*ptriangleattributelist++ = gtri[i].zone_index;

  	  	*ptrianglelist++ = gtri[i].B;
  	  	*ptrianglelist++ = gtri[i].C;
  	  	*ptrianglelist++ = gtri[i].D;
  	    	*ptriangleattributelist++ = gtri[i].zone_index;
  	  }
  	  else
  	  {
  	  	*ptrianglelist++ = gtri[i].A;
  	  	*ptrianglelist++ = gtri[i].B;
  	  	*ptrianglelist++ = gtri[i].C;
  	  	*ptriangleattributelist++ = gtri[i].zone_index;
  	  }

  }
  for(int i=0;i<gtri.size();i++)
  {
  	 double Scale;

  	 if(gtri[i].D!=-1)
  	 {
  	  	Scale = set_scale(gtri[i].A,gtri[i].B,gtri[i].D,ref);
  	 	*ptrianglearealist++ = gtri[i].area/2*Scale*Scale;

  	 	Scale = set_scale(gtri[i].B,gtri[i].C,gtri[i].D,ref);
  	 	*ptrianglearealist++ = gtri[i].area/2*Scale*Scale;
  	 }
  	 else
  	 {
  	 	Scale = set_scale(gtri[i].A,gtri[i].B,gtri[i].C,ref);
  	 	*ptrianglearealist++ = gtri[i].area*Scale*Scale;

  	 }
  }
  in.numberofsegments =  0;
  for(int i=0;i<gsegment.size();i++)
  	 in.numberofsegments+=gsegment[i].edge_num;
  in.segmentlist =  (int *) malloc(in.numberofsegments * 2 * sizeof(int));
  in.segmentmarkerlist = (int *) malloc(in.numberofsegments * sizeof(int));
  int *psegmentlist =  in.segmentlist;
  int *psegmentmarkerlist = in.segmentmarkerlist;
  for(int i=0;i<gsegment.size();i++)
    for(int j=0;j<gsegment[i].edge_num;j++)
    {
    	 *psegmentlist++ = gsegment[i].edge_array[2*j];
    	 *psegmentlist++ = gsegment[i].edge_array[2*j+1];
         *psegmentmarkerlist++ = gsegment[i].bc_index;
    }

  in.numberofholes = 0;
  in.numberofregions = 0;


  // Make necessary initializations so that Triangle can return a
  // triangulation in `out'.

  out.pointlist = (REAL *) NULL;
  out.pointattributelist = (REAL *) NULL;
  out.pointmarkerlist = (int *) NULL;
  out.trianglelist = (int *) NULL;
  out.triangleattributelist = (REAL *) NULL;
  out.segmentlist = (int *) NULL;
  out.segmentmarkerlist = (int *) NULL;

  // Refine the triangulation according to the attached
  // triangle area constraints.

  triangulate("praq30zCVL", &in, &out, (struct triangulateio *) NULL);

  //printf("Refined triangulation:\n\n");
  //report(&out, 1, 1, 0, 1, 0, 0);
  //Export file to simple cgns format
  //tri_to_cgns("out.cgns",&out,'z');

  gnode.resize(out.numberofpoints);
  gtri.resize(out.numberoftriangles);
  for(int i=0; i<gsegment.size(); i++)
  {
  	gsegment[i].edge_array.clear();
  	gsegment[i].edge_num = 0;
  }

  ppointattributelist = out.pointattributelist;
  for(int i=0;i<out.numberofpoints;i++)
  {
  	  //Node tmpnode;
  	  gnode[i].x = out.pointlist[2*i];
  	  gnode[i].y = out.pointlist[2*i+1];
  	  gnode[i].bc_index = out.pointmarkerlist[i];

  }

  ptrianglelist = out.trianglelist;
  for(int i=0;i<out.numberoftriangles;i++)
  {
  	gtri[i].zone_index = static_cast<int>( out.triangleattributelist[i]+0.5 );
  	//gtri[i].zone =(int)out.triangleattributelist[i]+0.5;
    	gtri[i].A = *ptrianglelist++;
    	gtri[i].B = *ptrianglelist++;
    	gtri[i].C = *ptrianglelist++;
    	gtri[i].D = -1;
    	gtri[i].area = TriArea(gnode[gtri[i].A].x,gnode[gtri[i].A].y,
    			       gnode[gtri[i].B].x,gnode[gtri[i].B].y,
    			       gnode[gtri[i].C].x,gnode[gtri[i].C].y);
  }

  for(int j=0;j<gsegment.size();j++)
   for(int i=0;i<out.numberofsegments;i++)
      if(out.segmentmarkerlist[i] == gsegment[j].bc_index)
      {
      	 gsegment[j].edge_array.push_back(out.segmentlist[2*i]);
    	 gsegment[j].edge_array.push_back(out.segmentlist[2*i+1]);
    	 gsegment[j].edge_num++;
      }

  field_to_zone();
  build_zone();
  //

  clear_data();
  bc.clear();
  if(setup_bc(cmdlist))   return 1;
  if(build_zonedata())    return 1;

  //re-arrange data
  ppointattributelist = out.pointattributelist;
  for(int i=0;i<out.numberofpoints;i++)
  {
  	int z = gnode[i].zone_index;
  	int n = gnode[i].index;
  	zonedata[z]->fs[n].Nd = fabs((*ppointattributelist++))/pow(mesh_scale,3);
    	zonedata[z]->fs[n].Na = fabs((*ppointattributelist++))/pow(mesh_scale,3);

  }
  setup_init_data(NULL,LatticeTemp);
  // Free all allocated arrays, including those allocated by Triangle.

  free(in.pointlist);
  free(in.pointmarkerlist);
  free(in.pointattributelist);
  free(in.trianglelist);
  free(in.triangleattributelist);
  free(in.trianglearealist);
  free(in.segmentlist);
  free(in.segmentmarkerlist);

  free(out.pointlist);
  free(out.pointmarkerlist);
  free(out.pointattributelist);
  free(out.trianglelist);
  free(out.triangleattributelist);
  free(out.segmentlist);
  free(out.segmentmarkerlist);

  return 0;
}
