#include "solver.h"
#include <cgnslib.h>

int BSolver::read_mesh(char* cgnsfile)
{
	//init mesh first
	set_mesh_scale(1e4); //scale cm to 1um
	if(import_cgns(cgnsfile)) return 1;
	build_zone();
	//zone_to_field();
	return 0;
}

int BSolver::build_zonedata()
{
	zonedata = new BZoneData*[zone_num];
	// determine zone material by zone label. the label is compatible with medici
	for(int i=0; i<zone_num; i++)
	{ //it is an Insulator zone
	  if(!strcmp(zone[i].zonelabel,"Ox")||!strcmp(zone[i].zonelabel,"SiO2"))
	  {
	  	zonedata[i] = new ISZone;
	  	ISZone *pzonedata = dynamic_cast< ISZone * >(zonedata[i]);
	  	pzonedata->pzone = &zone[i];
	  	pzonedata->zone_index = zone[i].zone_index;
		pzonedata->pzone = &zone[i];
		pzonedata->node_num = zone[i].davcell.size();
       		pzonedata->material_type = Insulator;
       		pzonedata->material = SiO2;
       		pzonedata->mt = new Material(SiO2,mesh_scale);
        	pzonedata->fs = new SimData[pzonedata->node_num];
        	pzonedata->aux = new SimAuxData[pzonedata->node_num];
	  	pzonedata->report();
	  }
	  //if this is a semiconductor zone
	  else if(!strcmp(zone[i].zonelabel,"Si")||
	          !strcmp(zone[i].zonelabel,"Silicon")||
	          !strcmp(zone[i].zonelabel,"GaAs")||
	          !strcmp(zone[i].zonelabel,"Ge") )
	  {
	  	zonedata[i] = new SMCZone;
	  	SMCZone *pzonedata = dynamic_cast< SMCZone * >(zonedata[i]);
	  	pzonedata->pzone = &zone[i];
                pzonedata->zone_index = zone[i].zone_index;
		if(!strcmp(zone[i].zonelabel,"Si")||!strcmp(zone[i].zonelabel,"Silicon"))
			pzonedata->material = Si;
		if(!strcmp(zone[i].zonelabel,"GaAs"))
			pzonedata->material = GaAs;
		if(!strcmp(zone[i].zonelabel,"Ge"))
			pzonedata->material = Ge;
		pzonedata->material_type = semiconductor;
		pzonedata->mt = new Material(pzonedata->material,mesh_scale);
		pzonedata->node_num = zone[i].davcell.size();
		pzonedata->fs  = new SimData[pzonedata->node_num];
		pzonedata->aux = new SimAuxData[pzonedata->node_num];
		for(int j=0;j<pzonedata->node_num;j++)
		{
			int nb_num = zone[i].davcell[j].nb_num;
			pzonedata->aux[j].Qp_r = new vector_double[nb_num];
			pzonedata->aux[j].Qn_r = new vector_double[nb_num];
			pzonedata->aux[j].Qp_s = new vector_double[nb_num];
			pzonedata->aux[j].Qn_s = new vector_double[nb_num];
		}
		pzonedata->electrode.clear();
		for(int j=0;j<bc.size();j++)
		{
			if(bc[j].psegment->zone_index==pzonedata->zone_index&&
			   (bc[j].BCType==OhmicContract||bc[j].BCType==SchottkyContract))
			pzonedata->electrode.push_back(j);
		}
		pzonedata->report();
	  }
	  //oh it is an electrode,medici can generate this region
	  else if(!strcmp(zone[i].zonelabel,"Elec"))
	  {
	  	zonedata[i] = new ElZone;
	  	ElZone *pzonedata = dynamic_cast< ElZone * >(zonedata[i]);
	  	pzonedata->pzone = &zone[i];
	  	pzonedata->zone_index = zone[i].zone_index;
		pzonedata->node_num = zone[i].davcell.size();
		pzonedata->material_type = metal;
       		pzonedata->material = Al;
       		pzonedata->fs = new SimData[pzonedata->node_num];
       		pzonedata->aux = new SimAuxData[pzonedata->node_num];
	  	pzonedata->report();
	  }
	  else
	  {
	  	printf("error, no region type specified\n");
	  	return 1;
	  }
	}
	return 0;
}

int  BSolver::setup_init_data(char* cgnsfile,double Tl)
{
	DeviceDepth = DeviceDepth*mesh_scale;
	for(int i=0; i<zone_num; i++)
	        if(zonedata[i]->Init(&zone[i],cgnsfile,Tl/300.0,mesh_scale)) return 1;
	return 0;

}

int  BSolver::setup_bc(list<CmdParseBuf>&)
{
	//the bc init here
	Segment **segment = new Segment*[bc_num];
	for(int i=1; i<=bc_num;i++)
		segment[i-1] = Get_segment(i);
	if(bc.InitBC(bc_num,segment,cmdlist,mesh_scale)) return 1;
	delete 	[] segment;
	return 0;
}

void BSolver::reorder()
{
	for(int z=0; z<zone_num; z++)
	{
	  int * old_to_new = reorder_zone(z);
	  SimData* fs_new = new SimData[zonedata[z]->node_num];

	  for(int i=0;i<zonedata[z]->node_num;i++)
	    fs_new[old_to_new[i]] = zonedata[z]->fs[i];
	  delete [] zonedata[z]->fs;
	  zonedata[z]->fs = fs_new;
	  //reorder aux data
	  //note: when copy fs ,the pointer is also copied
	  SimAuxData* aux_new = new SimAuxData[zonedata[z]->node_num];
	  for(int i=0;i<zonedata[z]->node_num;i++)
	  	  aux_new[old_to_new[i]] = zonedata[z]->aux[i];
	  delete [] zonedata[z]->aux;
	  zonedata[z]->aux = aux_new;

	  delete [] old_to_new;
	}
	build_zone();
}

int  ISZone::Init(ZONE* zone, char* cgnsfile, double Tl,double mesh_scale)
{
        for(int i=0;i<node_num; i++)
       	{
       	 	fs[i].T =  Tl;
       	 	fs[i].P =  0;
       	 	fs[i].n =  0;
       	 	fs[i].p =  0;
       	 	fs[i].Nd = 0;
       	 	fs[i].Na = 0;
       	 	Set_Vector_zero(fs[i].Qn);
       	 	Set_Vector_zero(fs[i].Qp);
       	}
	return 0;
}

int  ElZone::Init(ZONE* zone, char* cgnsfile, double Tl,double mesh_scale)
{
       	for(int i=0;i<node_num; i++)
       	{
       	 	fs[i].T =  Tl;
       	 	fs[i].P =  0;
       	 	fs[i].n =  0;
       	 	fs[i].p =  0;
       	 	fs[i].Nd = 0;
       	 	fs[i].Na = 0;
       	 	Set_Vector_zero(fs[i].Qn);
       	 	Set_Vector_zero(fs[i].Qp);
       	}

       	return 0;
}


int  SMCZone::Init(ZONE* zone, char* cgnsfile, double Tl,double mesh_scale)
{
     	if(cgnsfile!=NULL)  if(import_doping(cgnsfile,mesh_scale)) return 1;
    	for(int i=0;i<node_num; i++)
	{
	       	 fs[i].T =  Tl;
	       	 aux[i].Ex = aux[i].Ey =0;
	    	 double nie = mt->ni(fs[i].T);
	      	 if((fs[i].Nd-fs[i].Na)>0)   //n-type
		 {
	  		fs[i].n=fs[i].Qn.v[0]=((fs[i].Nd-fs[i].Na)+sqrt(pow(fs[i].Nd-fs[i].Na,2)+4*nie*nie))/2;
			fs[i].p=fs[i].Qp.v[0]=nie*nie/fs[i].n;
	 	 }
	 	 else		       //p-type
	 	 {
	 		fs[i].p=fs[i].Qp.v[0]=((fs[i].Na-fs[i].Nd)+sqrt(pow(fs[i].Na-fs[i].Nd,2)+4*nie*nie))/2;
	 		fs[i].n=fs[i].Qn.v[0]=nie*nie/fs[i].p;
	 	 }
	 	 fs[i].Qp.v[1]=0;
	 	 fs[i].Qp.v[2]=0;
	 	 fs[i].Qp.v[3]=3.0/2.0*fs[i].Qp.v[0]*mt->k*fs[i].T;
	 	 fs[i].Qn.v[1]=0;
	 	 fs[i].Qn.v[2]=0;
	 	 fs[i].Qn.v[3]=3.0/2.0*fs[i].Qn.v[0]*mt->k*fs[i].T;
	 	 fs[i].P = mt->Vbi(fs[i].Na,fs[i].Nd,fs[i].T);

	}
       return 0;
}

void  SMCZone::sol_ddm_to_hdm()
{
	for(int i=0;i<node_num; i++)
	{
		fs[i].Qp.v[0]=fs[i].p;
	 	fs[i].Qp.v[1]=0;
	 	fs[i].Qp.v[2]=0;
	 	fs[i].Qp.v[3]=3.0/2.0*fs[i].Qp.v[0]*mt->k*fs[i].T;

	 	fs[i].Qn.v[0]=fs[i].n;
	 	fs[i].Qn.v[1]=0;
	 	fs[i].Qn.v[2]=0;
	 	fs[i].Qn.v[3]=3.0/2.0*fs[i].Qn.v[0]*mt->k*fs[i].T;
	}
}

void  SMCZone::sol_hdm_to_ddm()
{

}

